@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)ant.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)ant.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -39,16 +39,35 @@ static char sccsid[] = "@(#)ant.c	3.9 96/05/25 xlockmore";
   ?       Triangle (3, 9? or 12)     <- 3, 9 and 12 are not implemented
 */
 
-ModeSpecOpt ant_opts =
-{0, NULL, NULL, NULL};
+#define DEF_TRUCHET  "False"
 
-extern      neighbors;
-extern      gridsize;
+extern int  neighbors;
+extern int  gridsize;
+static Bool truchet;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-truchet", ".ant.truchet", XrmoptionNoArg, (caddr_t) "on"},
+	{"+truchet", ".ant.truchet", XrmoptionNoArg, (caddr_t) "off"}
+};
+static argtype vars[] =
+{
+	{(caddr_t *) & truchet, "truchet", "Truchet", DEF_TRUCHET, t_Bool}
+};
+static OptionStruct desc[] =
+{
+	{"-/+truchet", "turn on/off Truchet lines"}
+};
+
+ModeSpecOpt ant_opts =
+{2, opts, 1, vars, desc};
 
 /* If you change the table you may have to change the following 2 constants */
 #define STATES 2
 #define COLORS 11
+#define MINANTS 1
 #define PATTERNSIZE 8
+#define REDRAWSTEP 2000		/* How much tape to draw per cycle */
 static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
 {
 	{0x11, 0x22, 0x11, 0x22, 0x11, 0x22, 0x11, 0x22},	/* grey+white | stripe */
@@ -74,6 +93,7 @@ typedef struct {
 
 typedef struct {
 	int         init_bits;
+	int         neighbors;
 	int         generation;
 	int         xs, ys;
 	int         xb, yb;
@@ -81,23 +101,27 @@ typedef struct {
 	int         width, height;
 	unsigned char ncolors, nstates;
 	int         n;
+	int         redrawing, redrawpos;
+	int         truchet;	/* Only for Turk modes */
 	statestruct machine[COLORS * STATES];
 	unsigned char *tape;
+	unsigned char *truchet_state;
 	antstruct  *ants;
 	unsigned char colors[COLORS - 1];
 	GC          stippled_GC;
 	Pixmap      pixmaps[COLORS - 1];
-	XPoint      hexagonList[6];
-} anthillstruct;
+	XPoint      hexagonList[7];
+} antfarmstruct;
 
-static XPoint hexagonUnit[6] =
+static XPoint hexagonUnit[7] =
 {
 	{0, 0},
 	{1, 1},
 	{0, 2},
 	{-1, 1},
 	{-1, -1},
-	{0, -2}
+	{0, -2},
+	{1, -1}
 };
 
 #define N 0
@@ -117,7 +141,7 @@ static XPoint hexagonUnit[6] =
 #define HL 4
 #define L 5
 
-static anthillstruct *ants = NULL;
+static antfarmstruct *antfarms = NULL;
 
 /* LANGTON'S ANT (10) Chaotic after 500, Builder after 10,000 (104p) */
 /* TURK'S 100 ANT Always chaotic?, tested past 150,000,000 */
@@ -153,9 +177,9 @@ static unsigned char tables[][3 * COLORS * STATES + 2] =
 static void
 fillcell(ModeInfo * mi, int col, int row, int stipple)
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 
-	if (neighbors == 6) {
+	if (ap->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
 		ap->hexagonList[0].x = ap->xb + ccol * ap->xs;
@@ -170,17 +194,73 @@ fillcell(ModeInfo * mi, int col, int row, int stipple)
 	}
 }
 
+static void
+truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
+{
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
+
+	if (ap->neighbors == 6) {
+		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+		int         side;
+		XPoint      hex, hex2;
+
+		/* Very crude approx of Sqrt 3, so it will not cause drawing errors. */
+		hex.x = ap->xb + ccol * ap->xs - (int) ((double) ap->xs * 1.5 / 2.0);
+		hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys * 1.5 / 2.0);
+		for (side = 0; side < 6; side++) {
+			if (side > 0) {
+				hex.x += ap->hexagonList[side].x;
+				hex.y += ap->hexagonList[side].y;
+			}
+			hex2.x = hex.x + ap->hexagonList[side + 1].x / 2;
+			hex2.y = hex.y + ap->hexagonList[side + 1].y / 2;
+			if (side >= 1 && side < 4)	/* Fudge correction */
+				hex2.y += 1;
+			if (truchetstate == side % 3)
+				/* Crude approx of 120 deg, so it will not cause drawing errors. */
+				XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				  hex2.x, hex2.y,
+					(int) ((double) ap->xs * 1.5), (int) ((double) ap->ys * 1.5),
+				  ((555 - (side * 60)) % 360) * 64, 90 * 64);
+		}
+	} else {
+		if (truchetstate) {
+			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				 ap->xb + ap->xs * col - ap->xs / 2,
+				 ap->yb + ap->ys * row + ap->ys / 2,
+				 ap->xs, ap->ys,
+				 1 * 64, 88 * 64);
+			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				 ap->xb + ap->xs * col + ap->xs / 2,
+				 ap->yb + ap->ys * row - ap->ys / 2,
+				 ap->xs, ap->ys,
+				 -91 * 64, -88 * 64);
+		} else {
+			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				 ap->xb + ap->xs * col - ap->xs / 2,
+				 ap->yb + ap->ys * row - ap->ys / 2,
+				 ap->xs, ap->ys,
+				 -1 * 64, -88 * 64);
+			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+				 ap->xb + ap->xs * col + ap->xs / 2,
+				 ap->yb + ap->ys * row + ap->ys / 2,
+				 ap->xs, ap->ys,
+				 91 * 64, 88 * 64);
+		}
+	}
+}
+
 static void
 drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 
 	if (!color) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
 		fillcell(mi, col, row, 0);
 		return;
 	}
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
+	if (MI_NPIXELS(mi) > 2) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
 			       MI_PIXEL(mi, ap->colors[color - 1]));
 		fillcell(mi, col, row, 0);
@@ -196,6 +276,21 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 	}
 }
 
+static void
+drawtruchet(ModeInfo * mi, int col, int row,
+	    unsigned char color, unsigned char truchetstate)
+{
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
+
+	if (!color)
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+	else if (MI_NPIXELS(mi) > 2 || color > ap->ncolors / 2)
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+	else
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+	truchetcell(mi, col, row, truchetstate);
+}
+
 static void
 draw_anant(ModeInfo * mi, int col, int row)
 {
@@ -204,7 +299,7 @@ draw_anant(ModeInfo * mi, int col, int row)
 	fillcell(mi, col, row, 0);
 #if 0				/* Can not see eyes */
 	{
-		anthillstruct *ap = &ants[MI_SCREEN(mi)];
+		antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 		Display    *display = MI_DISPLAY(mi);
 		Window      window = MI_WINDOW(mi);
 
@@ -248,13 +343,12 @@ static void
 RandomSoup(mi)
 	ModeInfo   *mi;
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         row, col, mrow = 0;
 
 	for (row = 0; row < ap->nrows; ++row) {
 		for (col = 0; col < ap->ncols; ++col) {
-			ap->old[col + mrow] =
-				(unsigned char) LRAND() % ((unsigned char) ap->ncolors);
+			ap->old[col + mrow] = (unsigned char) NRAND((int) ap->ncolors);
 			drawcell(mi, col, row, ap->old[col + mrow]);
 		}
 		mrow += gridsize;
@@ -266,7 +360,7 @@ RandomSoup(mi)
 static void
 GetTable(ModeInfo * mi, int i)
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         j, total;
 	unsigned char *patptr;
 
@@ -282,20 +376,20 @@ GetTable(ModeInfo * mi, int i)
 		ap->machine[j].direction = *patptr++;
 		ap->machine[j].next = *patptr++;
 	}
+	ap->truchet = False;
 }
-
 static void
 GetTurk(ModeInfo * mi, int i)
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         power2, j, number, total;
 
-	/* To force a number, say <i = 2;> has  i + 2 or 4 digits in binary */
+	/* To force a number, say <i = 2;> has  i + 2 (or 4) digits in binary */
 	power2 = 1 << (i + 1);
 	/* Dont want numbers which in binary are all 1's. */
-	number = (LRAND() % (power2 - 1)) + power2;
-
+	number = NRAND(power2 - 1) + power2;
 	/* To force a particular number, say <number = 10;> */
+
 	ap->ncolors = i + 2;
 	ap->nstates = 1;
 	total = ap->ncolors * ap->nstates;
@@ -305,6 +399,8 @@ GetTurk(ModeInfo * mi, int i)
 		ap->machine[j].next = 0;
 		power2 >>= 1;
 	}
+	if (truchet)
+		ap->truchet = 1;
 	if (MI_WIN_IS_VERBOSE(mi))
 		(void) printf("Turk's number %d, colors %d\n", number, ap->ncolors);
 }
@@ -315,39 +411,54 @@ init_ant(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	XGCValues   gcv;
-	anthillstruct *ap;
+	antfarmstruct *ap;
 	int         col, row, i, dir;
 
-	if (ants == NULL) {
-		if ((ants = (anthillstruct *) calloc(MI_NUM_SCREENS(mi),
-					    sizeof (anthillstruct))) == NULL)
+	if (antfarms == NULL) {
+		if ((antfarms = (antfarmstruct *) calloc(MI_NUM_SCREENS(mi),
+					    sizeof (antfarmstruct))) == NULL)
 			return;
 	}
-	ap = &ants[MI_SCREEN(mi)];
-	if ((MI_WIN_IS_MONO(mi) || MI_NPIXELS(mi) <= 2) && (ap->init_bits == 0)) {
+	ap = &antfarms[MI_SCREEN(mi)];
+	if ((MI_NPIXELS(mi) <= 2) && (ap->init_bits == 0)) {
 		gcv.fill_style = FillOpaqueStippled;
 		ap->stippled_GC = XCreateGC(display, window, GCFillStyle, &gcv);
-		for (i = 0; i < COLORS; i++)
+		for (i = 0; i < COLORS - 1; i++)
 			BITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
 	}
 	ap->generation = 0;
 	ap->n = MI_BATCHCOUNT(mi);
-	if (ap->n < 1)
-		ap->n = 1;
+	if (ap->n < -MINANTS) {
+		/* if ap->n is random ... the size can change */
+		if (ap->ants != NULL) {
+			(void) free((void *) ap->ants);
+			ap->ants = NULL;
+		}
+		ap->n = NRAND(-ap->n - MINANTS + 1) + MINANTS;
+	} else if (ap->n < MINANTS)
+		ap->n = MINANTS;
 
 	ap->width = MI_WIN_WIDTH(mi);
 	ap->height = MI_WIN_HEIGHT(mi);
 
-	if (neighbors != 4 && neighbors != 6)
-		neighbors = 4;
+	if (neighbors == 4)
+		ap->neighbors = 4;
+	else if (neighbors == 6)
+		ap->neighbors = 6;
+	else
+		ap->neighbors = (LRAND() & 1) ? 4 : 6;
 	if (gridsize < 4)
 		gridsize = 4;
 
-	if (neighbors == 6) {
+	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
 
-		nccols = min(ap->width, 2 * (gridsize + 1));
-		ncrows = min(ap->height, 2 * (gridsize + 2));
+		if (ap->width < 2)
+			ap->width = 2;
+		if (ap->height < 4)
+			ap->height = 4;
+		nccols = MIN(ap->width, 2 * (gridsize + 1));
+		ncrows = MIN(ap->height, 2 * (gridsize + 2));
 		ap->ncols = nccols >> 1;
 		ap->nrows = (ncrows >> 2) << 1;
 		nccols = ap->ncols << 1;
@@ -362,42 +473,44 @@ init_ant(ModeInfo * mi)
 		ap->yb = (ap->height - ap->ys * ncrows) / 2 + ap->ys;
 		ap->ncols -= 1;
 		ap->nrows -= 2;
-		for (i = 0; i < 6; i++) {
+		for (i = 0; i < 7; i++) {
 			ap->hexagonList[i].x = (ap->xs - 1) * hexagonUnit[i].x;
 			ap->hexagonList[i].y = (ap->ys - 1) * hexagonUnit[i].y / 2;
 		}
 	} else {
-		ap->ncols = min(ap->width, gridsize);
-		ap->nrows = min(ap->height, gridsize);
+		ap->ncols = MIN(ap->width, gridsize);
+		ap->nrows = MIN(ap->height, gridsize);
 		ap->xs = ap->width / ap->ncols;
 		ap->ys = ap->height / ap->nrows;
 		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
 	}
-	XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
-	XFillRectangle(display, window, MI_GC(mi), 0, 0, ap->width, ap->height);
+	XClearWindow(display, MI_WINDOW(mi));
 
-	if (!(LRAND() % COLORS))
-		GetTable(mi, (int) (LRAND() % NTABLES));
+	if (!NRAND(COLORS))
+		GetTable(mi, (int) (NRAND(NTABLES)));
 	else
-		GetTurk(mi, (int) (LRAND() % (COLORS - 1)));
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
+		GetTurk(mi, (int) (NRAND(COLORS - 1)));
+	if (MI_NPIXELS(mi) > 2)
 		for (i = 0; i < (int) ap->ncolors; i++)
-			ap->colors[i] = (LRAND() % MI_NPIXELS(mi) +
+			ap->colors[i] = (NRAND(MI_NPIXELS(mi)) +
 			     i * MI_NPIXELS(mi)) / ((int) (ap->ncolors - 1));
-	if (ap->ants == NULL)
-		ap->ants = (antstruct *) malloc(ap->n * sizeof (antstruct));
-	if (ap->tape == NULL)
-		ap->tape = (unsigned char *)
-			calloc(gridsize * gridsize, sizeof (unsigned char));
+	if (ap->ants != NULL)
+		(void) free((char *) ap->ants);
+	ap->ants = (antstruct *) malloc(ap->n * sizeof (antstruct));
+	if (ap->tape != NULL)
+		(void) free((char *) ap->tape);
+	ap->tape = (unsigned char *)
+		calloc(ap->ncols * ap->nrows, sizeof (unsigned char));
 
-	else
-		(void) memset((char *) ap->tape, 0,
-			      gridsize * gridsize * sizeof (unsigned char));
+	if (ap->truchet_state != NULL)
+		(void) free((char *) ap->truchet_state);
+	ap->truchet_state = (unsigned char *)
+		calloc(ap->ncols * ap->nrows, sizeof (unsigned char));
 
 	col = ap->ncols / 2;
 	row = ap->nrows / 2;
-	dir = LRAND() % neighbors;
+	dir = NRAND(ap->neighbors);
 	/* Have them all start in the same spot, why not? */
 	for (i = 0; i < ap->n; i++) {
 		ap->ants[i].col = col;
@@ -411,11 +524,11 @@ init_ant(ModeInfo * mi)
 void
 draw_ant(ModeInfo * mi)
 {
-	anthillstruct *ap = &ants[MI_SCREEN(mi)];
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	antstruct  *anant;
 	statestruct *status;
 	int         i, state_pos, tape_pos;
-	unsigned char color, temp1, temp2;
+	unsigned char color, temp1, temp2, old_dir;
 
 	for (i = 0; i < ap->n; i++) {
 		anant = &ap->ants[i];
@@ -427,18 +540,35 @@ draw_ant(ModeInfo * mi)
 		ap->tape[tape_pos] = status->color;	/* write on tape */
 
 		/* Find direction of Bees or Ants. */
-		temp1 = (neighbors == 4 && status->direction > 1) ?
+		old_dir = anant->direction;
+		temp1 = (ap->neighbors == 4 && status->direction > 1) ?
 			(unsigned char) ((int) (status->direction + 1) / 2) : status->direction;
-		temp2 = (neighbors == 4 && anant->direction > 1) ?
+		temp2 = (ap->neighbors == 4 && anant->direction > 1) ?
 			anant->direction - 1 : anant->direction;
-		anant->direction = (unsigned char) ((int) (temp1 + temp2) % neighbors);
-		anant->direction = (neighbors == 4 && anant->direction > 1) ?
+		anant->direction = (unsigned char) ((int) (temp1 + temp2) % ap->neighbors);
+		anant->direction = (ap->neighbors == 4 && anant->direction > 1) ?
 			anant->direction + 1 : anant->direction;
+		if (ap->truchet) {
+			int         a, b;
 
+			if (ap->neighbors == 4) {
+				a = ((old_dir > 1) ? old_dir - 1 : old_dir) / 2;
+				b = ((anant->direction > 1) ?
+				anant->direction - 1 : anant->direction) / 2;
+				a = ((a && !b) || (b && !a));
+				drawtruchet(mi, anant->col, anant->row, status->color, a);
+			} else {
+				a = old_dir % 3;
+				b = anant->direction % 3;
+				a = 3 - (a + b);
+				drawtruchet(mi, anant->col, anant->row, status->color, a);
+			}
+			ap->truchet_state[tape_pos] = a;
+		}
 		anant->state = status->next;
 
 		/* If edge than wrap it */
-		if (neighbors == 6) {
+		if (ap->neighbors == 6) {
 			switch (anant->direction) {
 				case NE:
 					if (!(anant->row & 1))
@@ -489,37 +619,58 @@ draw_ant(ModeInfo * mi)
 		}
 		draw_anant(mi, anant->col, anant->row);
 	}
-
 	if (++ap->generation > MI_CYCLES(mi))
 		init_ant(mi);
+	if (ap->redrawing) {
+		int         i;
+
+		for (i = 0; i < REDRAWSTEP; i++) {
+			if (ap->tape[ap->redrawpos]) {
+				drawcell(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->nrows,
+					 ap->tape[ap->redrawpos]);
+				if (ap->truchet)
+					drawtruchet(mi, ap->redrawpos % ap->ncols, ap->redrawpos / ap->nrows,
+						    ap->tape[ap->redrawpos], ap->truchet_state[ap->redrawpos]);
+			}
+			if (++(ap->redrawpos) >= ap->ncols * ap->nrows) {
+				ap->redrawing = 0;
+				break;
+			}
+		}
+	}
 }
 
 void
 release_ant(ModeInfo * mi)
 {
-	if (ants != NULL) {
+	if (antfarms != NULL) {
 		int         screen, shade;
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			if (ants[screen].stippled_GC != NULL) {
-				XFreeGC(MI_DISPLAY(mi), ants[screen].stippled_GC);
-				ants[screen].stippled_GC = NULL;
+			antfarmstruct *ap = &antfarms[screen];
+
+			if (ap->stippled_GC != NULL) {
+				XFreeGC(MI_DISPLAY(mi), ap->stippled_GC);
 			}
-			if (ants[screen].init_bits != 0) {
+			if (ap->init_bits != 0) {
 				for (shade = 0; shade < COLORS - 1; shade++)
-					XFreePixmap(MI_DISPLAY(mi), ants[screen].pixmaps[shade]);
-				ants[screen].init_bits = 0;
-			}
-			if (ants[screen].tape != NULL) {
-				(void) free((void *) ants[screen].tape);
-				ants[screen].tape = NULL;
-			}
-			if (ants[screen].tape != NULL) {
-				(void) free((void *) ants[screen].ants);
-				ants[screen].tape = NULL;
+					XFreePixmap(MI_DISPLAY(mi), ap->pixmaps[shade]);
 			}
+			if (ap->tape != NULL)
+				(void) free((void *) ap->tape);
+			if (ap->ants != NULL)
+				(void) free((void *) ap->ants);
 		}
-		(void) free((void *) ants);
-		ants = NULL;
+		(void) free((void *) antfarms);
+		antfarms = NULL;
 	}
 }
+
+void
+refresh_ant(ModeInfo * mi)
+{
+	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
+
+	ap->redrawing = 1;
+	ap->redrawpos = 0;
+}
