@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)bat.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)bat.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -73,6 +73,9 @@ static char sccsid[] = "@(#)bat.c	3.9 96/05/25 xlockmore";
 #define PENETRATION 0.4
 #define SLIPAGE 4
 #define TIME 32
+#define MINBATS 1
+
+#define ARE_XBM -1
 
 #define ORIENTS 8
 #define ORIENTCYCLE 32
@@ -80,10 +83,9 @@ static char sccsid[] = "@(#)bat.c	3.9 96/05/25 xlockmore";
 #define CW (ORIENTS-1)
 #define DIR(x)	(((x)>=0)?CCW:CW)
 #define SIGN(x)	(((x)>=0)?1:-1)
-#define ABS(x)	(((x)>=0)?x:-(x))
 
 ModeSpecOpt bat_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 static XImage bimages[] =
 {
@@ -93,7 +95,6 @@ static XImage bimages[] =
 	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1},
 	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1}
 };
-static XImage *images[ORIENTS / 2 + 1];
 
 typedef struct {
 	int         x, y, xlast, ylast;
@@ -107,19 +108,22 @@ typedef struct {
 	int         width, height;
 	int         nbats;
 	int         restartnum;
+	int         initialized;
+	int         graphics_format;
 	batstruct  *bats;
+	XImage     *images[ORIENTS / 2 + 1];
 } bouncestruct;
 
-static bouncestruct bounces[MAXSCREENS];
+static bouncestruct *bounces = NULL;
 
-static void checkCollision(int a_bat);
-static void drawabat(Window win, batstruct * bat);
-static void movebat(batstruct * bat);
+static void checkCollision(bouncestruct * bp, int a_bat);
+static void drawabat(ModeInfo * mi, batstruct * bat);
+static void movebat(bouncestruct * bp, batstruct * bat);
 static void flapbat(batstruct * bat, int dir, int *vel);
-static int  collide(int a_bat);
-static void XEraseImage(Display * display, Window win, GC gc, int x, int y, int xlast, int ylast, int xsize, int ysize);
+static int  collide(bouncestruct * bp, int a_bat);
+extern void XEraseImage(Display * display, Window window, GC gc,
+		   int x, int y, int xlast, int ylast, int xsize, int ysize);
 
-static      first = 1;
 
 static unsigned char *bits[] =
 {
@@ -137,85 +141,50 @@ static char **pixs[] =
 static void
 init_images(ModeInfo * mi)
 {
-	int         i;
+	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+	int         i = 0;
 
 #if HAS_XPM
-	int         xpm_ret = 0;
+	XpmAttributes attrib;
+	Display    *display = MI_DISPLAY(mi);
+	int         screen = MI_SCREEN(mi);
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
+	attrib.visual = DefaultVisual(display, screen);
+	attrib.colormap = DefaultColormap(display, screen);
+	attrib.depth = DefaultDepth(display, screen);
+	attrib.valuemask = XpmVisual | XpmColormap | XpmDepth;
+
+	if (MI_NPIXELS(mi) > 2) {
 		for (i = 0; i <= ORIENTS / 2; i++)
-			xpm_ret += XpmCreateImageFromData(dsp, pixs[i], &(images[i]),
-				   (XImage **) NULL, (XpmAttributes *) NULL);
-	if (MI_WIN_IS_MONO(mi) || MI_NPIXELS(mi) <= 2 || xpm_ret != 0)
+			if (XpmSuccess != XpmCreateImageFromData(MI_DISPLAY(mi), pixs[i],
+				&(bp->images[i]), (XImage **) NULL, &attrib))
+				break;
+		bp->initialized = i;
+		if (bp->initialized <= ORIENTS / 2) {	/* All or nothing */
+			for (i = 0; i < bp->initialized; i++)
+				XDestroyImage(bp->images[i]);
+		}
+	}
+	if (bp->initialized <= ORIENTS / 2)
 #endif
-		for (i = 0; i <= ORIENTS / 2; i++) {
-			bimages[i].data = (char *) bits[i];
-			bimages[i].width = bat0_width;
-			bimages[i].height = bat0_height;
-			bimages[i].bytes_per_line = (bat0_width + 7) / 8;
-			images[i] = &(bimages[i]);
-		}
-}
 
-void
-init_bat(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	bouncestruct *bp = &bounces[screen];
-	int         i;
-
-	if (first) {
-		init_images(mi);
-		first = 0;
+	{
+		bp->initialized = ARE_XBM;	/* ie. not using xpm */
+		if (!bimages[i].data)	/* Only need to do this once */
+			for (i = 0; i <= ORIENTS / 2; i++) {
+				bimages[i].data = (char *) bits[i];
+				bimages[i].width = bat0_width;
+				bimages[i].height = bat0_height;
+				bimages[i].bytes_per_line = (bat0_width + 7) / 8;
+			}
+		for (i = 0; i <= ORIENTS / 2; i++)
+			bp->images[i] = &(bimages[i]);
 	}
-	bp->width = MI_WIN_WIDTH(mi);;
-	bp->height = MI_WIN_HEIGHT(mi);
-	bp->restartnum = TIME;
-
-	bp->nbats = MI_BATCHCOUNT(mi);
-	if (bp->nbats < 1)
-		bp->nbats = 1;
-	if (!bp->bats)
-		bp->bats = (batstruct *) malloc(bp->nbats * sizeof (batstruct));
-	i = 0;
-	while (i < bp->nbats) {
-		if (bat0_width > bp->width / 2 || bat0_height > bp->height / 2) {
-			bp->bats[i].sizex = 7;
-			bp->bats[i].sizey = 3;
-			bp->bats[i].size = (bp->bats[i].sizex + bp->bats[i].sizey) / 2;
-		} else {
-			bp->bats[i].sizex = bat0_width;
-			bp->bats[i].sizey = bat0_height;
-			bp->bats[i].size = (bp->bats[i].sizex + bp->bats[i].sizey) / 2;
-		}
-		bp->bats[i].vx = ((LRAND() & 1) ? -1 : 1) * (LRAND() % MAX_STRENGTH + 1);
-		bp->bats[i].x = (bp->bats[i].vx >= 0) ? 0 : bp->width - bp->bats[i].sizex;
-		bp->bats[i].y = LRAND() % (bp->height / 2);
-		if (i == collide(i)) {
-			if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-				bp->bats[i].color = Scr[screen].pixels[LRAND() % Scr[screen].npixels];
-			else
-				bp->bats[i].color = WhitePixel(dsp, screen);
-			bp->bats[i].xlast = -1;
-			bp->bats[i].ylast = 0;
-			bp->bats[i].spincount = 1;
-			bp->bats[i].spindelay = 1;
-			bp->bats[i].vy = ((LRAND() & 1) ? -1 : 1) * (LRAND() % MAX_STRENGTH);
-			bp->bats[i].spindir = 0;
-			bp->bats[i].vang = 0;
-			bp->bats[i].orient = LRAND() % ORIENTS;
-			i++;
-		} else
-			bp->nbats--;
-	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, bp->width, bp->height);
 }
 
 static void
-checkCollision(int a_bat)
+checkCollision(bouncestruct * bp, int a_bat)
 {
-	bouncestruct *bp = &bounces[screen];
 	int         i, amount, spin, d, size;
 	double      x, y;
 
@@ -228,13 +197,13 @@ checkCollision(int a_bat)
 			if (d > 0 && d < size) {
 				amount = size - d;
 				if (amount > PENETRATION * size)
-					amount = PENETRATION * size;
-				bp->bats[i].vx += amount * x / d;
-				bp->bats[i].vy += amount * y / d;
+					amount = (int) (PENETRATION * size);
+				bp->bats[i].vx += (int) ((double) amount * x / d);
+				bp->bats[i].vy += (int) ((double) amount * y / d);
 				bp->bats[i].vx -= bp->bats[i].vx / FRICTION;
 				bp->bats[i].vy -= bp->bats[i].vy / FRICTION;
-				bp->bats[a_bat].vx -= amount * x / d;
-				bp->bats[a_bat].vy -= amount * y / d;
+				bp->bats[a_bat].vx -= (int) ((double) amount * x / d);
+				bp->bats[a_bat].vy -= (int) ((double) amount * y / d);
 				bp->bats[a_bat].vx -= bp->bats[a_bat].vx / FRICTION;
 				bp->bats[a_bat].vy -= bp->bats[a_bat].vy / FRICTION;
 				spin = (bp->bats[i].vang - bp->bats[a_bat].vang) /
@@ -247,69 +216,53 @@ checkCollision(int a_bat)
 					bp->bats[i].spindelay = 1;
 					bp->bats[i].spindir = 0;
 				} else
-					bp->bats[i].spindelay = M_PI * bp->bats[i].size /
-						(ABS(bp->bats[i].vang)) + 1;
+					bp->bats[i].spindelay = (int) ((double) M_PI *
+						bp->bats[i].size / (ABS(bp->bats[i].vang))) + 1;
 				if (!bp->bats[a_bat].vang) {
 					bp->bats[a_bat].spindelay = 1;
 					bp->bats[a_bat].spindir = 0;
 				} else
-					bp->bats[a_bat].spindelay = M_PI * bp->bats[a_bat].size /
-						(ABS(bp->bats[a_bat].vang)) + 1;
+					bp->bats[a_bat].spindelay = (int) ((double) M_PI *
+						bp->bats[a_bat].size / (ABS(bp->bats[a_bat].vang))) + 1;
 				return;
 			}
 		}
 	}
 }
 
-void
-draw_bat(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	bouncestruct *bp = &bounces[screen];
-	int         i;
-
-	for (i = 0; i < bp->nbats; i++) {
-		drawabat(win, &bp->bats[i]);
-		movebat(&bp->bats[i]);
-	}
-	for (i = 0; i < bp->nbats; i++)
-		checkCollision(i);
-	if (!(LRAND() % TIME))	/* Put some randomness into the time */
-		bp->restartnum--;
-	if (!bp->restartnum)
-		init_bat(mi);
-}
-
 static void
-drawabat(Window win, batstruct * bat)
+drawabat(ModeInfo * mi, batstruct * bat)
 {
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+
 	if (bat->sizex < bat0_width) {
 		if (bat->xlast != -1) {
-			XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-			XFillRectangle(dsp, win, Scr[screen].gc,
+			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XFillRectangle(display, window, MI_GC(mi),
 			     bat->xlast, bat->ylast, bat->sizex, bat->sizey);
 		}
-		XSetForeground(dsp, Scr[screen].gc, bat->color);
-		XFillRectangle(dsp, win, Scr[screen].gc,
+		XSetForeground(display, MI_GC(mi), bat->color);
+		XFillRectangle(display, window, MI_GC(mi),
 			       bat->x, bat->y, bat->sizex, bat->sizey);
 	} else {
-		XSetForeground(dsp, Scr[screen].gc, bat->color);
-		XPutImage(dsp, win, Scr[screen].gc,
-			  images[(bat->orient > ORIENTS / 2) ? ORIENTS - bat->orient : bat->orient],
+		XSetForeground(display, MI_GC(mi), bat->color);
+		XPutImage(display, window, MI_GC(mi),
+			  bp->images[(bat->orient > ORIENTS / 2) ?
+				     ORIENTS - bat->orient : bat->orient],
 			  0, 0, bat->x, bat->y, bat->sizex, bat->sizey);
 		if (bat->xlast != -1) {
-			XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-			XEraseImage(dsp, win, Scr[screen].gc,
+			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XEraseImage(display, window, MI_GC(mi),
 				    bat->x, bat->y, bat->xlast, bat->ylast, bat->sizex, bat->sizey);
 		}
 	}
 }
 
 static void
-movebat(batstruct * bat)
+movebat(bouncestruct * bp, batstruct * bat)
 {
-	bouncestruct *bp = &bounces[screen];
-
 	bat->xlast = bat->x;
 	bat->ylast = bat->y;
 	bat->x += bat->vx;
@@ -349,20 +302,19 @@ movebat(batstruct * bat)
 static void
 flapbat(batstruct * bat, int dir, int *vel)
 {
-	*vel -= (*vel + SIGN(*vel * dir) * bat->spindelay * ORIENTCYCLE /
-		 (M_PI * bat->size)) / SLIPAGE;
+	*vel -= (int) ((*vel + SIGN(*vel * dir) *
+    bat->spindelay * ORIENTCYCLE / (M_PI * bat->size)) / SLIPAGE);
 	if (*vel) {
 		bat->spindir = DIR(*vel * dir);
 		bat->vang = *vel * ORIENTCYCLE;
-		bat->spindelay = M_PI * bat->size / (ABS(bat->vang)) + 1;
+		bat->spindelay = (int) ((double) M_PI * bat->size / (ABS(bat->vang))) + 1;
 	} else
 		bat->spindir = 0;
 }
 
 static int
-collide(int a_bat)
+collide(bouncestruct * bp, int a_bat)
 {
-	bouncestruct *bp = &bounces[screen];
 	int         i, d, x, y;
 
 	for (i = 0; i < a_bat; i++) {
@@ -375,30 +327,119 @@ collide(int a_bat)
 	return i;
 }
 
-/* This stops some flashing, could be more efficient */
-static void
-XEraseImage(Display * display, Window win, GC gc, int x, int y, int xlast, int ylast, int xsize, int ysize)
+void
+init_bat(ModeInfo * mi)
 {
-	if (ylast < y) {
-		if (y < ylast + ysize)
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, y - ylast);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
-	} else if (ylast > y) {
-		if (y > ylast - ysize)
-			XFillRectangle(display, win, gc, xlast, y + ysize, xsize, ylast - y);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
+	bouncestruct *bp;
+	int         i, tryagain = 0;
+
+	if (bounces == NULL) {
+		if ((bounces = (bouncestruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (bouncestruct))) == NULL)
+			return;
 	}
-	if (xlast < x) {
-		if (x < xlast + xsize)
-			XFillRectangle(display, win, gc, xlast, ylast, x - xlast, ysize);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
-	} else if (xlast > x) {
-		if (x > xlast - xsize)
-			XFillRectangle(display, win, gc, x + xsize, ylast, xlast - x, ysize);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
+	bp = &bounces[MI_SCREEN(mi)];
+	if (!bp->initialized) {
+		init_images(mi);
+	}
+	bp->width = MI_WIN_WIDTH(mi);
+	bp->height = MI_WIN_HEIGHT(mi);
+	if (bp->width < 2)
+		bp->width = 2;
+	if (bp->height < 2)
+		bp->height = 2;
+	bp->restartnum = TIME;
+
+	bp->nbats = MI_BATCHCOUNT(mi);
+	if (bp->nbats < -MINBATS) {
+		/* if bp->nbats is random ... the size can change */
+		if (bp->bats != NULL) {
+			(void) free((void *) bp->bats);
+			bp->bats = NULL;
+		}
+		bp->nbats = NRAND(-bp->nbats - MINBATS + 1) + MINBATS;
+	} else if (bp->nbats < MINBATS)
+		bp->nbats = MINBATS;
+	if (!bp->bats)
+		bp->bats = (batstruct *) malloc(bp->nbats * sizeof (batstruct));
+	i = 0;
+	while (i < bp->nbats) {
+		if (bat0_width > bp->width / 2 || bat0_height > bp->height / 2) {
+			bp->bats[i].sizex = 7;
+			bp->bats[i].sizey = 3;
+			bp->bats[i].size = (bp->bats[i].sizex + bp->bats[i].sizey) / 2;
+		} else {
+			bp->bats[i].sizex = bat0_width;
+			bp->bats[i].sizey = bat0_height;
+			bp->bats[i].size = (bp->bats[i].sizex + bp->bats[i].sizey) / 2;
+		}
+		bp->bats[i].vx = ((LRAND() & 1) ? -1 : 1) * (NRAND(MAX_STRENGTH) + 1);
+		bp->bats[i].x = (bp->bats[i].vx >= 0) ? 0 : bp->width - bp->bats[i].sizex;
+		bp->bats[i].y = NRAND(bp->height / 2);
+		if (i == collide(bp, i) || tryagain >= 8) {
+			if (MI_NPIXELS(mi) > 2)
+				bp->bats[i].color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+			else
+				bp->bats[i].color = MI_WIN_WHITE_PIXEL(mi);
+			bp->bats[i].xlast = -1;
+			bp->bats[i].ylast = 0;
+			bp->bats[i].spincount = 1;
+			bp->bats[i].spindelay = 1;
+			bp->bats[i].vy = ((LRAND() & 1) ? -1 : 1) * NRAND(MAX_STRENGTH);
+			bp->bats[i].spindir = 0;
+			bp->bats[i].vang = 0;
+			bp->bats[i].orient = NRAND(ORIENTS);
+			i++;
+		} else
+			tryagain++;
+	}
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
+
+void
+draw_bat(ModeInfo * mi)
+{
+	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+	int         i;
+
+	for (i = 0; i < bp->nbats; i++) {
+		drawabat(mi, &bp->bats[i]);
+		movebat(bp, &bp->bats[i]);
+	}
+	for (i = 0; i < bp->nbats; i++)
+		checkCollision(bp, i);
+	if (!NRAND(TIME))	/* Put some randomness into the time */
+		bp->restartnum--;
+	if (!bp->restartnum)
+		init_bat(mi);
+}
+
+void
+release_bat(ModeInfo * mi)
+{
+	if (bounces != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			bouncestruct *bp = &bounces[screen];
+
+#ifdef HAS_XPM
+			int         i;
+
+			for (i = 0; i < bp->initialized; i++)	/* ignores ARE_XBM */
+				XDestroyImage(bp->images[i]);
+#endif
+
+			if (bp->bats != NULL)
+				(void) free((void *) bp->bats);
+		}
+		(void) free((void *) bounces);
+		bounces = NULL;
 	}
 }
+
+void
+refresh_bat(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
