@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)blot.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)blot.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -31,7 +31,7 @@ static char sccsid[] = "@(#)blot.c	3.9 96/05/25 xlockmore";
 #include "xlock.h"
 
 ModeSpecOpt blot_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         width;
@@ -39,20 +39,27 @@ typedef struct {
 	int         xmid, ymid;
 	int         offset;
 	int         xsym, ysym;
-	int         size;
+	unsigned long size;
 	int         pix;
 	int         count;
+	XPoint     *pointBuffer;
+	unsigned long pointBufferSize;
 } blotstruct;
 
-static blotstruct blots[MAXSCREENS];
-static XPoint *pointBuffer = 0;	/* pointer for XDrawPoints */
-static unsigned long pointBufferSize = 0;
+static blotstruct *blots = NULL;
 
 void
 init_blot(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	blotstruct *bp = &blots[screen];
+	blotstruct *bp;
+	Display    *display = MI_DISPLAY(mi);
+
+	if (blots == NULL) {
+		if ((blots = (blotstruct *) calloc(MI_NUM_SCREENS(mi),
+					       sizeof (blotstruct))) == NULL)
+			return;
+	}
+	bp = &blots[MI_SCREEN(mi)];
 
 	bp->width = MI_WIN_WIDTH(mi);
 	bp->height = MI_WIN_HEIGHT(mi);
@@ -65,44 +72,43 @@ init_blot(ModeInfo * mi)
 	bp->pix = 0;
 	if (bp->offset <= 0)
 		bp->offset = 3;
-	bp->size = MI_BATCHCOUNT(mi);
-	if (bp->size >= 100 || bp->size < 1)
-		bp->size = 6;
-	/* Fudge the size so it takes up the whole screen */
-	bp->size *= bp->width * bp->height / 1024;
+	if (MI_BATCHCOUNT(mi) < 0)
+		bp->size = NRAND(-MI_BATCHCOUNT(mi) + 1);
+	else
+		bp->size = MI_BATCHCOUNT(mi);
 
-	if (!pointBuffer || pointBufferSize < (bp->size * sizeof (XPoint))) {
-		if (pointBuffer != NULL)
-			(void) free((void *) pointBuffer);
-		pointBuffer = (XPoint *) malloc(bp->size * sizeof (XPoint));
-		pointBufferSize = bp->size * sizeof (XPoint);
+	/* Fudge the size so it takes up the whole screen */
+	bp->size *= (unsigned long) (bp->width / 32 + 1) * (bp->height / 32 + 1);
+	if (!bp->pointBuffer || bp->pointBufferSize < bp->size * sizeof (XPoint)) {
+		if (bp->pointBuffer != NULL)
+			(void) free((void *) bp->pointBuffer);
+		bp->pointBufferSize = bp->size * sizeof (XPoint);
+		bp->pointBuffer = (XPoint *) malloc(bp->pointBufferSize);
 	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, bp->width, bp->height);
-	XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+	XClearWindow(display, MI_WINDOW(mi));
+	XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
 	bp->count = 0;
 }
 
 void
 draw_blot(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	blotstruct *bp = &blots[MI_SCREEN(mi)];
+	XPoint     *xp = bp->pointBuffer;
 	int         x, y;
-	int         k;
-	XPoint     *xp = pointBuffer;
-	blotstruct *bp = &blots[screen];
+	unsigned long k;
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[bp->pix]);
-		if (++bp->pix >= Scr[screen].npixels)
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, bp->pix));
+		if (++bp->pix >= MI_NPIXELS(mi))
 			bp->pix = 0;
 	}
 	x = bp->xmid;
 	y = bp->ymid;
 	k = bp->size;
 	while (k >= 4) {
-		x += ((LRAND() % (1 + (bp->offset << 1))) - bp->offset);
-		y += ((LRAND() % (1 + (bp->offset << 1))) - bp->offset);
+		x += (NRAND(1 + (bp->offset << 1)) - bp->offset);
+		y += (NRAND(1 + (bp->offset << 1)) - bp->offset);
 		k--;
 		xp->x = x;
 		xp->y = y;
@@ -126,8 +132,31 @@ draw_blot(ModeInfo * mi)
 			xp++;
 		}
 	}
-	XDrawPoints(dsp, win, Scr[screen].gc,
-		    pointBuffer, bp->size - k, CoordModeOrigin);
+	XDrawPoints(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+		    bp->pointBuffer, (bp->size - k), CoordModeOrigin);
 	if (++bp->count > MI_CYCLES(mi))
 		init_blot(mi);
 }
+
+
+void
+release_blot(ModeInfo * mi)
+{
+	if (blots != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			if (blots[screen].pointBuffer != NULL) {
+				(void) free((void *) blots[screen].pointBuffer);
+			}
+		}
+		(void) free((void *) blots);
+		blots = NULL;
+	}
+}
+
+void
+refresh_blot(ModeInfo * mi)
+{
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
