@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)bounce.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)bounce.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -62,12 +63,17 @@ static char sccsid[] = "@(#)bounce.c	3.9 96/05/25 xlockmore";
 #include "bitmaps/bounce-1.xbm"
 #include "bitmaps/bounce-2.xbm"
 #include "bitmaps/bounce-3.xbm"
+#include "bitmaps/bounce-mask.xbm"
+#define BALLBITS(n,w,h)\
+  bp->pixmaps[bp->init_orients++]=\
+		XCreateBitmapFromData(display,window,(char *)n,w,h)
 
 #define MAX_STRENGTH 24
 #define FRICTION 24
 #define PENETRATION 0.3
 #define SLIPAGE 4
 #define TIME 32
+#define MINBALLS 1
 
 #define ORIENTS 4
 #define ORIENTCYCLE 16
@@ -75,21 +81,12 @@ static char sccsid[] = "@(#)bounce.c	3.9 96/05/25 xlockmore";
 #define CW (ORIENTS-1)
 #define DIR(x)	(((x)>=0)?CCW:CW)
 #define SIGN(x)	(((x)>=0)?1:-1)
-#define ABS(x)	(((x)>=0)?x:-(x))
 
 ModeSpecOpt bounce_opts =
-{0, NULL, NULL, NULL};
-
-static XImage bimages[] =
-{
-	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1},
-	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1},
-	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1},
-	{0, 0, 0, XYBitmap, 0, LSBFirst, 8, LSBFirst, 8, 1},
-};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
-	int         x, y, xlast, ylast;
+	int         x, y, xlast, ylast, orientlast;
 	int         spincount, spindelay, spindir, orient;
 	int         vx, vy, vang;
 	int         mass, size;
@@ -101,94 +98,22 @@ typedef struct {
 	int         nballs;
 	int         restartnum;
 	ballstruct *balls;
+	GC          stippledGC;
+	Pixmap      pixmaps[ORIENTS + 1];
+	int         init_orients;
 } bouncestruct;
 
-static bouncestruct bounces[MAXSCREENS];
+static bouncestruct *bounces = NULL;
 
-static void checkCollision(int aball);
-static void drawball(Window win, ballstruct * ball);
-static void moveball(ballstruct * ball);
+static void checkCollision(bouncestruct * bp, int aball);
+static void drawball(ModeInfo * mi, ballstruct * ball);
+static void moveball(bouncestruct * bp, ballstruct * ball);
 static void spinball(ballstruct * ball, int dir, int *vel);
-static int  collide(int aball);
-static void XEraseImage(Display * display, Window win, GC gc, int x, int y, int xlast, int ylast, int xsize, int ysize);
-
-static      first = 1;
-
-static unsigned char *bits[] =
-{
-	bounce0_bits, bounce1_bits, bounce2_bits, bounce3_bits
-};
+static int  collide(bouncestruct * bp, int aball);
 
 static void
-init_images(void)
+checkCollision(bouncestruct * bp, int aball)
 {
-	int         i;
-
-	for (i = 0; i < ORIENTS; i++) {
-		bimages[i].data = (char *) bits[i];
-		bimages[i].width = bounce0_width;
-		bimages[i].height = bounce0_height;
-		bimages[i].bytes_per_line = (bounce0_width + 7) / 8;
-	}
-}
-
-void
-init_bounce(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	bouncestruct *bp = &bounces[screen];
-	int         i;
-
-	if (first) {
-		init_images();
-		first = 0;
-	}
-	bp->width = MI_WIN_WIDTH(mi);
-	bp->height = MI_WIN_HEIGHT(mi);
-	bp->restartnum = TIME;
-
-	bp->nballs = MI_BATCHCOUNT(mi);
-	if (bp->nballs < 1)
-		bp->nballs = 1;
-	if (!bp->balls)
-		bp->balls = (ballstruct *) malloc(bp->nballs * sizeof (ballstruct));
-	i = 0;
-	while (i < bp->nballs) {
-		if (bounce0_width > bp->width / 2 || bounce0_height > bp->height / 2)
-			bp->balls[i].size = 5;
-		else
-			bp->balls[i].size = (bounce0_width + bounce0_height) / 2;
-		bp->balls[i].vx = ((LRAND() & 1) ? -1 : 1) *
-			(LRAND() % MAX_STRENGTH + 1);
-		bp->balls[i].x = (bp->balls[i].vx >= 0) ?
-			0 : bp->width - bp->balls[i].size;
-		bp->balls[i].y = LRAND() % (bp->height / 2);
-		if (i == collide(i)) {
-			if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-				bp->balls[i].color =
-					Scr[screen].pixels[LRAND() % Scr[screen].npixels];
-			else
-				bp->balls[i].color = WhitePixel(dsp, screen);
-			bp->balls[i].xlast = -1;
-			bp->balls[i].ylast = 0;
-			bp->balls[i].spincount = 1;
-			bp->balls[i].spindelay = 1;
-			bp->balls[i].vy = ((LRAND() & 1) ? -1 : 1) * (LRAND() % MAX_STRENGTH);
-			bp->balls[i].spindir = 0;
-			bp->balls[i].vang = 0;
-			bp->balls[i].orient = LRAND() % ORIENTS;
-			i++;
-		} else
-			bp->nballs--;
-	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, bp->width, bp->height);
-}
-
-static void
-checkCollision(int aball)
-{
-	bouncestruct *bp = &bounces[screen];
 	int         i, amount, spin, d, size;
 	double      x, y;
 
@@ -201,13 +126,13 @@ checkCollision(int aball)
 			if (d > 0 && d < size) {
 				amount = size - d;
 				if (amount > PENETRATION * size)
-					amount = PENETRATION * size;
-				bp->balls[i].vx += amount * x / d;
-				bp->balls[i].vy += amount * y / d;
+					amount = (int) (PENETRATION * size);
+				bp->balls[i].vx += (int) ((double) amount * x / d);
+				bp->balls[i].vy += (int) ((double) amount * y / d);
 				bp->balls[i].vx -= bp->balls[i].vx / FRICTION;
 				bp->balls[i].vy -= bp->balls[i].vy / FRICTION;
-				bp->balls[aball].vx -= amount * x / d;
-				bp->balls[aball].vy -= amount * y / d;
+				bp->balls[aball].vx -= (int) ((double) amount * x / d);
+				bp->balls[aball].vy -= (int) ((double) amount * y / d);
 				bp->balls[aball].vx -= bp->balls[aball].vx / FRICTION;
 				bp->balls[aball].vy -= bp->balls[aball].vy / FRICTION;
 				spin = (bp->balls[i].vang - bp->balls[aball].vang) /
@@ -220,71 +145,76 @@ checkCollision(int aball)
 					bp->balls[i].spindelay = 1;
 					bp->balls[i].spindir = 0;
 				} else
-					bp->balls[i].spindelay = M_PI * bp->balls[i].size /
-						(ABS(bp->balls[i].vang)) + 1;
+					bp->balls[i].spindelay = (int) ((double) M_PI *
+						bp->balls[i].size / (ABS(bp->balls[i].vang))) + 1;
 				if (!bp->balls[aball].vang) {
 					bp->balls[aball].spindelay = 1;
 					bp->balls[aball].spindir = 0;
 				} else
-					bp->balls[aball].spindelay = M_PI * bp->balls[aball].size /
-						(ABS(bp->balls[aball].vang)) + 1;
+					bp->balls[aball].spindelay = (int) ((double) M_PI *
+						bp->balls[aball].size / (ABS(bp->balls[aball].vang))) + 1;
 				return;
 			}
 		}
 	}
 }
 
-void
-draw_bounce(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	bouncestruct *bp = &bounces[screen];
-	int         i;
-
-	for (i = 0; i < bp->nballs; i++) {
-		drawball(win, &bp->balls[i]);
-		moveball(&bp->balls[i]);
-	}
-	for (i = 0; i < bp->nballs; i++)
-		checkCollision(i);
-	if (!(LRAND() % TIME))	/* Put some randomness into the time */
-		bp->restartnum--;
-	if (!bp->restartnum)
-		init_bounce(mi);
-}
-
 static void
-drawball(Window win, ballstruct * ball)
+drawball(ModeInfo * mi, ballstruct * ball)
 {
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+
 	if (ball->size <= 5) {
 		if (ball->xlast != -1) {
-			XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-			XFillRectangle(dsp, win, Scr[screen].gc,
+			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XFillRectangle(display, window, MI_GC(mi),
 			   ball->xlast, ball->ylast, ball->size, ball->size);
 		}
-		XSetForeground(dsp, Scr[screen].gc, ball->color);
-		XFillRectangle(dsp, win, Scr[screen].gc,
+		XSetForeground(display, MI_GC(mi), ball->color);
+		XFillRectangle(display, window, MI_GC(mi),
 			       ball->x, ball->y, ball->size, ball->size);
 	} else {
-		XSetForeground(dsp, Scr[screen].gc, ball->color);
-		XPutImage(dsp, win, Scr[screen].gc,
-			  &(bimages[ball->orient]),
-			  0, 0, ball->x, ball->y, ball->size, ball->size);
+		bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+
 		if (ball->xlast != -1) {
-			XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-			XEraseImage(dsp, win, Scr[screen].gc,
-				    ball->x, ball->y, ball->xlast, ball->ylast, ball->size, ball->size);
+#ifndef FLASH
+			extern void XEraseImage(Display * display, Window window, GC gc,
+						int x, int y, int xlast, int ylast, int xsize, int ysize);
+
+#endif
+			XSetForeground(display, bp->stippledGC, MI_WIN_BLACK_PIXEL(mi));
+			XSetStipple(display, bp->stippledGC, bp->pixmaps[ORIENTS]);
+			XSetFillStyle(display, bp->stippledGC, FillStippled);
+			XSetTSOrigin(display, bp->stippledGC, ball->xlast, ball->ylast);
+#ifdef FLASH
+			XFillRectangle(display, window, bp->stippledGC,
+			   ball->xlast, ball->ylast, ball->size, ball->size);
+#else
+			XEraseImage(display, window, bp->stippledGC, ball->x, ball->y,
+			   ball->xlast, ball->ylast, ball->size, ball->size);
+#endif
 		}
+		XSetTSOrigin(display, bp->stippledGC, ball->x, ball->y);
+		XSetForeground(display, bp->stippledGC, ball->color);
+		XSetStipple(display, bp->stippledGC, bp->pixmaps[ball->orient]);
+#ifdef FLASH
+		XSetFillStyle(display, bp->stippledGC, FillStippled);
+#else
+		XSetFillStyle(display, bp->stippledGC, FillOpaqueStippled);
+#endif
+		XFillRectangle(display, window, bp->stippledGC,
+			       ball->x, ball->y, ball->size, ball->size);
+		XFlush(display);
 	}
 }
 
 static void
-moveball(ballstruct * ball)
+moveball(bouncestruct * bp, ballstruct * ball)
 {
-	bouncestruct *bp = &bounces[screen];
-
 	ball->xlast = ball->x;
 	ball->ylast = ball->y;
+	ball->orientlast = ball->orient;
 	ball->x += ball->vx;
 	if (ball->x > (bp->width - ball->size)) {
 		/* Bounce off the right edge */
@@ -324,20 +254,20 @@ moveball(ballstruct * ball)
 static void
 spinball(ballstruct * ball, int dir, int *vel)
 {
-	*vel -= (*vel + SIGN(*vel * dir) * ball->spindelay * ORIENTCYCLE /
-		 (M_PI * ball->size)) / SLIPAGE;
+	*vel -= (int) ((*vel + SIGN(*vel * dir) * ball->spindelay * ORIENTCYCLE /
+    (M_PI * ball->size)) / SLIPAGE);
 	if (*vel) {
 		ball->spindir = DIR(*vel * dir);
 		ball->vang = *vel * ORIENTCYCLE;
-		ball->spindelay = M_PI * ball->size / (ABS(ball->vang)) + 1;
+		ball->spindelay =
+			(int) ((double) M_PI * ball->size / (ABS(ball->vang))) + 1;
 	} else
 		ball->spindir = 0;
 }
 
 static int
-collide(int aball)
+collide(bouncestruct * bp, int aball)
 {
-	bouncestruct *bp = &bounces[screen];
 	int         i, d, x, y;
 
 	for (i = 0; i < aball; i++) {
@@ -350,30 +280,131 @@ collide(int aball)
 	return i;
 }
 
-/* This stops some flashing, could be more efficient */
-static void
-XEraseImage(Display * display, Window win, GC gc, int x, int y, int xlast, int ylast, int xsize, int ysize)
+void
+init_bounce(ModeInfo * mi)
 {
-	if (ylast < y) {
-		if (y < ylast + ysize)
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, y - ylast);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
-	} else if (ylast > y) {
-		if (y > ylast - ysize)
-			XFillRectangle(display, win, gc, xlast, y + ysize, xsize, ylast - y);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	bouncestruct *bp;
+	XGCValues   gcv;
+	int         i, tryagain = 0;
+
+	if (bounces == NULL) {
+		if ((bounces = (bouncestruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (bouncestruct))) == NULL)
+			return;
 	}
-	if (xlast < x) {
-		if (x < xlast + xsize)
-			XFillRectangle(display, win, gc, xlast, ylast, x - xlast, ysize);
+	bp = &bounces[MI_SCREEN(mi)];
+
+	if (!bp->stippledGC) {
+		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
+		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		if ((bp->stippledGC = XCreateGC(display, window,
+				 GCForeground | GCBackground, &gcv)) == None)
+			return;
+	}
+	if (!bp->init_orients) {
+		BALLBITS(bounce0_bits, bounce0_width, bounce0_height);
+		BALLBITS(bounce1_bits, bounce1_width, bounce1_height);
+		BALLBITS(bounce2_bits, bounce2_width, bounce2_height);
+		BALLBITS(bounce3_bits, bounce3_width, bounce3_height);
+		BALLBITS(bouncemask_bits, bouncemask_width, bouncemask_height);
+	}
+	bp->width = MI_WIN_WIDTH(mi);
+	bp->height = MI_WIN_HEIGHT(mi);
+	if (bp->width < 2)
+		bp->width = 2;
+	if (bp->height < 2)
+		bp->height = 2;
+	bp->restartnum = TIME;
+
+	bp->nballs = MI_BATCHCOUNT(mi);
+	if (bp->nballs < -MINBALLS) {
+		/* if bp->nballs is random ... the size can change */
+		if (bp->balls != NULL) {
+			(void) free((void *) bp->balls);
+			bp->balls = NULL;
+		}
+		bp->nballs = NRAND(-bp->nballs - MINBALLS + 1) + MINBALLS;
+	} else if (bp->nballs < MINBALLS)
+		bp->nballs = MINBALLS;
+	if (!bp->balls)
+		bp->balls = (ballstruct *) malloc(bp->nballs * sizeof (ballstruct));
+	i = 0;
+	while (i < bp->nballs) {
+		if (bounce0_width > bp->width / 2 || bounce0_height > bp->height / 2)
+			bp->balls[i].size = 5;
 		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
-	} else if (xlast > x) {
-		if (x > xlast - xsize)
-			XFillRectangle(display, win, gc, x + xsize, ylast, xlast - x, ysize);
-		else
-			XFillRectangle(display, win, gc, xlast, ylast, xsize, ysize);
+			bp->balls[i].size = (bounce0_width + bounce0_height) / 2;
+		bp->balls[i].vx = ((LRAND() & 1) ? -1 : 1) *
+			(NRAND(MAX_STRENGTH) + 1);
+		bp->balls[i].x = (bp->balls[i].vx >= 0) ?
+			0 : bp->width - bp->balls[i].size;
+		bp->balls[i].y = NRAND(bp->height / 2);
+		if (i == collide(bp, i) || tryagain >= 8) {
+			if (MI_NPIXELS(mi) > 2)
+				bp->balls[i].color =
+					MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+			else
+				bp->balls[i].color = MI_WIN_WHITE_PIXEL(mi);
+			bp->balls[i].xlast = -1;
+			bp->balls[i].ylast = 0;
+			bp->balls[i].orientlast = 0;
+			bp->balls[i].spincount = 1;
+			bp->balls[i].spindelay = 1;
+			bp->balls[i].vy = ((LRAND() & 1) ? -1 : 1) * NRAND(MAX_STRENGTH);
+			bp->balls[i].spindir = 0;
+			bp->balls[i].vang = 0;
+			bp->balls[i].orient = NRAND(ORIENTS);
+			i++;
+		} else
+			tryagain++;
+	}
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
+
+void
+draw_bounce(ModeInfo * mi)
+{
+	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+	int         i;
+
+	for (i = 0; i < bp->nballs; i++) {
+		drawball(mi, &bp->balls[i]);
+		moveball(bp, &bp->balls[i]);
+	}
+	for (i = 0; i < bp->nballs; i++)
+		checkCollision(bp, i);
+	if (!NRAND(TIME))	/* Put some randomness into the time */
+		bp->restartnum--;
+	if (!bp->restartnum)
+		init_bounce(mi);
+}
+
+void
+release_bounce(ModeInfo * mi)
+{
+	if (bounces != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			bouncestruct *bp = &bounces[screen];
+			int         bits;
+
+			if (bp->balls != NULL)
+				(void) free((void *) bp->balls);
+			if (bp->stippledGC != NULL)
+				XFreeGC(MI_DISPLAY(mi), bp->stippledGC);
+			for (bits = 0; bits < bp->init_orients; bits++)
+				XFreePixmap(MI_DISPLAY(mi), bp->pixmaps[bits]);
+		}
+		(void) free((void *) bounces);
+		bounces = NULL;
 	}
 }
+
+void
+refresh_bounce(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
