@@ -1,13 +1,13 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)eyes.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)eyes.c	3.11 96/09/20 xlockmore";
 
 #endif
 
 /*-
  * eyes.c - Follow the bouncing Grelb
  *
- * Copyright 1996 by Ron Hitchens <ron@utw.com>
+ * Copyright 1996 by Ron Hitchens <ronh@utw.com>
  *
  * Adapted from the ubiquitous xeyes demo supplied by MIT with the
  * X Window System.
@@ -31,7 +31,7 @@ static char sccsid[] = "@(#)eyes.c	3.9 96/05/25 xlockmore";
 /* ---------------------------------------------------------------------- */
 
 /* definitions for the xlock version of xeyes */
-#define MAX_EYES		10	/* max number of active eye pairs */
+#define MAX_EYES		200	/* limit or uses too much memory */
 #define MIN_EYE_SIZE		50	/* smallest size eyes can be */
 #define FLY_ICON_SIZE		5	/* the size of a fly when iconic */
 #define FLY_MAX_SPEED		10	/* max (slowest) delay between steps */
@@ -89,11 +89,11 @@ typedef struct _TPoint {
 /* ---------------------------------------------------------------------- */
 
 /* aliases for vars defined in the bitmap file */
-#define FLY_WIDTH		life_width
-#define FLY_HEIGHT		life_height
-#define FLY_BITS		life_bits
+#define FLY_WIDTH		image_width
+#define FLY_HEIGHT		image_height
+#define FLY_BITS		image_bits
 
-#include "bitmaps/life-grelb.xbm"
+#include "eyes.xbm"
 
 typedef struct {		/* info about a "fly" */
 	int         x, y;
@@ -123,19 +123,18 @@ typedef struct {		/* info about a pair of eyes */
 
 typedef struct {		/* per-screen info */
 	int         num_eyes;
-	Eyes        eyes[MAX_EYES];
+	Eyes       *eyes;
 	Fly         fly;
+	GC          flyGC;
+	GC          eyeGC;
+	Pixmap      flypix;
 } EyeScrInfo;
 
 /* ---------------------------------------------------------------------- */
 
 /* xlock defaults for eyes: this var must be visible outside this module */
 ModeSpecOpt eyes_opts =
-{0, NULL, NULL, NULL};
-
-static Pixmap flypix = None;
-static GC   flygc = None;
-static GC   eyegc = None;
+{0, NULL, 0, NULL, NULL};
 
 static EyeScrInfo *eye_info = NULL;
 
@@ -278,9 +277,11 @@ computePupil(int num, TPoint mouse)
  */
 
 static void
-make_eye(Display * display, Drawable d, Eyes * e, int n, int full)
+make_eye(ModeInfo * mi, Drawable d, Eyes * e, int n, int full)
 {
-	GC          gc = eyegc;
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = ep->eyeGC;
 	XRectangle *bbox = &e->bbox;
 	XRectangle  tmp1, tmp2;
 
@@ -419,7 +420,7 @@ init_fly(ModeInfo * mi, Fly * f)
 	}
 	f->oldx = -(f->width);	/* prevent undraw 1st time */
 
-	if (MI_WIN_IS_MONO(mi)) {
+	if (MI_NPIXELS(mi) <= 2) {
 		f->pixel = MI_WIN_WHITE_PIXEL(mi);	/* always white when mono */
 	} else {
 		f->pixel = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
@@ -435,15 +436,16 @@ unpaint_fly(ModeInfo * mi, Fly * f)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
 
 	if (MI_WIN_IS_ICONIC(mi)) {
 		XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
 		XFillArc(display, window, MI_GC(mi), f->oldx, f->oldy,
 			 f->width, f->height, 90 * 64, 360 * 64);
 	} else {
-		XSetForeground(display, flygc, MI_WIN_BLACK_PIXEL(mi));
-		XSetTSOrigin(display, flygc, f->oldx, f->oldy);
-		XFillRectangle(display, window, flygc, f->oldx, f->oldy,
+		XSetForeground(display, ep->flyGC, MI_WIN_BLACK_PIXEL(mi));
+		XSetTSOrigin(display, ep->flyGC, f->oldx, f->oldy);
+		XFillRectangle(display, window, ep->flyGC, f->oldx, f->oldy,
 			       f->width, f->height);
 	}
 }
@@ -458,22 +460,23 @@ unpaint_fly(ModeInfo * mi, Fly * f)
 static void
 paint_fly(ModeInfo * mi, Fly * f)
 {
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	int         x = f->x, y = f->y;
 
 	if (MI_WIN_IS_ICONIC(mi)) {
 		/* don't need to unpaint when iconic
-		 * flygc has stipple set, don't use when iconic
+		 * ep->flyGC has stipple set, don't use when iconic
 		 */
 		XSetForeground(display, MI_GC(mi), f->pixel);
 		XFillArc(display, window, MI_GC(mi), x, y,
 			 f->width, f->height, 90 * 64, 360 * 64);
 	} else {
 		unpaint_fly(mi, f);
-		XSetForeground(display, flygc, f->pixel);
-		XSetTSOrigin(display, flygc, x, y);
-		XFillRectangle(display, window, flygc,
+		XSetForeground(display, ep->flyGC, f->pixel);
+		XSetTSOrigin(display, ep->flyGC, x, y);
+		XFillRectangle(display, window, ep->flyGC,
 			       x, y, f->width, f->height);
 	}
 }
@@ -551,6 +554,7 @@ move_fly(ModeInfo * mi, Fly * f)
 static void
 create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 {
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	int         win_width = MI_WIN_WIDTH(mi);
@@ -582,7 +586,7 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 
 	e->pupil_pixel = black_pixel;	/* pupil is always black */
 
-	if (MI_WIN_IS_MONO(mi)) {
+	if (MI_NPIXELS(mi) <= 2) {
 		/* TODO: stipple the eyelid? */
 		e->eyelid_pixel = black_pixel;
 		e->eyeball_pixel = white_pixel;
@@ -606,11 +610,13 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 		e->width = win_width;
 		e->height = win_height;
 	} else {
-		e->width = NRAND(maxw - MIN_EYE_SIZE) + MIN_EYE_SIZE;
-		e->x = NRAND(win_width - e->width);
-
+		if (maxw - MIN_EYE_SIZE > MIN_EYE_SIZE)
+			e->width = NRAND(maxw - MIN_EYE_SIZE) + MIN_EYE_SIZE;
+		else
+			e->width = NRAND(MIN_EYE_SIZE) + MIN_EYE_SIZE;
+		e->x = (win_width - e->width > 0) ? NRAND(win_width - e->width) : 0;
 		e->height = NRAND(e->width * 3 / 4) + (e->width / 4);
-		e->y = NRAND(win_height - e->height);
+		e->y = (win_height - e->height > 0) ? NRAND(win_height - e->height) : 0;
 
 		/* check for overlap with other eyes */
 		for (i = 0; i < num_eyes; i++) {
@@ -646,12 +652,12 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 		     W_MIN_X, W_MAX_X, W_MIN_Y, W_MAX_Y);
 
 	/* clear the offscreen pixmap to background color */
-	XSetForeground(display, eyegc, black_pixel);
-	XFillRectangle(display, e->pixmap, eyegc, 0, 0, e->width, e->height);
+	XSetForeground(display, ep->eyeGC, black_pixel);
+	XFillRectangle(display, e->pixmap, ep->eyeGC, 0, 0, e->width, e->height);
 
 	/* make the full eye images in the offscreen Pixmap */
-	make_eye(display, e->pixmap, e, 0, True);
-	make_eye(display, e->pixmap, e, 1, True);
+	make_eye(mi, e->pixmap, e, 0, True);
+	make_eye(mi, e->pixmap, e, 1, True);
 }
 
 
@@ -664,9 +670,10 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 static void
 paint_eyes(ModeInfo * mi, Eyes * e, Fly * f, Eyes * eyes, int num_eyes)
 {
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	GC          gc = eyegc;
+	GC          gc = ep->eyeGC;
 	Bool        iconic = MI_WIN_IS_ICONIC(mi);
 	long        now = seconds();
 	int         focusx = (f->x + (f->width / 2)) - e->x;
@@ -721,7 +728,7 @@ paint_eyes(ModeInfo * mi, Eyes * e, Fly * f, Eyes * eyes, int num_eyes)
 	}
 	for (i = 0; i < 2; i++) {
 		/* update the eye, calculates the changed rectangle */
-		make_eye(display, pix, e, i, False);
+		make_eye(mi, pix, e, i, False);
 
 		/* Only blit the change if the full image has been painted */
 		if (e->painted) {
@@ -758,54 +765,31 @@ paint_eyes(ModeInfo * mi, Eyes * e, Fly * f, Eyes * eyes, int num_eyes)
 static void
 one_time_init(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
-	int         n = MI_NUM_SCREENS(mi);
-	int         i;
-
 	if (stuff_alloced) {
 		return;		/* only once, dude */
 	}
-	if (flypix == None) {
-		flypix = XCreateBitmapFromData(display, window,
-				   (char *) FLY_BITS, FLY_WIDTH, FLY_HEIGHT);
-		if (flypix == None) {
-			return;
-		}
-	}
-	if (flygc == None) {
-		if ((flygc = XCreateGC(display, window,
-			   (unsigned long) 0, (XGCValues *) NULL)) == None) {
-			return;
-		}
-		XSetStipple(display, flygc, flypix);
-		XSetFillStyle(display, flygc, FillStippled);
-	}
-	if (eyegc == None) {
-		if ((eyegc = XCreateGC(display, window,
-			   (unsigned long) 0, (XGCValues *) NULL)) == None) {
-			return;
-		}
-	}
 	if (eye_info == NULL) {
-		if ((eye_info = (EyeScrInfo *) calloc(n, sizeof (EyeScrInfo))) == NULL) {
+		if ((eye_info = (EyeScrInfo *) calloc(MI_NUM_SCREENS(mi),
+					       sizeof (EyeScrInfo))) == NULL)
 			return;
-		}
-		for (i = 0; i < n; i++) {
-			int         j;
-
-			(void) memset((char *) &eye_info[i], 0, sizeof (EyeScrInfo));
-
-			for (j = 0; j < MAX_EYES; j++) {
-				/* don't assume None == 0 */
-				eye_info[i].eyes[j].pixmap = None;
-			}
-
-		}
 	}
 	stuff_alloced = True;
 }
 
+static void
+free_eyes(Display * display, EyeScrInfo * ep)
+{
+	int         en;
+
+	if (ep->eyes) {
+		for (en = 0; en < ep->num_eyes; en++)
+			if (ep->eyes[en].pixmap != None)
+				XFreePixmap(display, ep->eyes[en].pixmap);
+		(void) free((void *) ep->eyes);
+		ep->eyes = NULL;
+	}
+}
+
 /* 
  *    Initialize them eyes.  Called each time the window changes.
  */
@@ -814,8 +798,8 @@ void
 init_eyes(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
-	int         screen = MI_SCREEN(mi);
-	int         num_eyes;
+	Window      window = MI_WINDOW(mi);
+	EyeScrInfo *ep;
 	int         i;
 
 	one_time_init(mi);
@@ -824,26 +808,57 @@ init_eyes(ModeInfo * mi)
 		/* if an alloc failed in one_time_init(), quit */
 		return;
 	}
+	ep = &eye_info[MI_SCREEN(mi)];
+
+	if (ep->flypix == None) {
+		ep->flypix = XCreateBitmapFromData(display, window,
+				   (char *) FLY_BITS, FLY_WIDTH, FLY_HEIGHT);
+		if (ep->flypix == None) {
+			return;
+		}
+	}
+	if (ep->flyGC == None) {
+		if ((ep->flyGC = XCreateGC(display, window,
+			   (unsigned long) 0, (XGCValues *) NULL)) == None) {
+			return;
+		}
+		XSetStipple(display, ep->flyGC, ep->flypix);
+		XSetFillStyle(display, ep->flyGC, FillStippled);
+	}
+	if (ep->eyeGC == None) {
+		if ((ep->eyeGC = XCreateGC(display, window,
+			   (unsigned long) 0, (XGCValues *) NULL)) == None) {
+			return;
+		}
+	}
 	/* don't want any exposure events from XCopyArea */
-	XSetGraphicsExposures(display, eyegc, False);
+	XSetGraphicsExposures(display, ep->eyeGC, False);
 
-	if (MI_WIN_IS_ICONIC(mi)) {
-		num_eyes = 1;
-	} else {
-		num_eyes = MI_BATCHCOUNT(mi);
-		if (num_eyes < 1)
-			num_eyes = 1;
-		if (num_eyes > MAX_EYES)
-			num_eyes = MAX_EYES;
+	free_eyes(display, ep);
+	if (MI_WIN_IS_ICONIC(mi))
+		ep->num_eyes = 1;
+	else {
+		ep->num_eyes = MI_BATCHCOUNT(mi);
+		/* MAX_EYES is used or one may quickly run out of memory */
+		if (ep->num_eyes > MAX_EYES)
+			ep->num_eyes = MAX_EYES;
+		if (ep->num_eyes < 0) {
+			if (ep->num_eyes < -MAX_EYES)
+				ep->num_eyes = NRAND(MAX_EYES) + 1;
+			else
+				ep->num_eyes = NRAND(-ep->num_eyes) + 1;	/* Add 1 so its not too boring */
+		}
 	}
-	eye_info[screen].num_eyes = num_eyes;
+	if (!ep->eyes)
+		ep->eyes = (Eyes *) calloc(ep->num_eyes, sizeof (Eyes));
 
-	for (i = 0; i < num_eyes; i++) {	/* place each eye pair */
-		create_eyes(mi, &(eye_info[screen].eyes[i]),
-			    eye_info[screen].eyes, num_eyes);
+	for (i = 0; i < ep->num_eyes; i++) {	/* place each eye pair */
+		/* don't assume None == 0 */
+		ep->eyes[i].pixmap = None;
+		create_eyes(mi, &(ep->eyes[i]), ep->eyes, ep->num_eyes);
 	}
 
-	init_fly(mi, &eye_info[screen].fly);	/* init the bouncer */
+	init_fly(mi, &(ep->fly));	/* init the bouncer */
 
 	XClearWindow(display, MI_WINDOW(mi));	/* clear the window */
 }
@@ -889,37 +904,23 @@ release_eyes(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 
-	if (flypix != None) {
-		XFreePixmap(display, flypix);
-		flypix = None;
-	}
-	if (flygc != None) {
-		XFreeGC(display, flygc);
-		flygc = None;
-	}
-	if (eyegc != None) {
-		XFreeGC(display, eyegc);
-		eyegc = None;
-	}
 	if (eye_info != NULL) {
 		int         screen;
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			Eyes       *eyes;
-			int         j;
+			EyeScrInfo *ep = &eye_info[screen];
 
-			eyes = eye_info[screen].eyes;
-
-			for (j = 0; j < MAX_EYES; j++) {
-				if (eyes[j].pixmap != None) {
-					XFreePixmap(display, eyes[j].pixmap);
-				}
-			}
+			if (ep->flyGC != None)
+				XFreeGC(display, ep->flyGC);
+			if (ep->eyeGC != None)
+				XFreeGC(display, ep->eyeGC);
+			if (ep->flypix != None)
+				XFreePixmap(display, ep->flypix);
+			free_eyes(display, ep);
 		}
-
 		(void) free((void *) eye_info);
-		eye_info = NULL;
 	}
+	eye_info = NULL;
 	stuff_alloced = False;
 }
 
