@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)galaxy.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)galaxy.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -35,9 +36,8 @@ static char sccsid[] = "@(#)galaxy.c	3.9 96/05/25 xlockmore";
 #define BOUNCE     1		/* Bounce from borders */
 #endif
 
-#define MAX_GALAXIES    5
+#define MINGALAXIES    1
 #define MAX_STARS    300
-#define MAX_HITITERATIONS  200
 #define MAX_IDELTAT    50
 /* These come originally from the Cluster-version */
 #define DEFAULT_GALAXIES  2
@@ -55,14 +55,14 @@ static char sccsid[] = "@(#)galaxy.c	3.9 96/05/25 xlockmore";
 
 #if (STARSIZE > 1)
 #define MyXDrawPoint(x,y) \
-            XFillArc(dsp,win,Scr[screen].gc,x,y,gp->starsize,gp->starsize,0,23040)
+            XFillArc(display,window,gc,x,y,gp->starsize,gp->starsize,0,23040)
 #else
 #define MyXDrawPoint(x,y) \
-            XDrawPoint(dsp,win,Scr[screen].gc,x,y)
+            XDrawPoint(display,window,gc,x,y)
 #endif /*STARSIZE */
 
 ModeSpecOpt galaxy_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef double Vector[3];
 typedef double Matrix[3][3];
@@ -78,6 +78,7 @@ typedef struct {
 	Star       *stars;
 	int         basecolor;
 	Vector      pos, vel;
+	int         galcol;
 } Galaxy;
 
 typedef struct {
@@ -87,14 +88,13 @@ typedef struct {
 		int         top;	/* y minimum */
 		int         bottom;	/* y maximum */
 	} clip;
-	int         galcol[MAX_GALAXIES];	/* colors */
 	Matrix      mat;	/* Movement of stars(?) */
 	double      scale;	/* Scale */
 	int         midx;	/* Middle of screen, x */
 	int         midy;	/* Middle of screen, y */
 	double      size;	/* */
 	Vector      diff;	/* */
-	Galaxy      galaxies[MAX_GALAXIES];	/* the Whole Universe */
+	Galaxy     *galaxies;	/* the Whole Universe */
 	double      f_deltat;	/* quality of calculation, calc'd by d_ideltat */
 	int         f_galaxies;	/* # galaxies */
 	int         f_stars;	/* # stars per galaxy */
@@ -106,21 +106,45 @@ typedef struct {
 #endif
 } unistruct;
 
-static unistruct universes[MAXSCREENS];
+static unistruct *universes = NULL;
+
+static void
+free_galaxies(unistruct * gp)
+{
+	int         gn;
+
+	if (gp->galaxies) {
+		for (gn = 0; gn < gp->f_galaxies; gn++)
+			if (gp->galaxies[gn].stars)
+				(void) free((void *) gp->galaxies[gn].stars);
+		(void) free((void *) gp->galaxies);
+		gp->galaxies = NULL;
+	}
+}
 
 void
 init_galaxy(ModeInfo * mi)
 {
-	unistruct  *gp = &universes[screen];
+	unistruct  *gp = &universes[MI_SCREEN(mi)];
 	int         i;
 
+	if (universes == NULL) {
+		if ((universes = (unistruct *) calloc(MI_NUM_SCREENS(mi),
+						sizeof (unistruct))) == NULL)
+			return;
+	}
+	gp = &universes[MI_SCREEN(mi)];
+
 	gp->f_galaxies = MI_BATCHCOUNT(mi);
-	if (gp->f_galaxies < 1)
-		gp->f_galaxies = 1;
-	else if (gp->f_galaxies > MAX_GALAXIES)
-		gp->f_galaxies = MAX_GALAXIES;
+	if (gp->f_galaxies < -MINGALAXIES) {
+		free_galaxies(gp);
+		gp->f_galaxies = NRAND(-gp->f_galaxies - MINGALAXIES + 1) + MINGALAXIES;
+	} else if (gp->f_galaxies < MINGALAXIES)
+		gp->f_galaxies = MINGALAXIES;
+	if (!gp->galaxies)
+		gp->galaxies = (Galaxy *) calloc(gp->f_galaxies, sizeof (Galaxy));
 	gp->f_stars = MAX_STARS;
-	gp->f_hititerations = MAX_HITITERATIONS;
+	gp->f_hititerations = MI_CYCLES(mi);
 	gp->f_deltat = ((double) MAX_IDELTAT) / 10000.0;
 
 	gp->clip.left = 0;
@@ -136,19 +160,20 @@ init_galaxy(ModeInfo * mi)
 	gp->starsize = STARSIZE;
 #endif
 
-	if (!gp->galaxies[0].stars)
-		for (i = 0; i < MAX_GALAXIES; ++i) {
+	for (i = 0; i < gp->f_galaxies; ++i)
+		if (!gp->galaxies[i].stars) {
 			gp->galaxies[i].starscnt = 0;	/* 0 valid entries */
 			gp->galaxies[i].stars = (Star *) malloc(gp->f_stars * sizeof (Star));
 		}
 }
 
-
 void
 draw_galaxy(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	unistruct  *gp = &universes[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	unistruct  *gp = &universes[MI_SCREEN(mi)];
 	double      d;		/* tmp */
 	int         i, j, k;	/* more tmp */
 
@@ -159,13 +184,13 @@ draw_galaxy(ModeInfo * mi)
 		gp->init = 0;
 		gp->step = 0;
 
-		for (i = 0; i < MAX_GALAXIES; ++i)
-			gp->galcol[i] = LRAND() % COLORBASE;
+		for (i = 0; i < gp->f_galaxies; ++i)
+			gp->galaxies[i].galcol = NRAND(COLORBASE);
 
 		for (i = 0; i < gp->f_galaxies; ++i) {
-			gp->galaxies[i].basecolor = gp->galcol[i];
+			gp->galaxies[i].basecolor = gp->galaxies[i].galcol;
 
-			gp->galaxies[i].starscnt = (LRAND() % (gp->f_stars / 2)) + gp->f_stars / 2;
+			gp->galaxies[i].starscnt = (NRAND(gp->f_stars / 2)) + gp->f_stars / 2;
 
 			w1 = 2.0 * M_PI * FLOATRAND;
 			w2 = 2.0 * M_PI * FLOATRAND;
@@ -190,7 +215,7 @@ draw_galaxy(ModeInfo * mi)
 			gp->galaxies[i].pos[2] = -gp->galaxies[i].vel[2] * gp->f_deltat *
 				gp->f_hititerations + FLOATRAND - 0.5;
 
-			gp->galaxies[i].mass = FLOATRAND * 1000.0 + 1.0;
+			gp->galaxies[i].mass = (int) (FLOATRAND * 1000.0) + 1;
 
 			/*w3=FLOATRAND; */
 			w3 = 0.0;
@@ -230,9 +255,7 @@ draw_galaxy(ModeInfo * mi)
 			}
 		}
 
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-		XFillRectangle(dsp, win, Scr[screen].gc,
-		gp->clip.left, gp->clip.top, gp->clip.right, gp->clip.bottom);
+		XClearWindow(display, window);
 
 #if 0
 		(void) printf("f_galaxies=%d, f_stars=%d, f_hititerations=%d\n",
@@ -271,7 +294,7 @@ draw_galaxy(ModeInfo * mi)
 			    gp->galaxies[i].stars[j].px <= gp->clip.right - STARSIZE &&
 			    gp->galaxies[i].stars[j].py >= gp->clip.top &&
 			    gp->galaxies[i].stars[j].py <= gp->clip.bottom - STARSIZE) {
-				XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
+				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 				MyXDrawPoint(gp->galaxies[i].stars[j].px, gp->galaxies[i].stars[j].py);
 			}
 			gp->galaxies[i].stars[j].px = (int) (gp->galaxies[i].stars[j].pos[0] *
@@ -304,11 +327,11 @@ draw_galaxy(ModeInfo * mi)
 			    gp->galaxies[i].stars[j].px <= gp->clip.right - STARSIZE &&
 			    gp->galaxies[i].stars[j].py >= gp->clip.top &&
 			    gp->galaxies[i].stars[j].py <= gp->clip.bottom - STARSIZE) {
-				if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-					XSetForeground(dsp, Scr[screen].gc,
-						       Scr[screen].pixels[gp->galaxies[i].stars[j].color]);
+				if (MI_NPIXELS(mi) > 2)
+					XSetForeground(display, gc,
+						       MI_PIXEL(mi, gp->galaxies[i].stars[j].color));
 				else
-					XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+					XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 				MyXDrawPoint(gp->galaxies[i].stars[j].px,
 					     gp->galaxies[i].stars[j].py);
 			}
@@ -340,3 +363,22 @@ draw_galaxy(ModeInfo * mi)
 	if (gp->step > gp->f_hititerations * 4)
 		gp->init = 1;
 }
+
+void
+release_galaxy(ModeInfo * mi)
+{
+	if (universes != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_galaxies(&universes[screen]);
+		(void) free((void *) universes);
+		universes = NULL;
+	}
+}
+
+void
+refresh_galaxy(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
