@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)grav.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)grav.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -55,9 +55,9 @@ static char sccsid[] = "@(#)grav.c	3.9 96/05/25 xlockmore";
 #define Planet(x,y)\
   if ((x) >= 0 && (y) >= 0 && (x) <= gp->width && (y) <= gp->height) {\
     if (planet->ri < 2)\
-     XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), (x), (y));\
+     XDrawPoint(display, window, gc, (x), (y));\
     else\
-     XFillArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),\
+     XFillArc(display, window, gc,\
       (x) - planet->ri / 2, (y) - planet->ri / 2, planet->ri, planet->ri,\
       0, 23040);\
    }
@@ -65,7 +65,7 @@ static char sccsid[] = "@(#)grav.c	3.9 96/05/25 xlockmore";
 #define FLOATRAND(min,max)	((min)+(LRAND()/MAXRAND)*((max)-(min)))
 
 ModeSpecOpt grav_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	double
@@ -77,61 +77,25 @@ typedef struct {
 
 typedef struct {
 	int         width, height;
-	int
-	            x, y, sr, nplanets;
+	int         x, y, sr, nplanets;
 	unsigned long starcolor;
 	planetstruct *planets;
 } gravstruct;
 
-static gravstruct gravs[MAXSCREENS];
-
-static void init_planet(ModeInfo * mi, planetstruct * planet);
-static void draw_planet(ModeInfo * mi, planetstruct * planet);
-
-void
-init_grav(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	gravstruct *gp = &gravs[screen];
-	unsigned char ball;
-
-	gp->width = MI_WIN_WIDTH(mi);
-	gp->height = MI_WIN_HEIGHT(mi);
-
-	gp->sr = STARRADIUS;
-
-	gp->nplanets = MI_BATCHCOUNT(mi);
-	if (gp->nplanets < 0)
-		gp->nplanets = 1;
-	else if (gp->nplanets > 20)
-		gp->nplanets = 10;
-
-	if (!gp->planets)
-		gp->planets = (planetstruct *) calloc(gp->nplanets, sizeof (planetstruct));
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, gp->width, gp->height);
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		gp->starcolor = Scr[screen].pixels[LRAND() % Scr[screen].npixels];
-	else
-		gp->starcolor = WhitePixel(dsp, screen);
-	for (ball = 0; ball < (unsigned char) gp->nplanets; ball++)
-		init_planet(mi, &gp->planets[ball]);
-
-	/* Draw centrepoint */
-	XDrawArc(dsp, win, Scr[screen].gc,
-		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
-		 0, 23040);
-}
+static gravstruct *gravs = NULL;
 
 static void
 init_planet(ModeInfo * mi, planetstruct * planet)
 {
-	gravstruct *gp = &gravs[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	gravstruct *gp = &gravs[MI_SCREEN(mi)];
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		planet->colors = Scr[screen].pixels[LRAND() % Scr[screen].npixels];
+	if (MI_NPIXELS(mi) > 2)
+		planet->colors = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	else
-		planet->colors = WhitePixel(dsp, screen);
+		planet->colors = MI_WIN_WHITE_PIXEL(mi);
 	/* Initialize positions */
 	POS(X) = FLOATRAND(-XR, XR);
 	POS(Y) = FLOATRAND(-YR, YR);
@@ -155,44 +119,13 @@ init_planet(ModeInfo * mi, planetstruct * planet)
 	Planet(planet->xi, planet->yi);
 }
 
-void
-draw_grav(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	gravstruct *gp = &gravs[screen];
-	register unsigned char ball;
-
-	/* Mask centrepoint */
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XDrawArc(dsp, win, Scr[screen].gc,
-		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
-		 0, 23040);
-
-	/* Resize centrepoint */
-	switch (LRAND() % 4) {
-		case 0:
-			if (gp->sr < STARRADIUS)
-				gp->sr++;
-			break;
-		case 1:
-			if (gp->sr > 2)
-				gp->sr--;
-	}
-
-	/* Draw centrepoint */
-	XSetForeground(dsp, Scr[screen].gc, gp->starcolor);
-	XDrawArc(dsp, win, Scr[screen].gc,
-		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
-		 0, 23040);
-
-	for (ball = 0; ball < (unsigned char) gp->nplanets; ball++)
-		draw_planet(mi, &gp->planets[ball]);
-}
-
 static void
 draw_planet(ModeInfo * mi, planetstruct * planet)
 {
-	gravstruct *gp = &gravs[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	gravstruct *gp = &gravs[MI_SCREEN(mi)];
 	double      D;		/* A distance variable to work with */
 	register unsigned char cmpt;
 
@@ -230,11 +163,11 @@ draw_planet(ModeInfo * mi, planetstruct * planet)
 		planet->xi = planet->yi = -1;
 
 	/* Mask */
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
+	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 	Planet(gp->x, gp->y);
 #ifdef TRAIL
-	XSetForeground(dsp, Scr[screen].gc, planet->colors);
-	XDrawPoint(dsp, win, Scr[screen].gc, gp->x, gp->y);
+	XSetForeground(display, gc, planet->colors);
+	XDrawPoint(display, MI_WINDOW(mi), gc, gp->x, gp->y);
 #endif
 	/* Move */
 	gp->x = planet->xi;
@@ -242,6 +175,111 @@ draw_planet(ModeInfo * mi, planetstruct * planet)
 	planet->ri = RADIUS;
 
 	/* Redraw */
-	XSetForeground(dsp, Scr[screen].gc, planet->colors);
+	XSetForeground(display, gc, planet->colors);
 	Planet(gp->x, gp->y);
 }
+
+void
+init_grav(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	gravstruct *gp;
+	unsigned char ball;
+
+	if (gravs == NULL) {
+		if ((gravs = (gravstruct *) calloc(MI_NUM_SCREENS(mi),
+					       sizeof (gravstruct))) == NULL)
+			return;
+	}
+	gp = &gravs[MI_SCREEN(mi)];
+
+	gp->width = MI_WIN_WIDTH(mi);
+	gp->height = MI_WIN_HEIGHT(mi);
+
+	gp->sr = STARRADIUS;
+
+	gp->nplanets = MI_BATCHCOUNT(mi);
+	if (gp->nplanets < 0) {
+		if (gp->planets) {
+			(void) free((void *) gp->planets);
+			gp->planets = NULL;
+		}
+		gp->nplanets = NRAND(-gp->nplanets) + 1;	/* Add 1 so its not too boring */
+	}
+	if (!gp->planets)
+		gp->planets = (planetstruct *) calloc(gp->nplanets, sizeof (planetstruct));
+
+	XClearWindow(display, MI_WINDOW(mi));
+
+	if (MI_NPIXELS(mi) > 2)
+		gp->starcolor = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+	else
+		gp->starcolor = MI_WIN_WHITE_PIXEL(mi);
+	for (ball = 0; ball < (unsigned char) gp->nplanets; ball++)
+		init_planet(mi, &gp->planets[ball]);
+
+	/* Draw centrepoint */
+	XDrawArc(display, MI_WINDOW(mi), gc,
+		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
+		 0, 23040);
+}
+
+void
+draw_grav(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	gravstruct *gp = &gravs[MI_SCREEN(mi)];
+	register unsigned char ball;
+
+	/* Mask centrepoint */
+	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XDrawArc(display, window, gc,
+		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
+		 0, 23040);
+
+	/* Resize centrepoint */
+	switch (NRAND(4)) {
+		case 0:
+			if (gp->sr < (int) STARRADIUS)
+				gp->sr++;
+			break;
+		case 1:
+			if (gp->sr > 2)
+				gp->sr--;
+	}
+
+	/* Draw centrepoint */
+	XSetForeground(display, gc, gp->starcolor);
+	XDrawArc(display, window, gc,
+		 gp->width / 2 - gp->sr / 2, gp->height / 2 - gp->sr / 2, gp->sr, gp->sr,
+		 0, 23040);
+
+	for (ball = 0; ball < (unsigned char) gp->nplanets; ball++)
+		draw_planet(mi, &gp->planets[ball]);
+}
+
+void
+release_grav(ModeInfo * mi)
+{
+	if (gravs != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			gravstruct *gp = &gravs[screen];
+
+			if (gp->planets)
+				(void) free((void *) gp->planets);
+		}
+		(void) free((void *) gravs);
+		gravs = NULL;
+	}
+}
+
+void
+refresh_grav(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
