@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)helix.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)helix.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -31,7 +32,7 @@ static char sccsid[] = "@(#)helix.c	3.9 96/05/25 xlockmore";
 #define ANGLES 360
 
 ModeSpecOpt helix_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 static double cos_array[ANGLES], sin_array[ANGLES];
 typedef struct {
@@ -40,39 +41,12 @@ typedef struct {
 	int         xmid, ymid;
 	int         color;
 	int         time;
+	int         radius1, radius2, d_angle, factor1, factor2, factor3,
+	            factor4;
+	int         redraw;
 } helixstruct;
 
-static helixstruct helixes[MAXSCREENS];
-
-static void random_helix(ModeInfo * mi);
-
-void
-init_helix(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	helixstruct *hp = &helixes[screen];
-	int         i;
-	static int  first = 1;
-
-	hp->width = MI_WIN_WIDTH(mi);
-	hp->height = MI_WIN_HEIGHT(mi);
-	hp->xmid = hp->width / 2;
-	hp->ymid = hp->height / 2;
-
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, hp->width, hp->height);
-
-	if (first) {
-		first = 0;
-		for (i = 0; i < ANGLES; i++) {
-			cos_array[i] = cos((((double) i) / (double) (ANGLES / 2)) * M_PI);
-			sin_array[i] = sin((((double) i) / (double) (ANGLES / 2)) * M_PI);;
-		}
-	}
-	hp->color = 0;
-	hp->time = 0;
-	random_helix(mi);
-}
+static helixstruct *helixes = NULL;
 
 static int
 gcd(int a, int b)
@@ -88,10 +62,13 @@ gcd(int a, int b)
 }
 
 static void
-helix(ModeInfo * mi, int radius1, int radius2, int d_angle, int factor1, int factor2, int factor3, int factor4)
+helix(ModeInfo * mi, int radius1, int radius2, int d_angle,
+      int factor1, int factor2, int factor3, int factor4)
 {
+	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	helixstruct *hp = &helixes[screen];
+	GC          gc = MI_GC(mi);
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
 	int         x_1, y_1, x_2, y_2, angle, limit;
 	int         i;
 
@@ -106,26 +83,26 @@ helix(ModeInfo * mi, int radius1, int radius2, int d_angle, int factor1, int fac
 		int         tmp;
 
 #define pmod(x,y) (tmp=(x%y),(tmp>=0?tmp:tmp+y))
-		x_1 = hp->xmid + (((double) radius1) *
+		x_1 = hp->xmid + (int) (((double) radius1) *
 				  sin_array[pmod((angle * factor1), ANGLES)]);
-		y_1 = hp->ymid + (((double) radius2) *
+		y_1 = hp->ymid + (int) (((double) radius2) *
 				  cos_array[pmod((angle * factor2), ANGLES)]);
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[hp->color]);
-			if (++hp->color >= Scr[screen].npixels)
+		if (MI_NPIXELS(mi) > 2) {
+			XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
+			if (++hp->color >= MI_NPIXELS(mi))
 				hp->color = 0;
 		}
-		XDrawLine(dsp, window, Scr[screen].gc, x_1, y_1, x_2, y_2);
-		x_2 = hp->xmid + (((double) radius2) *
+		XDrawLine(display, window, gc, x_1, y_1, x_2, y_2);
+		x_2 = hp->xmid + (int) (((double) radius2) *
 				  sin_array[pmod((angle * factor3), ANGLES)]);
-		y_2 = hp->ymid + (((double) radius1) *
+		y_2 = hp->ymid + (int) (((double) radius1) *
 				  cos_array[pmod((angle * factor4), ANGLES)]);
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[hp->color]);
-			if (++hp->color >= Scr[screen].npixels)
+		if (MI_NPIXELS(mi) > 2) {
+			XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
+			if (++hp->color >= MI_NPIXELS(mi))
 				hp->color = 0;
 		}
-		XDrawLine(dsp, window, Scr[screen].gc, x_1, y_1, x_2, y_2);
+		XDrawLine(display, window, gc, x_1, y_1, x_2, y_2);
 		angle += d_angle;
 	}
 }
@@ -133,58 +110,114 @@ helix(ModeInfo * mi, int radius1, int radius2, int d_angle, int factor1, int fac
 static void
 random_helix(ModeInfo * mi)
 {
-	helixstruct *hp = &helixes[screen];
-	int         radius, radius1, radius2, d_angle, factor1, factor2,
-	            factor3, factor4;
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
+	int         radius;
 	double      divisor;
 
-	radius = min(hp->xmid, hp->ymid);
+	radius = MIN(hp->xmid, hp->ymid);
 
-	d_angle = 0;
-	factor1 = 2;
-	factor2 = 2;
-	factor3 = 2;
-	factor4 = 2;
+	hp->d_angle = 0;
+	hp->factor1 = 2;
+	hp->factor2 = 2;
+	hp->factor3 = 2;
+	hp->factor4 = 2;
 
 	divisor = ((LRAND() / MAXRAND * 3.0 + 1) * (((LRAND() & 1) * 2) - 1));
 
 	if ((LRAND() & 1) == 0) {
-		radius1 = radius;
-		radius2 = radius / divisor;
+		hp->radius1 = radius;
+		hp->radius2 = (int) ((double) radius / divisor);
 	} else {
-		radius2 = radius;
-		radius1 = radius / divisor;
+		hp->radius2 = radius;
+		hp->radius1 = (int) ((double) radius / divisor);
 	}
 
-	while (gcd(ANGLES, d_angle) >= 2)
-		d_angle = LRAND() % ANGLES;
+	while (gcd(ANGLES, hp->d_angle) >= 2)
+		hp->d_angle = NRAND(ANGLES);
 
 #define random_factor()				\
-  (((LRAND() % 7) ? ((LRAND() & 1) + 1) : 3)	\
-   * (((LRAND() & 1) * 2) - 1))
+  (((NRAND(7)) ? ((LRAND() & 1) + 1) : 3)	* (((LRAND() & 1) * 2) - 1))
 
-	while (gcd(gcd(gcd(factor1, factor2), factor3), factor4) != 1) {
-		factor1 = random_factor();
-		factor2 = random_factor();
-		factor3 = random_factor();
-		factor4 = random_factor();
+	while (gcd(gcd(gcd(hp->factor1, hp->factor2), hp->factor3), hp->factor4)
+	       != 1) {
+		hp->factor1 = random_factor();
+		hp->factor2 = random_factor();
+		hp->factor3 = random_factor();
+		hp->factor4 = random_factor();
 	}
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[hp->color]);
-		if (++hp->color >= Scr[screen].npixels)
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
+		if (++hp->color >= MI_NPIXELS(mi))
 			hp->color = 0;
 	} else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 
-	helix(mi, radius1, radius2, d_angle,
-	      factor1, factor2, factor3, factor4);
+	helix(mi, hp->radius1, hp->radius2, hp->d_angle,
+	      hp->factor1, hp->factor2, hp->factor3, hp->factor4);
+}
+
+void
+init_helix(ModeInfo * mi)
+{
+	helixstruct *hp;
+	int         i;
+	static int  first = 1;
+
+	if (helixes == NULL) {
+		if ((helixes = (helixstruct *) calloc(MI_NUM_SCREENS(mi),
+					      sizeof (helixstruct))) == NULL)
+			return;
+	}
+	hp = &helixes[MI_SCREEN(mi)];
+
+	hp->width = MI_WIN_WIDTH(mi);
+	hp->height = MI_WIN_HEIGHT(mi);
+	hp->xmid = hp->width / 2;
+	hp->ymid = hp->height / 2;
+	hp->redraw = 0;
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	if (first) {
+		first = 0;
+		for (i = 0; i < ANGLES; i++) {
+			cos_array[i] = cos((((double) i) / (double) (ANGLES / 2)) * M_PI);
+			sin_array[i] = sin((((double) i) / (double) (ANGLES / 2)) * M_PI);;
+		}
+	}
+	hp->color = 0;
+	hp->time = 0;
+	random_helix(mi);
 }
 
 void
 draw_helix(ModeInfo * mi)
 {
-	helixstruct *hp = &helixes[screen];
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
 
 	if (++hp->time > MI_CYCLES(mi))
 		init_helix(mi);
+	if (hp->redraw) {
+		helix(mi, hp->radius1, hp->radius2, hp->d_angle,
+		      hp->factor1, hp->factor2, hp->factor3, hp->factor4);
+		hp->redraw = 0;
+	}
+}
+
+void
+release_helix(ModeInfo * mi)
+{
+	if (helixes != NULL) {
+		(void) free((void *) helixes);
+		helixes = NULL;
+	}
+}
+
+void
+refresh_helix(ModeInfo * mi)
+{
+	helixstruct *hp = &helixes[MI_SCREEN(mi)];
+
+	hp->redraw = 1;
 }
