@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)hop.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)hop.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -41,7 +42,7 @@ static char sccsid[] = "@(#)hop.c	3.9 96/05/25 xlockmore";
 #define OPS 3
 
 ModeSpecOpt hop_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         centerx;
@@ -59,28 +60,42 @@ typedef struct {
 	int         bufsize;
 } hopstruct;
 
-static hopstruct hops[MAXSCREENS];
+static hopstruct *hops = NULL;
 static XPoint *pointBuffer = 0;	/* pointer for XDrawPoints */
 
 void
 init_hop(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	hopstruct  *hp;
 	double      range;
-	hopstruct  *hp = &hops[screen];
+
+	if (hops == NULL) {
+		if ((hops = (hopstruct *) calloc(MI_NUM_SCREENS(mi),
+						 sizeof (hopstruct))) == NULL)
+			return;
+	}
+	hp = &hops[MI_SCREEN(mi)];
 
 	hp->centerx = MI_WIN_WIDTH(mi) / 2;
 	hp->centery = MI_WIN_HEIGHT(mi) / 2;
 	/* Make the other operations less common since they are less interesting */
-	if (LRAND() % 6)
+	if (NRAND(6))
 		hp->op = SQRT;
 	else
-		hp->op = LRAND() % (OPS - 1) + 1;
+#ifdef OFFENDING
+		/* Sine hop may be too close to a Nazi cross to suit some.
+		   Compile with -DOFFENDING to weed this out.
+		 */
+		hp->op = NRAND(OPS - 2) + 2;
+#else
+		hp->op = NRAND(OPS - 1) + 1;
+#endif
 	switch (hp->op) {
 		case SQRT:
 			range = sqrt((double) hp->centerx * hp->centerx +
-				     (double) hp->centery * hp->centery) /
-				(10.0 + LRAND() % 10);
+				     (double) hp->centery * hp->centery) / (10.0 + NRAND(10));
 
 			hp->a = (LRAND() / MAXRAND) * range - range / 2.0;
 			hp->b = (LRAND() / MAXRAND) * range - range / 2.0;
@@ -106,10 +121,9 @@ init_hop(ModeInfo * mi)
 	if (!pointBuffer)
 		pointBuffer = (XPoint *) malloc(hp->bufsize * sizeof (XPoint));
 
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0,
-		       hp->centerx * 2, hp->centery * 2);
-	XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+	XClearWindow(display, MI_WINDOW(mi));
+
+	XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 	hp->count = 0;
 }
 
@@ -117,16 +131,15 @@ init_hop(ModeInfo * mi)
 void
 draw_hop(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	hopstruct  *hp = &hops[MI_SCREEN(mi)];
 	double      oldj, oldi;
 	XPoint     *xp = pointBuffer;
-	hopstruct  *hp = &hops[screen];
 	int         k = hp->bufsize;
 
 	hp->inc++;
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[hp->pix]);
-		if (++hp->pix >= Scr[screen].npixels)
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, hp->pix));
+		if (++hp->pix >= MI_NPIXELS(mi))
 			hp->pix = 0;
 	}
 	while (k--) {
@@ -152,14 +165,33 @@ draw_hop(ModeInfo * mi)
 				oldi = hp->i + 4 * hp->inc / hp->centerx;
 				hp->j = sin(hp->c * hp->i) - cos(hp->d * hp->j);
 				hp->i = sin(hp->a * oldj) - cos(hp->b * oldi);
-				xp->x = hp->centerx + (int) hp->centerx * (hp->i + hp->j) / 4;
-				xp->y = hp->centery - (int) hp->centery * (hp->i - hp->j) / 4;
+				xp->x = hp->centerx + (int) (hp->centerx * (hp->i + hp->j) / 4.0);
+				xp->y = hp->centery - (int) (hp->centery * (hp->i - hp->j) / 4.0);
 				break;
 		}
 		xp++;
 	}
-	XDrawPoints(dsp, win, Scr[screen].gc,
+	XDrawPoints(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 		    pointBuffer, hp->bufsize, CoordModeOrigin);
 	if (++hp->count > MI_CYCLES(mi))
 		init_hop(mi);
 }
+
+void
+release_hop(ModeInfo * mi)
+{
+	if (hops != NULL) {
+		(void) free((void *) hops);
+		hops = NULL;
+	}
+	if (pointBuffer) {
+		(void) free((void *) pointBuffer);
+		pointBuffer = NULL;
+	}
+}
+
+void
+refresh_hop(ModeInfo * mi)
+{
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
