@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)kaleid.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)kaleid.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -21,10 +22,10 @@ static char sccsid[] = "@(#)kaleid.c	3.9 96/05/25 xlockmore";
 
 #include "xlock.h"
 
-#define INTRAND(min,max) (LRAND()%((max+1)-(min))+(min))
+#define INTRAND(min,max) (NRAND(((max+1)-(min)))+(min))
 
 ModeSpecOpt kaleid_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         x;
@@ -42,56 +43,65 @@ typedef struct {
 	int         itercount;
 } kaleidstruct;
 
-static kaleidstruct kaleids[MAXSCREENS];
+static kaleidstruct *kaleids = NULL;
 
 void
 init_kaleid(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	kaleidstruct *kp = &kaleids[screen];
+	kaleidstruct *kp = &kaleids[MI_SCREEN(mi)];
+
+	if (kaleids == NULL) {
+		if ((kaleids = (kaleidstruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (kaleidstruct))) == NULL)
+			return;
+	}
+	kp = &kaleids[MI_SCREEN(mi)];
 
 	kp->width = MI_WIN_WIDTH(mi);
 	kp->height = MI_WIN_HEIGHT(mi);
-
+	if (kp->width < 2)
+		kp->width = 2;
+	if (kp->height < 2)
+		kp->height = 2;
 	kp->pix = 0;
 	kp->cx = kp->width / 2;
 	kp->cy = kp->height / 2;
 	kp->m = (kp->cx > kp->cy) ? kp->cx : kp->cy;
 	kp->m = kp->m ? kp->m : 1;
-	kp->x1 = LRAND() % kp->m + 1;
-	kp->x2 = LRAND() % kp->m + 1;
-	kp->y1 = LRAND() % kp->x1 + 1;
-	kp->y2 = LRAND() % kp->x2 + 1;
-	kp->xv1 = LRAND() % 7 - 3;
-	kp->xv2 = LRAND() % 7 - 3;
-	kp->yv1 = LRAND() % 7 - 3;
-	kp->yv2 = LRAND() % 7 - 3;
+	kp->x1 = NRAND(kp->m) + 1;
+	kp->x2 = NRAND(kp->m) + 1;
+	kp->y1 = NRAND(kp->x1) + 1;
+	kp->y2 = NRAND(kp->x2) + 1;
+	kp->xv1 = NRAND(7) - 3;
+	kp->xv2 = NRAND(7) - 3;
+	kp->yv1 = NRAND(7) - 3;
+	kp->yv2 = NRAND(7) - 3;
 	kp->itercount = 0;
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, kp->width, kp->height);
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
 
 void
 draw_kaleid(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	kaleidstruct *kp = &kaleids[MI_SCREEN(mi)];
 	XSegment    segs[8];
-	kaleidstruct *kp = &kaleids[screen];
 
-	if (!(LRAND() % 50)) {
-		kp->x1 = LRAND() % kp->m + 1;
-		kp->x2 = LRAND() % kp->m + 1;
-		kp->y1 = LRAND() % kp->x1;
-		kp->y2 = LRAND() % kp->x2;
+	if (!(NRAND(50))) {
+		kp->x1 = NRAND(kp->m) + 1;
+		kp->x2 = NRAND(kp->m) + 1;
+		kp->y1 = NRAND(kp->x1);
+		kp->y2 = NRAND(kp->x2);
 	}
-	if (!(LRAND() % 10)) {
-		kp->xv1 = LRAND() % 7 - 3;
-		kp->xv2 = LRAND() % 7 - 3;
-		kp->yv1 = LRAND() % 7 - 3;
-		kp->yv2 = LRAND() % 7 - 3;
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[kp->pix]);
-			if (++kp->pix >= Scr[screen].npixels)
+	if (!(NRAND(10))) {
+		kp->xv1 = NRAND(7) - 3;
+		kp->xv2 = NRAND(7) - 3;
+		kp->yv1 = NRAND(7) - 3;
+		kp->yv2 = NRAND(7) - 3;
+		if (MI_NPIXELS(mi) > 2) {
+			XSetForeground(display, gc, MI_PIXEL(mi, kp->pix));
+			if (++kp->pix >= MI_NPIXELS(mi))
 				kp->pix = 0;
 		}
 	}
@@ -146,7 +156,7 @@ draw_kaleid(ModeInfo * mi)
 	segs[7].y1 = kp->cy + kp->xb + kp->oy;
 	segs[7].x2 = kp->cx + kp->yc + kp->ox;
 	segs[7].y2 = kp->cy + kp->xd + kp->oy;
-	XDrawSegments(dsp, win, Scr[screen].gc, segs, 8);
+	XDrawSegments(display, MI_WINDOW(mi), gc, segs, 8);
 	kp->x1 = (kp->x1 + kp->xv1) % kp->m;
 	kp->y1 = (kp->y1 + kp->yv1) % kp->m;
 	kp->x2 = (kp->x2 + kp->xv2) % kp->m;
@@ -154,13 +164,28 @@ draw_kaleid(ModeInfo * mi)
 	kp->itercount++;
 
 	if (kp->itercount > MI_CYCLES(mi)) {
-		XClearWindow(dsp, win);
+		XClearWindow(display, MI_WINDOW(mi));
 		kp->itercount = 0;
 	}
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[kp->pix]);
-		if (++kp->pix >= Scr[screen].npixels)
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(display, gc, MI_PIXEL(mi, kp->pix));
+		if (++kp->pix >= MI_NPIXELS(mi))
 			kp->pix = 0;
 	} else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+}
+
+void
+release_kaleid(ModeInfo * mi)
+{
+	if (kaleids != NULL) {
+		(void) free((void *) kaleids);
+		kaleids = NULL;
+	}
+}
+
+void
+refresh_kaleid(ModeInfo * mi)
+{
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
