@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)laser.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)laser.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -22,8 +23,7 @@ static char sccsid[] = "@(#)laser.c	3.9 96/05/25 xlockmore";
 #define MINREDRAW 3		/* Number of redrawn on each frame */
 #define MAXREDRAW 8
 
-#define MINLASER  2		/* Laser number */
-#define MAXLASER  10
+#define MINLASER  1		/* Laser number */
 
 #define MINWIDTH  2		/* Laser ray width range */
 #define MAXWIDTH 40
@@ -35,11 +35,10 @@ static char sccsid[] = "@(#)laser.c	3.9 96/05/25 xlockmore";
 
 #define COLORSTEP 2		/* Laser color step */
 
-#define MAX_RAND(max) (LRAND() % (max))
 #define RANGE_RAND(min,max) ((min) + LRAND() % ((max) - (min)))
 
 ModeSpecOpt laser_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef enum {
 	TOP, RIGHT, BOTTOM, LEFT
@@ -67,48 +66,54 @@ typedef struct {
 	int         sw;		/* stack width */
 	int         so;		/* stack offset */
 	int         time;	/* up time */
-	GC          stippled_GC;
+	GC          stippledGC;
 	laserstruct *laser;
 } lasersstruct;
 
-static lasersstruct lasers[MAXSCREENS];
+static lasersstruct *lasers = NULL;
 
 static XGCValues gcv_black;	/* for black color */
 
 void
 init_laser(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
 	int         i, c = 0;
 	XGCValues   gcv;
-	Screen     *scr;
-	lasersstruct *lp = &lasers[screen];
+	lasersstruct *lp;
+
+	if (lasers == NULL) {
+		if ((lasers = (lasersstruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (lasersstruct))) == NULL)
+			return;
+	}
+	lp = &lasers[MI_SCREEN(mi)];
 
-	scr = ScreenOfDisplay(dsp, screen);
 	lp->width = MI_WIN_WIDTH(mi);
 	lp->height = MI_WIN_HEIGHT(mi);
 	lp->time = 0;
 
 	lp->ln = MI_BATCHCOUNT(mi);
-	if (lp->ln >= MAXLASER)
-		lp->ln = RANGE_RAND(MINLASER, MAXLASER);
-	/* Absolute minimum */
-	if (lp->ln < MINLASER)
+	if (lp->ln < -MINLASER) {
+		/* if lp->ln is random ... the size can change */
+		if (lp->laser != NULL) {
+			(void) free((void *) lp->laser);
+			lp->laser = NULL;
+		}
+		lp->ln = NRAND(-lp->ln - MINLASER + 1) + MINLASER;
+	} else if (lp->ln < MINLASER)
 		lp->ln = MINLASER;
 
 	if (!lp->laser) {
-		lp->laser = (laserstruct *) malloc(MAXLASER * sizeof (laserstruct));
+		lp->laser = (laserstruct *) malloc(lp->ln * sizeof (laserstruct));
 
-		gcv.foreground = WhitePixelOfScreen(scr);
-		gcv.background = BlackPixelOfScreen(scr);
-		gcv_black.foreground = BlackPixelOfScreen(scr);
+		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
+		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv_black.foreground = MI_WIN_BLACK_PIXEL(mi);
 
-		lp->stippled_GC = XCreateGC(dsp, win,
-					  GCForeground | GCBackground, &gcv);
+		lp->stippledGC = XCreateGC(MI_DISPLAY(mi), MI_WINDOW(mi),
+					   GCForeground | GCBackground, &gcv);
 	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, lp->width, lp->height);
-
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
 	if (MINDIST < lp->width - MINDIST)
 		lp->cx = RANGE_RAND(MINDIST, lp->width - MINDIST);
@@ -123,54 +128,55 @@ init_laser(ModeInfo * mi)
 	lp->sw = 0;
 	lp->so = 0;
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		c = LRAND() % Scr[screen].npixels;
+	if (MI_NPIXELS(mi) > 2)
+		c = NRAND(MI_NPIXELS(mi));
 
 	for (i = 0; i < lp->ln; i++) {
 		laserstruct *l = &lp->laser[i];
 
-		l->bn = (border) MAX_RAND(4);
+		l->bn = (border) NRAND(4);
 
 		switch (l->bn) {
 			case TOP:
-				l->bx = MAX_RAND(lp->width);
+				l->bx = NRAND(lp->width);
 				l->by = 0;
 				break;
 			case RIGHT:
 				l->bx = lp->width;
-				l->by = MAX_RAND(lp->height);
+				l->by = NRAND(lp->height);
 				break;
 			case BOTTOM:
-				l->bx = MAX_RAND(lp->width);
+				l->bx = NRAND(lp->width);
 				l->by = lp->height;
 				break;
 			case LEFT:
 				l->bx = 0;
-				l->by = MAX_RAND(lp->height);
+				l->by = NRAND(lp->height);
 		}
 
-		l->dir = MAX_RAND(2);
+		l->dir = LRAND() & 1;
 		l->speed = ((RANGE_RAND(MINSPEED, MAXSPEED) * lp->width) / 1000) + 1;
-		if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-			l->gcv.foreground = Scr[screen].pixels[c];
-			c = (c + COLORSTEP) % Scr[screen].npixels;
+		if (MI_NPIXELS(mi) > 2) {
+			l->gcv.foreground = MI_PIXEL(mi, c);
+			c = (c + COLORSTEP) % MI_NPIXELS(mi);
 		} else
-			l->gcv.foreground = WhitePixel(dsp, screen);
+			l->gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
 	}
 }
 
 static void
-draw_laser_once(Window win)
+draw_laser_once(ModeInfo * mi)
 {
-	lasersstruct *lp = &lasers[screen];
+	Display    *display = MI_DISPLAY(mi);
+	lasersstruct *lp = &lasers[MI_SCREEN(mi)];
 	int         i;
 
 	for (i = 0; i < lp->ln; i++) {
 		laserstruct *l = &lp->laser[i];
 
 		if (lp->sw >= lp->lw) {
-			XChangeGC(dsp, lp->stippled_GC, GCForeground, &gcv_black);
-			XDrawLine(dsp, win, lp->stippled_GC,
+			XChangeGC(display, lp->stippledGC, GCForeground, &gcv_black);
+			XDrawLine(display, MI_WINDOW(mi), lp->stippledGC,
 				  lp->cx, lp->cy,
 				  l->sx[lp->so], l->sy[lp->so]);
 		}
@@ -244,8 +250,9 @@ draw_laser_once(Window win)
 			}
 		}
 
-		XChangeGC(dsp, lp->stippled_GC, GCForeground, &l->gcv);
-		XDrawLine(dsp, win, lp->stippled_GC, lp->cx, lp->cy, l->bx, l->by);
+		XChangeGC(display, lp->stippledGC, GCForeground, &l->gcv);
+		XDrawLine(display, MI_WINDOW(mi), lp->stippledGC,
+			  lp->cx, lp->cy, l->bx, l->by);
 
 		l->sx[lp->so] = l->bx;
 		l->sy[lp->so] = l->by;
@@ -261,13 +268,37 @@ draw_laser_once(Window win)
 void
 draw_laser(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	lasersstruct *lp = &lasers[screen];
+	lasersstruct *lp = &lasers[MI_SCREEN(mi)];
 	int         i;
 
 	for (i = 0; i < lp->lr; i++)
-		draw_laser_once(win);
+		draw_laser_once(mi);
 
 	if (++lp->time > MI_CYCLES(mi))
 		init_laser(mi);
 }
+
+void
+release_laser(ModeInfo * mi)
+{
+	if (lasers != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			lasersstruct *lp = &lasers[screen];
+
+			if (lp->laser != NULL)
+				(void) free((void *) lp->laser);
+			if (lp->stippledGC != NULL)
+				XFreeGC(MI_DISPLAY(mi), lp->stippledGC);
+		}
+		(void) free((void *) lasers);
+		lasers = NULL;
+	}
+}
+
+void
+refresh_laser(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
