@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)life1d.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)life1d.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -34,7 +35,13 @@ static char sccsid[] = "@(#)life1d.c	3.9 96/05/25 xlockmore";
  */
 
 #include "xlock.h"
-#include "bitmaps/x11.xbm"
+
+/* aliases for vars defined in the bitmap file */
+#define CELL_WIDTH   image_width
+#define CELL_HEIGHT    image_height
+#define CELL_BITS    image_bits
+
+#include "life1d.xbm"
 
 #if defined( TOTALISTIC ) && defined( LCAU )
 #undef TOTALISTIC
@@ -49,9 +56,6 @@ static char sccsid[] = "@(#)life1d.c	3.9 96/05/25 xlockmore";
 #endif
 #endif
 
-#define MAXROWS 200
-#define MAXCOLS 175
-#define BORDER 512		/* Need a big border, the lifeform may reflect off edge */
 #ifdef TOTALISTIC
 #define STATES 5
 #define RADIUS 4
@@ -63,7 +67,9 @@ static char sccsid[] = "@(#)life1d.c	3.9 96/05/25 xlockmore";
 #endif
 
 ModeSpecOpt life1d_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
+
+extern int  gridsize;
 
 static XImage logo =
 {
@@ -78,21 +84,21 @@ typedef struct {
 	int         xs, ys;
 	int         xb, yb;
 	int         generation;
-	int         nrows, ncols;
+	int         nrows, ncols, border;
 	int         width, height;
-	int         k, r, code;
-	unsigned char buffer[MAXROWS * MAXCOLS];
-	unsigned char newcells[MAXROWS + 2 * BORDER];
-	unsigned char oldcells[MAXROWS + 2 * (RADIUS + BORDER)];
+	int         k, r;
+	long        code;
 	unsigned char nextstate[MAXSUM_SIZE];
-	unsigned char colors[STATES - 1];
+	int         colors[STATES - 1];
+
+	unsigned char *newcells;
+	unsigned char *oldcells;
+	unsigned char *buffer;
 } life1dstruct;
 
-static life1dstruct lifes[MAXSCREENS];
+static life1dstruct *life1ds = NULL;
 static int  icon_width, icon_height;
 
-static int  initialized = 0;
-
 #ifdef TOTALISTIC
 static int  totalistic_rules[][3] =
 {
@@ -350,48 +356,48 @@ unsigned char lcau41_rules[][64] =
 static void
 drawcell(ModeInfo * mi, int col, int row, unsigned int state)
 {
-	Window      win = MI_WINDOW(mi);
-	life1dstruct *lp = &lifes[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 
 	if (!state) {
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-		XFillRectangle(dsp, win, Scr[screen].gc,
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XFillRectangle(display, window, gc,
 		lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
 		return;
 	}
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		XSetForeground(dsp, Scr[screen].gc,
-			       Scr[screen].pixels[lp->colors[state - 1]]);
+	if (MI_NPIXELS(mi) > 2)
+		XSetForeground(display, gc, MI_PIXEL(mi, lp->colors[state - 1]));
 	else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
 	if (lp->pixelmode)
-		XFillRectangle(dsp, win, Scr[screen].gc,
+		XFillRectangle(display, window, gc,
 		lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
 	else
-		XPutImage(dsp, win, Scr[screen].gc, &logo,
+		XPutImage(display, window, gc, &logo,
 			  0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
 			  icon_width, icon_height);
 }
 
 static void
-RandomSoup(int n, int v)
+RandomSoup(life1dstruct * lp, int n, int v)
 {
-	life1dstruct *lp = &lifes[screen];
 	int         col;
 
 	v /= 2;
 	if (v < 1)
 		v = 1;
 	for (col = lp->ncols / 2 - v; col < lp->ncols / 2 + v; ++col)
-		if ((LRAND() % 100) < n && col >= 0 && col < lp->ncols)
-			lp->newcells[col + BORDER] = (unsigned char) (LRAND() % (lp->k - 1)) + 1;
+		if (NRAND(100) < n && col >= 0 && col < lp->ncols)
+			lp->newcells[col + lp->border] = (unsigned char) NRAND(lp->k - 1) + 1;
 }
 
-static long int
+static long
 power(int x, int n)
 {				/* raise x to the nth power n >= 0 */
 	int         i;
-	long int    p = 1;
+	long        p = 1;
 
 	for (i = 1; i <= n; ++i)
 		p = p * x;
@@ -399,24 +405,23 @@ power(int x, int n)
 }
 
 static void
-GetRule(int i)
+GetRule(life1dstruct * lp, int i)
 {
-	life1dstruct *lp = &lifes[screen];
-	int         maxsum_size, j;
+	long        maxsum_size, j;
 
 #ifdef TOTALISTIC
-	int         code, pow;
+	long        code, pow;
 
 	lp->k = totalistic_rules[i][0];
 	lp->r = totalistic_rules[i][1];
 	maxsum_size = (lp->k - 1) * (lp->r * 2 + 1) + 1;
 	code = lp->code = totalistic_rules[i][2];
 
-	pow = power(lp->k, maxsum_size - 1);
+	pow = power(lp->k, maxsum_size - 1);	/* Should be less than max long */
 	for (j = 0; j < maxsum_size; j++) {
-		lp->nextstate[maxsum_size - 1 - j] = (unsigned char) ((int) code / pow);
-		code -= (int) lp->nextstate[maxsum_size - 1 - j] * pow;
-		pow /= lp->k;
+		lp->nextstate[maxsum_size - 1 - j] = (unsigned char) (code / pow);
+		code -= (long) lp->nextstate[maxsum_size - 1 - j] * pow;
+		pow /= (long) lp->k;
 	}
 #else
 	lp->r = 1;
@@ -456,8 +461,8 @@ GetRule(int i)
 static int
 compare(ModeInfo * mi)
 {
+	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 	int         row, col, tryagain = False;
-	life1dstruct *lp = &lifes[screen];
 	unsigned char *initl, *cmpr;
 
 	initl = lp->buffer + (lp->nrows - 1) * lp->ncols;
@@ -465,7 +470,7 @@ compare(ModeInfo * mi)
 		cmpr = lp->buffer + row * lp->ncols;
 		for (col = 0; col < lp->ncols; col++) {
 			tryagain = False;
-			if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
+			if (MI_NPIXELS(mi) > 2) {
 				if (*(initl + col) != *cmpr) {
 					tryagain = True;
 					break;
@@ -487,76 +492,96 @@ compare(ModeInfo * mi)
 void
 init_life1d(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	life1dstruct *lp = &lifes[screen];
+	life1dstruct *lp;
 	int         i;
 
-	lp->generation = 0;
-	icon_width = life_width;
-	icon_height = life_height;
+	if (life1ds == NULL) {
+		if ((life1ds = (life1dstruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (life1dstruct))) == NULL)
+			return;
+	}
+	lp = &life1ds[MI_SCREEN(mi)];
 
-	if (!initialized) {
-		initialized = 1;
-		logo.data = (char *) life_bits;
+	lp->generation = 0;
+
+	if (!logo.data) {
+		icon_width = CELL_WIDTH;
+		icon_height = CELL_HEIGHT;
+		logo.data = (char *) CELL_BITS;
 		logo.width = icon_width;
 		logo.height = icon_height;
-		logo.bytes_per_line = (icon_width + 7) / 8;
+		logo.bytes_per_line = (logo.width + 7) / 8;
 	}
+	if (lp->newcells != NULL)
+		(void) free((void *) lp->newcells);
+	if (lp->oldcells != NULL)
+		(void) free((void *) lp->oldcells);
+	if (lp->buffer != NULL)
+		(void) free((void *) lp->buffer);
 	lp->width = MI_WIN_WIDTH(mi);
 	lp->height = MI_WIN_HEIGHT(mi);
+	if (lp->width < 2)
+		lp->width = 2;
+	if (lp->height < 2)
+		lp->height = 2;
 	lp->pixelmode = (lp->width + lp->height < 8 * (icon_width + icon_height));
 	if (lp->pixelmode) {
-		lp->ncols = min(lp->width / 2, MAXCOLS);
-		lp->nrows = min(lp->height / 2, MAXROWS);
+		lp->ncols = MIN(lp->width / 2, gridsize);
+		lp->nrows = MIN(lp->height / 2, gridsize);
 	} else {
-		lp->ncols = min(lp->width / icon_width, MAXCOLS);
-		lp->nrows = min(lp->height / icon_height, MAXROWS);
+		lp->ncols = MIN(lp->width / icon_width, gridsize);
+		lp->nrows = MIN(lp->height / icon_height, gridsize);
 	}
+	lp->border = (lp->nrows / 2 + 1) * MI_CYCLES(mi);
+	lp->newcells = (unsigned char *) calloc(lp->ncols + 2 * lp->border,
+						sizeof (unsigned char));
+
+	lp->oldcells = (unsigned char *) calloc(lp->ncols + 2 * (RADIUS * lp->border),
+						sizeof (unsigned char));
+
+	lp->buffer = (unsigned char *) calloc(lp->ncols * lp->nrows,
+					      sizeof (unsigned char));
+
 	lp->xs = lp->width / lp->ncols;
 	lp->ys = lp->height / lp->nrows;
 	lp->xb = (lp->width - lp->xs * lp->ncols) / 2;
 	lp->yb = (lp->height - lp->ys * lp->nrows) / 2;
 
-	(void) memset((char *) lp->buffer, 0, sizeof (lp->buffer));
-	(void) memset((char *) lp->newcells, 0, sizeof (lp->newcells));
-	(void) memset((char *) lp->oldcells, 0, sizeof (lp->oldcells));
-
-	GetRule((int) (LRAND() % NRULES));
+	GetRule(lp, (int) NRAND(NRULES));
 	if (MI_WIN_IS_VERBOSE(mi)) {
-		(void) printf("colors %d, radius %d, code %d, rule ",
+		(void) printf("colors %d, radius %d, code %ld, rule ",
 			      lp->k, lp->r, lp->code);
 		for (i = (lp->k - 1) * (lp->r * 2 + 1); i >= 0; i--)
-			(void) printf("%d", lp->nextstate[i]);
+			(void) printf("%d", (int) lp->nextstate[i]);
 		(void) printf("\n");
 	}
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		for (i = 0; i < lp->k; i++)
-			lp->colors[i] = (LRAND() % Scr[screen].npixels +
-				      i * Scr[screen].npixels) / (lp->k - 1);
-	RandomSoup(40, 25);
-	(void) memcpy((char *) (lp->oldcells + RADIUS + BORDER),
-		      (char *) (lp->newcells + BORDER), MAXCOLS);
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, lp->width, lp->height);
+	if (MI_NPIXELS(mi) > 2)
+		for (i = 0; i < lp->k - 1; i++)
+			lp->colors[i] = (NRAND(MI_NPIXELS(mi)) + i * MI_NPIXELS(mi)) /
+				(lp->k - 1);
+	RandomSoup(lp, 40, 25);
+	(void) memcpy((char *) (lp->oldcells + RADIUS + lp->border),
+		      (char *) (lp->newcells + lp->border), lp->ncols);
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 }
 
 void
 draw_life1d(ModeInfo * mi)
 {
+	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 	int         row, col;
-	life1dstruct *lp = &lifes[screen];
 
 	if (lp->generation > MI_CYCLES(mi) || (lp->generation && compare(mi)))
 		init_life1d(mi);
 
 	for (col = 0; col < lp->ncols; col++)
-		if (lp->buffer[col] != lp->newcells[col + BORDER])
-			drawcell(mi, col, 0, lp->newcells[col + BORDER]);
-	(void) memcpy((char *) lp->buffer, (char *) (lp->newcells + BORDER),
+		if (lp->buffer[col] != lp->newcells[col + lp->border])
+			drawcell(mi, col, 0, lp->newcells[col + lp->border]);
+	(void) memcpy((char *) lp->buffer, (char *) (lp->newcells + lp->border),
 		      lp->ncols);
 
 	for (row = 1; row < lp->nrows; row++) {
-		for (col = 0; col < MAXCOLS + 2 * BORDER; col++) {
+		for (col = 0; col < lp->ncols + 2 * lp->border; col++) {
 #ifdef TOTALISTIC
 			int         sum = 0, m;
 
@@ -573,13 +598,47 @@ draw_life1d(ModeInfo * mi)
 			lp->newcells[col] = lp->nextstate[sum];
 		}
 		(void) memcpy((char *) (lp->oldcells + RADIUS),
-			      (char *) lp->newcells, MAXCOLS + 2 * BORDER);
+			  (char *) lp->newcells, lp->ncols + 2 * lp->border);
 
 		for (col = 0; col < lp->ncols; col++)
-			if (lp->buffer[col + row * lp->ncols] != lp->newcells[col + BORDER])
-				drawcell(mi, col, row, lp->newcells[col + BORDER]);
+			if (lp->buffer[col + row * lp->ncols] != lp->newcells[col + lp->border])
+				drawcell(mi, col, row, lp->newcells[col + lp->border]);
 		(void) memcpy((char *) (lp->buffer + row * lp->ncols),
-			      (char *) (lp->newcells + BORDER), lp->ncols);
+			    (char *) (lp->newcells + lp->border), lp->ncols);
 	}
 	lp->generation++;
 }
+
+void
+release_life1d(ModeInfo * mi)
+{
+	if (life1ds != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			life1dstruct *lp = &life1ds[screen];
+
+			if (lp->newcells != NULL)
+				(void) free((void *) lp->newcells);
+			if (lp->oldcells != NULL)
+				(void) free((void *) lp->oldcells);
+			if (lp->buffer != NULL)
+				(void) free((void *) lp->buffer);
+		}
+		(void) free((void *) life1ds);
+		life1ds = NULL;
+	}
+}
+
+void
+refresh_life1d(ModeInfo * mi)
+{
+	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
+	int         row, col, nrow;
+  
+	for (row = 0; row < lp->nrows; row++) {
+    nrow = row * lp->ncols;
+		for (col = 0; col < lp->ncols; col++)
+				drawcell(mi, col, row, lp->buffer[col + nrow]);
+  }
+}
