@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)life3d.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)life3d.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -44,10 +45,10 @@ static char sccsid[] = "@(#)life3d.c	3.9 96/05/25 xlockmore";
 #define RT_ANGLE 90
 #define HALFRT_ANGLE 45
 /* Store state of cell in top bit. Reserve low bits for count of living nbrs */
-#define Set3D(x,y,z) SetMem((unsigned int)x,(unsigned int)y,(unsigned int)z,ON)
-#define Reset3D(x,y,z) SetMem((unsigned int)x,(unsigned int)y,(unsigned int)z,OFF)
+#define Set3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,ON)
+#define Reset3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,OFF)
 
-#define SetList3D(x,y,z) SetMem((unsigned int)x,(unsigned int)y,(unsigned int)z,ON), AddToList((unsigned int)x,(unsigned int)y,(unsigned int)z)
+#define SetList3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,ON), AddToList(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z)
 
 #define CellState3D(c) ((c)&ON)
 #define CellNbrs3D(c) ((c)&0x1f)	/* 26 <= 31 */
@@ -71,8 +72,25 @@ static char sccsid[] = "@(#)life3d.c	3.9 96/05/25 xlockmore";
 #define WHITE 4
 #define NUMPTS 42
 
+#define DEF_RULE3D  "4555"
+
+static int  rule3d;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-rule3d", ".life3d.rule3d", XrmoptionSepArg, (caddr_t) NULL},
+};
+static argtype vars[] =
+{
+	{(caddr_t *) & rule3d, "rule3d", "Rule3D", DEF_RULE3D, t_Int}
+};
+static OptionStruct desc[] =
+{
+     {"-rule3d", "<living_min><living_max><birth_min><birth_max> parameters"}
+};
+
 ModeSpecOpt life3d_opts =
-{0, NULL, NULL, NULL};
+{1, opts, 1, vars, desc};
 
 typedef struct _CellList {
 	unsigned char x, y, z;	/* Location in world coordinates */
@@ -84,6 +102,9 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
+ int  living_min, living_max, birth_min, birth_max;
+  int         pattern, patterned_rule;
+	int         rule3d;
 	int         initialized;
 	int         wireframe;	/* FALSE */
 	int         ox, oy, oz;	/* origin */
@@ -98,31 +119,27 @@ typedef struct {
 	unsigned long color[COLORS];
 	int         alt, azm;
 	double      dist;
-	int         living_min, living_max, newcell_min, newcell_max;
 	CellList   *ptrhead, *ptrend, eraserhead, eraserend;
 	CellList   *buckethead[NBUCKETS], *bucketend[NBUCKETS];		/* comfortable upper b */
 } life3dstruct;
 
-static life3dstruct life3ds[MAXSCREENS];
+
+static life3dstruct *life3ds = NULL;
 
 static int  DrawCube(ModeInfo * mi, CellList * cell);
 
-/* PICK ONLY ONE */
-#if !defined( LIFE_4555 ) && !defined( LIFE_5766 ) && !defined( LIFE_5655 ) && !defined( LIFE_6767 )
-/* Enter here which life rule you want or compile with -DLIFE_xxxx */
-#define LIFE_4555
-#if 0
-#define LIFE_4555
-#define LIFE_5766
+/* 4555 life is probably the best */
+/* 5766 life has gliders like 2d 2333 life */
 /* There are no known gliders for 5655 or 6767 so the others may be better */
-#define LIFE_5655
-#define LIFE_6767
-#endif
-#endif
+#define LIFE_4555 0
+#define LIFE_5766 1
+#define LIFE_GLIDERS 2
+#define LIFE_5655 2
+#define LIFE_6767 3
+#define LIFE_RULES 4
 
-static int  patterns[][3 * NUMPTS + 1] =
+static int  patterns_4555[][3 * NUMPTS + 1] =
 {
-#ifdef LIFE_4555
 #if 0
 /* Still life */
 	{			/* V */
@@ -395,8 +412,10 @@ static int  patterns[][3 * NUMPTS + 1] =
 		1, 6, 5, 2, 6, 5,
 		127
 	}
-#endif
-#ifdef LIFE_5766
+};
+
+static int  patterns_5766[][3 * NUMPTS + 1] =
+{
 	{			/* KNIFE-SWITCH BLINKER */
 		0, -1, -1,
 		-1, 0, -1, 0, 0, -1, 1, 0, -1,
@@ -647,10 +666,12 @@ static int  patterns[][3 * NUMPTS + 1] =
 		-2, 0, 2, -1, 0, 2,
 		127
 	}
-#endif
+};
+
 /* Many names of 5655 & 6767 are made up by bagleyd They are listed in order
    given by Carter Bays all p2 except "H"  */
-#ifdef LIFE_5655
+static int  patterns_5655[][3 * NUMPTS + 1] =
+{
 	{			/* SEESAW */
 		0, -1, -1,
 		-1, 0, -1, 0, 0, -1,
@@ -1032,8 +1053,9 @@ static int  patterns[][3 * NUMPTS + 1] =
 		-2, 0, 1, -1, 0, 1, 0, 0, 1,
 		127
 	}
-#endif
-#ifdef LIFE_6767
+};
+static int  patterns_6767[][3 * NUMPTS + 1] =
+{
 	{			/* WALKING BOX */
 		-1, -1, -2,
 		0, 0, -2,
@@ -1108,18 +1130,54 @@ static int  patterns[][3 * NUMPTS + 1] =
 		-1, 1, 1, 0, 1, 1,
 		127
 	}
-#endif
 };
 
-#define NPATS (sizeof patterns / sizeof patterns[0])
+static int npatterns[] = {
+      (sizeof patterns_4555 / sizeof patterns_4555[0]),
+      (sizeof patterns_5766 / sizeof patterns_5766[0]),
+      (sizeof patterns_5655 / sizeof patterns_5655[0]),
+      (sizeof patterns_6767 / sizeof patterns_6767[0]),
+};
+
+static int
+Rule2Array(int rule)
+{
+		switch (rule) {
+			case 4555:
+				return LIFE_4555;
+			case 5766:
+				return LIFE_5766;
+			case 5655:
+				return LIFE_5655;
+			case 6767:
+				return LIFE_6767;
+   default:
+     return LIFE_RULES;
+}
+}
+
+static int
+Array2Rule(int array)
+{
+	switch (array) {
+		case LIFE_4555:
+		return 4555;
+	case LIFE_5766:
+		return 5766;
+	case LIFE_5655:
+				return 5655;
+	case LIFE_6767:
+		return 6767;
+   default:
+     return 0;
+   }
+}
 
 /*--- list ---*/
 /* initialise the state of all cells to OFF */
 static void
-Init3D(void)
+Init3D(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
-
 	lp->ptrhead = lp->ptrend = NULL;
 	lp->eraserhead.next = &lp->eraserend;
 	lp->eraserend.prev = &lp->eraserhead;
@@ -1134,9 +1192,8 @@ NewCell(void)
 /* Function that adds the cell (assumed live) at (x,y,z) onto the search *
    list so that it is scanned in future generations */
 static void
-AddToList(unsigned int x, unsigned int y, unsigned int z)
+AddToList(life3dstruct * lp, unsigned int x, unsigned int y, unsigned int z)
 {
-	life3dstruct *lp = &life3ds[screen];
 	CellList   *tmp;
 
 	tmp = NewCell();
@@ -1155,10 +1212,8 @@ AddToList(unsigned int x, unsigned int y, unsigned int z)
 }
 
 static void
-AddToEraseList(CellList * cell)
+AddToEraseList(life3dstruct * lp, CellList * cell)
 {
-	life3dstruct *lp = &life3ds[screen];
-
 	cell->next = &lp->eraserend;
 	cell->prev = lp->eraserend.prev;
 	lp->eraserend.prev->next = cell;
@@ -1166,10 +1221,8 @@ AddToEraseList(CellList * cell)
 }
 
 static void
-DelFromList(CellList * cell)
+DelFromList(life3dstruct * lp, CellList * cell)
 {
-	life3dstruct *lp = &life3ds[screen];
-
 	if (cell != lp->ptrhead)
 		cell->prev->next = cell->next;
 	else {
@@ -1186,7 +1239,7 @@ DelFromList(CellList * cell)
 			lp->ptrend->next = NULL;
 	}
 
-	AddToEraseList(cell);
+	AddToEraseList(lp, cell);
 }
 
 static void
@@ -1202,9 +1255,8 @@ DelFromEraseList(CellList * cell)
    needed. */
 
 static void
-MemInit(void)
+MemInit(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	int         i;
 
 	for (i = 0; i < BASESIZE; ++i)
@@ -1213,10 +1265,9 @@ MemInit(void)
 }
 
 static void
-BaseOffset(unsigned int x, unsigned int y, unsigned int z, int *b, int *o)
+BaseOffset(life3dstruct * lp,
+	   unsigned int x, unsigned int y, unsigned int z, int *b, int *o)
 {
-	life3dstruct *lp = &life3ds[screen];
-
 	*b = ((x & 0x7c) << 7) + ((y & 0x7c) << 2) + ((z & 0x7c) >> 2);
 	*o = (x & 3) + ((y & 3) << 2) + ((z & 3) << 4);
 
@@ -1227,46 +1278,44 @@ BaseOffset(unsigned int x, unsigned int y, unsigned int z, int *b, int *o)
 }
 
 static int
-GetMem(unsigned int x, unsigned int y, unsigned int z)
+GetMem(life3dstruct * lp, unsigned int x, unsigned int y, unsigned int z)
 {
-	life3dstruct *lp = &life3ds[screen];
 	int         b, o;
 
 	if (lp->memstart)
-		MemInit();
-	BaseOffset(x, y, z, &b, &o);
+		MemInit(lp);
+	BaseOffset(lp, x, y, z, &b, &o);
 	return lp->base[b][o];
 }
 
 static void
-SetMem(unsigned int x, unsigned int y, unsigned int z, unsigned int val)
+SetMem(life3dstruct * lp,
+       unsigned int x, unsigned int y, unsigned int z, unsigned int val)
 {
-	life3dstruct *lp = &life3ds[screen];
 	int         b, o;
 
 	if (lp->memstart)
-		MemInit();
+		MemInit(lp);
 
-	BaseOffset(x, y, z, &b, &o);
+	BaseOffset(lp, x, y, z, &b, &o);
 	lp->base[b][o] = val;
 }
 
 static void
-ChangeMem(unsigned int x, unsigned int y, unsigned int z, unsigned int val)
+ChangeMem(life3dstruct * lp,
+	  unsigned int x, unsigned int y, unsigned int z, unsigned int val)
 {
-	life3dstruct *lp = &life3ds[screen];
 	int         b, o;
 
 	if (lp->memstart)
-		MemInit();
-	BaseOffset(x, y, z, &b, &o);
+		MemInit(lp);
+	BaseOffset(lp, x, y, z, &b, &o);
 	lp->base[b][o] += val;
 }
 
 static void
-ClearMem(void)
+ClearMem(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	int         i, j, count;
 
 	for (i = 0; i < BASESIZE; ++i)
@@ -1288,7 +1337,7 @@ ClearMem(void)
  * torus.
  */
 static void
-IncrementNbrs3D(CellList * cell)
+IncrementNbrs3D(life3dstruct * lp, CellList * cell)
 {
 	int         xc, yc, zc, x, y, z;
 
@@ -1299,18 +1348,18 @@ IncrementNbrs3D(CellList * cell)
 		for (y = yc - 1; y != yc + 2; ++y)
 			for (x = xc - 1; x != xc + 2; ++x)
 				if (x != xc || y != yc || z != zc)
-					ChangeMem((unsigned int) x, (unsigned int) y, (unsigned int) z, 1);
+					ChangeMem(lp,
+						  (unsigned int) x, (unsigned int) y, (unsigned int) z, 1);
 }
 
 static void
-End3D(void)
+End3D(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	CellList   *ptr;
 
 	while (lp->ptrhead != NULL) {
-		SetMem(lp->ptrhead->x, lp->ptrhead->y, lp->ptrhead->z, OFF);
-		DelFromList(lp->ptrhead);
+		SetMem(lp, lp->ptrhead->x, lp->ptrhead->y, lp->ptrhead->z, OFF);
+		DelFromList(lp, lp->ptrhead);
 	}
 	ptr = lp->eraserhead.next;
 	while (ptr != &lp->eraserend) {
@@ -1320,9 +1369,8 @@ End3D(void)
 }
 
 static void
-RunLife3D(void)
+RunLife3D(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	unsigned int x, y, z, xc, yc, zc;
 	int         c;
 	CellList   *ptr, *ptrnextcell;
@@ -1330,7 +1378,7 @@ RunLife3D(void)
 	/* Step 1 - Add 1 to all neighbours of living cells. */
 	ptr = lp->ptrhead;
 	while (ptr != NULL) {
-		IncrementNbrs3D(ptr);
+		IncrementNbrs3D(lp, ptr);
 		ptr = ptr->next;
 	}
 
@@ -1352,31 +1400,30 @@ RunLife3D(void)
 			for (y = yc - 1; y != yc + 2; ++y)
 				for (x = xc - 1; x != xc + 2; ++x)
 					if (x != xc || y != yc || z != zc)
-						if ((c = GetMem(x, y, z))) {
+						if ((c = GetMem(lp, x, y, z))) {
 							if (CellState3D(c) == OFF) {
-								if (CellNbrs3D(c) >= lp->newcell_min &&
-								    CellNbrs3D(c) <= lp->newcell_max)
+								if (CellNbrs3D(c) >= lp->birth_min &&
+								    CellNbrs3D(c) <= lp->birth_max)
 									SetList3D(x, y, z);
 								else
 									Reset3D(x, y, z);
 							}
 						}
-		c = GetMem(xc, yc, zc);
+		c = GetMem(lp, xc, yc, zc);
 		if (CellNbrs3D(c) < lp->living_min || CellNbrs3D(c) > lp->living_max) {
-			SetMem(ptr->x, ptr->y, ptr->z, OFF);
-			DelFromList(ptr);
+			SetMem(lp, ptr->x, ptr->y, ptr->z, OFF);
+			DelFromList(lp, ptr);
 		} else
 			Set3D(xc, yc, zc);
 		ptr = ptrnextcell;
 	}
-	ClearMem();
+	ClearMem(lp);
 }
 
 #if 0
 static int
-CountCells3D()
+CountCells3D(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	CellList   *ptr;
 	int         count = 0;
 
@@ -1389,9 +1436,8 @@ CountCells3D()
 }
 
 void
-DisplayList()
+DisplayList(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	CellList   *ptr;
 	int         count = 0;
 
@@ -1409,8 +1455,8 @@ DisplayList()
 static void
 RandomSoup(ModeInfo * mi, int n, int v)
 {
-	life3dstruct *lp = &life3ds[screen];
-	unsigned int x, y, z;
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
+	int         x, y, z;
 
 	v /= 2;
 	if (v < 1)
@@ -1418,20 +1464,33 @@ RandomSoup(ModeInfo * mi, int n, int v)
 	for (z = lp->nstacks / 2 - v; z < lp->nstacks / 2 + v; ++z)
 		for (y = lp->nrows / 2 - v; y < lp->nrows / 2 + v; ++y)
 			for (x = lp->ncolumns / 2 - v; x < lp->ncolumns / 2 + v; ++x)
-				if ((LRAND() % 100) < n)
+				if (NRAND(100) < n)
 					SetList3D(x, y, z);
 	if (MI_WIN_IS_VERBOSE(mi))
 		(void) printf("random pattern\n");
 }
 
 static void
-GetPattern(ModeInfo * mi, int i)
+GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 {
-	life3dstruct *lp = &life3ds[screen];
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 	int         x, y, z;
-	int        *patptr;
+	int        *patptr = NULL;
 
-	patptr = &patterns[i][0];
+	switch (pattern_rule) {
+		case LIFE_4555:
+      patptr = &patterns_4555[pattern][0];
+		  break;
+	case LIFE_5766:
+      patptr = &patterns_5766[pattern][0];
+		  break;
+	case LIFE_5655:
+      patptr = &patterns_5655[pattern][0];
+		  break;
+	case LIFE_6767:
+      patptr = &patterns_6767[pattern][0];
+		  break;
+  }
 	while ((x = *patptr++) != 127) {
 		y = *patptr++;
 		z = *patptr++;
@@ -1443,13 +1502,12 @@ GetPattern(ModeInfo * mi, int i)
 			SetList3D(x, y, z);
 	}
 	if (MI_WIN_IS_VERBOSE(mi))
-		(void) printf("table number %d\n", i);
+		(void) printf("table number %d\n", pattern);
 }
 
 static void
-NewViewpoint(double x, double y, double z)
+NewViewpoint(life3dstruct * lp, double x, double y, double z)
 {
-	life3dstruct *lp = &life3ds[screen];
 	double      k, l, d1, d2;
 
 	k = x * x + y * y;
@@ -1464,9 +1522,9 @@ NewViewpoint(double x, double y, double z)
 }
 
 static void
-NewPoint(double x, double y, double z, register XPoint * cubepts)
+NewPoint(life3dstruct * lp, double x, double y, double z,
+	 register XPoint * cubepts)
 {
-	life3dstruct *lp = &life3ds[screen];
 	double      p1, E;
 
 	p1 = x * lp->vx + y * lp->vy;
@@ -1479,9 +1537,8 @@ NewPoint(double x, double y, double z, register XPoint * cubepts)
 /* Chain together all cells that are at the same distance. These * cannot
    mutually overlap. */
 static void
-SortList(void)
+SortList(life3dstruct * lp)
 {
-	life3dstruct *lp = &life3ds[screen];
 	short       dist;
 	double      d, x, y, z, rsize;
 	int         i, r;
@@ -1584,7 +1641,7 @@ SortList(void)
 					x = (double) ptr->x - lp->ox;
 					y = (double) ptr->y - lp->oy;
 					z = (double) ptr->z - lp->oz;
-					NewPoint(x, y, z, &point);
+					NewPoint(lp, x, y, z, &point);
 
 					r = (int) (rsize * (double) EyeToScreen / (double) ptr->dist);
 					if (point.x + r >= 0 && point.y + r >= 0 &&
@@ -1597,10 +1654,12 @@ SortList(void)
 }
 
 static void
-DrawFace(ModeInfo * mi, int color, XPoint * cubepts, int p1, int p2, int p3, int p4)
+DrawFace(ModeInfo * mi, int color, XPoint * cubepts,
+	 int p1, int p2, int p3, int p4)
 {
-	Window      win = MI_WINDOW(mi);
-	life3dstruct *lp = &life3ds[screen];
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 	XPoint      facepts[5];
 
 	facepts[0] = cubepts[p1];
@@ -1610,15 +1669,15 @@ DrawFace(ModeInfo * mi, int color, XPoint * cubepts, int p1, int p2, int p3, int
 	facepts[4] = cubepts[p1];
 
 	if (!lp->wireframe) {
-		XSetForeground(dsp, Scr[screen].gc, lp->color[color]);
-		XFillPolygon(dsp, win, Scr[screen].gc, facepts, 4, Convex, CoordModeOrigin);
+		XSetForeground(display, gc, lp->color[color]);
+		XFillPolygon(display, MI_WINDOW(mi), gc, facepts, 4,
+			     Convex, CoordModeOrigin);
 	}
-	if (color == BLACK || ((MI_WIN_IS_MONO(mi) || MI_NPIXELS(mi) <= 2) &&
-			       !lp->wireframe))
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
+	if (color == BLACK || (MI_NPIXELS(mi) <= 2 && !lp->wireframe))
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 	else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
-	XDrawLines(dsp, win, Scr[screen].gc, facepts, 5, CoordModeOrigin);
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	XDrawLines(display, MI_WINDOW(mi), gc, facepts, 5, CoordModeOrigin);
 }
 
 #define LEN 0.45
@@ -1627,7 +1686,7 @@ DrawFace(ModeInfo * mi, int color, XPoint * cubepts, int p1, int p2, int p3, int
 static int
 DrawCube(ModeInfo * mi, CellList * cell)
 {
-	life3dstruct *lp = &life3ds[screen];
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 	XPoint      cubepts[8];	/* screen coords for point */
 	int         i = 0, out;
 	unsigned int mask;
@@ -1641,7 +1700,7 @@ DrawCube(ModeInfo * mi, CellList * cell)
 	for (dz = z - LEN; dz <= z + LEN2; dz += LEN2)
 		for (dy = y - LEN; dy <= y + LEN2; dy += LEN2)
 			for (dx = x - LEN; dx <= x + LEN2; dx += LEN2) {
-				NewPoint(dx, dy, dz, &cubepts[i]);
+				NewPoint(lp, dx, dy, dz, &cubepts[i]);
 				if (cubepts[i].x < 0 || cubepts[i].x >= lp->width ||
 				    cubepts[i].y < 0 || cubepts[i].y >= lp->height)
 					++out;
@@ -1677,12 +1736,12 @@ DrawCube(ModeInfo * mi, CellList * cell)
 static void
 DrawScreen(ModeInfo * mi)
 {
-	life3dstruct *lp = &life3ds[screen];
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 	CellList   *ptr;
 	CellList   *eraserptr;
 	int         i;
 
-	SortList();
+	SortList(lp);
 
 	/* Erase dead cubes */
 	eraserptr = lp->eraserhead.next;
@@ -1708,40 +1767,175 @@ DrawScreen(ModeInfo * mi)
 		int         count = 0, v = 0;
 
 
-	(void) printf("Pop=%-4d  Viewpoint (%3d,%3d,%3d)  Origin (%3d,%3d,%3d)  Mode %dx%d\
+		(void) printf("Pop=%-4d  Viewpoint (%3d,%3d,%3d)  Origin (%3d,%3d,%3d)  Mode %dx%d\
 (%d,%d) %d\n",
-		      count, (int) (lp->vx + lp->ox), (int) (lp->vy + lp->oy),
-	   (int) (lp->vz + lp->oz), (int) lp->ox, (int) lp->oy, (int) lp->oz,
-		      lp->width, lp->height, lp->alt, lp->azm, v);
+		     count, (int) (lp->vx + lp->ox), (int) (lp->vy + lp->oy),
+			      (int) (lp->vz + lp->oz), (int) lp->ox, (int) lp->oy, (int) lp->oz,
+			      lp->width, lp->height, lp->alt, lp->azm, v);
 	}
 #endif
 }
 
+static void
+shooter(life3dstruct * lp)
+{
+	int         hsp, vsp, asp, hoff = 1, voff = 1, aoff = 1, r, c2,
+	            r2, s2;
+
+	/* Generate the glider at the edge of the screen */
+#define V 10
+#define V2 (V/2)
+	c2 = lp->ncolumns / 2;
+	r2 = lp->nrows / 2;
+	s2 = lp->nstacks / 2;
+	r = NRAND(3);
+	if (!r) {
+		hsp = NRAND(V2) + c2 - V2 / 2;
+		vsp = (LRAND() & 1) ? r2 - V : r2 + V;
+		asp = (LRAND() & 1) ? s2 - V : s2 + V;
+		if (asp > s2)
+			aoff = -1;
+		if (vsp > r2)
+			voff = -1;
+		if (hsp > c2)
+			hoff = -1;
+		if (lp->rule3d == 4555) {
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+		} else if (lp->rule3d == 5766) {
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 2 * aoff);
+		}
+	} else if (r == 1) {
+		hsp = (LRAND() & 1) ? c2 - V : c2 + V;
+		vsp = (LRAND() & 1) ? r2 - V : r2 + V;
+		asp = NRAND(V2) + s2 - V2 / 2;
+		if (asp > s2)
+			aoff = -1;
+		if (vsp > r2)
+			voff = -1;
+		if (hsp > c2)
+			hoff = -1;
+		if (lp->rule3d == 4555) {
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 3 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+		} else if (lp->rule3d == 5766) {
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+		}
+	} else {
+		hsp = (LRAND() & 1) ? c2 - V : c2 + V;
+		vsp = NRAND(V2) + r2 - V2 / 2;
+		asp = (LRAND() & 1) ? s2 - V : s2 + V;
+		if (asp > s2)
+			aoff = -1;
+		if (vsp > r2)
+			voff = -1;
+		if (hsp > c2)
+			hoff = -1;
+		if (lp->rule3d == 4555) {
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+		} else if (lp->rule3d == 5766) {
+			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+		}
+	}
+}
+
+void
+initrule(ModeInfo * mi)
+{
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
+
+		   if (MI_WIN_IS_VERBOSE(mi))
+			  (void) printf("rule3d %d\n", lp->rule3d);
+		   lp->living_min = lp->rule3d / 1000;
+		   lp->living_max = (lp->rule3d / 100) % 10;
+		   lp->birth_min = (lp->rule3d / 10) % 10;
+		   lp->birth_max = lp->rule3d % 10;
+}
+
 void
 init_life3d(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	life3dstruct *lp = &life3ds[screen];
-	int         i;
+	life3dstruct *lp;
+	int         i, pattern_rule, npats;
+
+	if (life3ds == NULL) {
+		if ((life3ds = (life3dstruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (life3dstruct))) == NULL)
+			return;
+	}
+	lp = &life3ds[MI_SCREEN(mi)];
 
 	lp->generation = 0;
 
+	lp->rule3d = rule3d;
+	if (rule3d == 0)	/* All rules with patterns */
+		lp->rule3d = Array2Rule(NRAND(LIFE_RULES));
+	else if (rule3d == 1)	/* All rules with known gliders */
+		lp->rule3d = Array2Rule(NRAND(LIFE_GLIDERS));
+	if (lp->rule3d != rule3d || !lp->initialized) {
+		/*
+		 * This stuff is not good for rules above 9 cubes but it is unlikely that
+		 * these modes would be much good anyway....
+		 * Rules should not have to be contiguous ...  that is, it should be
+		 * possible to have rules where cells will live if there are 5 or 7
+		 * (not 6) cells around them.   These rules are not general enough
+		 * to allow for that.
+		 */
+    initrule(mi);
+	}
 	if (!lp->initialized) {
-		/* 4555 life
-		   (5766 life is good too (has lifeforms and a glider similar to life)
-		   also 5655 and 6767 are interesting) */
-#ifdef LIFE_4555
-		lp->living_min = 4;
-		lp->living_max = 5;
-		lp->newcell_min = 5;
-		lp->newcell_max = 5;
-#endif
-#ifdef LIFE_5766
-		lp->living_min = 5;
-		lp->living_max = 7;
-		lp->newcell_min = 6;
-		lp->newcell_max = 6;
-#endif
 		lp->dist = 50.0 /*30.0 */ ;
 		lp->alt = 20 /*30 */ ;
 		lp->azm = 10 /*30 */ ;
@@ -1753,29 +1947,29 @@ init_life3d(ModeInfo * mi)
 		lp->oz = lp->nstacks / 2;
 		lp->wireframe = 0;
 		lp->initialized = 1;
-		Init3D();
+		Init3D(lp);
 	} else
-		End3D();
-	lp->color[0] = BlackPixel(dsp, screen);
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		i = LRAND() % 3;
+		End3D(lp);
+	lp->color[0] = MI_WIN_BLACK_PIXEL(mi);
+	if (MI_NPIXELS(mi) > 2) {
+		i = NRAND(3);
 
-		lp->color[i + 1] = Scr[screen].pixels[LRAND() %
-					  (Scr[screen].npixels / COLORBASE)];
-		lp->color[(i + 1) % 3 + 1] = Scr[screen].pixels[LRAND() %
-								(Scr[screen].npixels / COLORBASE) + Scr[screen].npixels / COLORBASE];
-		lp->color[(i + 2) % 3 + 1] = Scr[screen].pixels[LRAND() %
-								(Scr[screen].npixels / COLORBASE) + 2 * Scr[screen].npixels / COLORBASE];
+		lp->color[i + 1] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi) / COLORBASE));
+		lp->color[(i + 1) % 3 + 1] = MI_PIXEL(mi,
+					  NRAND(MI_NPIXELS(mi) / COLORBASE) +
+						 MI_NPIXELS(mi) / COLORBASE);
+		lp->color[(i + 2) % 3 + 1] = MI_PIXEL(mi,
+					  NRAND(MI_NPIXELS(mi) / COLORBASE) +
+					     2 * MI_NPIXELS(mi) / COLORBASE);
 	} else
-		lp->color[1] = lp->color[2] = lp->color[3] = WhitePixel(dsp, screen);
-	lp->color[4] = WhitePixel(dsp, screen);
+		lp->color[1] = lp->color[2] = lp->color[3] = MI_WIN_WHITE_PIXEL(mi);
+	lp->color[4] = MI_WIN_WHITE_PIXEL(mi);
 	lp->width = MI_WIN_WIDTH(mi);
 	lp->height = MI_WIN_HEIGHT(mi);
 	lp->memstart = 1;
 	/*lp->tablesMade = 0; */
 
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, lp->width, lp->height);
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
 	if (lp->alt > 89)
 		lp->alt = 89;
@@ -1785,147 +1979,34 @@ init_life3d(ModeInfo * mi)
 	lp->vx = (sin(lp->azm * IP) * cos(lp->alt * IP) * lp->dist);
 	lp->vy = (cos(lp->azm * IP) * cos(lp->alt * IP) * lp->dist);
 	lp->vz = (sin(lp->alt * IP) * lp->dist);
-	NewViewpoint(lp->vx, lp->vy, lp->vz);
+	NewViewpoint(lp, lp->vx, lp->vy, lp->vz);
 
-	i = LRAND() % (NPATS + 2);
-	if (i >= NPATS)
+  pattern_rule = Rule2Array(lp->rule3d);
+  if (pattern_rule < LIFE_RULES)
+    npats = npatterns[pattern_rule];
+  else
+    npats = 0;
+	lp->pattern = NRAND(npats + 2);
+	if (lp->pattern >= npats)
 		RandomSoup(mi, 30, 10);
 	else
-		GetPattern(mi, i);
+		GetPattern(mi, pattern_rule, lp->pattern);
 
 	DrawScreen(mi);
 }
 
-static void
-shooter(void)
-{
-	int         hsp, vsp, asp, hoff = 1, voff = 1, aoff = 1, r, c2,
-	            r2, s2;
-	life3dstruct *lp = &life3ds[screen];
-
-	/* Generate the glider at the edge of the screen */
-#define V 10
-#define V2 (V/2)
-	c2 = lp->ncolumns / 2;
-	r2 = lp->nrows / 2;
-	s2 = lp->nstacks / 2;
-	r = LRAND() % 3;
-	if (!r) {
-		hsp = LRAND() % V2 + c2 - V2 / 2;
-		vsp = (LRAND() % 2) ? r2 - V : r2 + V;
-		asp = (LRAND() % 2) ? s2 - V : s2 + V;
-		if (asp > s2)
-			aoff = -1;
-		if (vsp > r2)
-			voff = -1;
-		if (hsp > c2)
-			hoff = -1;
-#ifdef LIFE_4555
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 3 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-#endif
-#ifdef LIFE_5766
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 2 * aoff);
-#endif
-	} else if (r == 1) {
-		hsp = (LRAND() % 2) ? c2 - V : c2 + V;
-		vsp = (LRAND() % 2) ? r2 - V : r2 + V;
-		asp = LRAND() % V2 + s2 - V2 / 2;
-		if (asp > s2)
-			aoff = -1;
-		if (vsp > r2)
-			voff = -1;
-		if (hsp > c2)
-			hoff = -1;
-#ifdef LIFE_4555
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 3 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 3 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-#endif
-#ifdef LIFE_5766
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-#endif
-	} else {
-		hsp = (LRAND() % 2) ? c2 - V : c2 + V;
-		vsp = LRAND() % V2 + r2 - V2 / 2;
-		asp = (LRAND() % 2) ? s2 - V : s2 + V;
-		if (asp > s2)
-			aoff = -1;
-		if (vsp > r2)
-			voff = -1;
-		if (hsp > c2)
-			hoff = -1;
-#ifdef LIFE_4555
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-#endif
-#ifdef LIFE_5766
-		SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 0 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
-		SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-#endif
-	}
-}
-
 void
 draw_life3d(ModeInfo * mi)
 {
-	life3dstruct *lp = &life3ds[screen];
+	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
 
-	RunLife3D();
+	RunLife3D(lp);
 	DrawScreen(mi);
 
-	if (++lp->generation > MI_CYCLES(mi) || !lp->visible)
-		/*CountCells3D() == 0) */
+	if (++lp->generation > MI_CYCLES(mi) || !lp->visible) {
+		/*CountCells3D(lp) == 0) */
 		init_life3d(mi);
+  }
 
 	/*
 	 * generate a randomized shooter aimed roughly toward the center of the
@@ -1934,5 +2015,71 @@ draw_life3d(ModeInfo * mi)
 
 	if (lp->generation && lp->generation %
 	    ((MI_BATCHCOUNT(mi) < 0) ? 1 : MI_BATCHCOUNT(mi)) == 0)
-		shooter();
+		shooter(lp);
+}
+
+void
+release_life3d(ModeInfo * mi)
+{
+	if (life3ds != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			life3dstruct *lp = &life3ds[screen];
+
+			End3D(lp);	/* Did I miss something? */
+		}
+		(void) free((void *) life3ds);
+		life3ds = NULL;
+	}
+}
+
+void
+refresh_life3d(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
+
+void
+change_life3d(ModeInfo * mi)
+{
+  life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
+  int         npats, pattern_rule;
+
+  lp->generation = 0;
+
+		End3D(lp);
+	lp->memstart = 1;
+	/*lp->tablesMade = 0; */
+
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+  lp->pattern = lp->pattern + 1;
+  pattern_rule = Rule2Array(lp->rule3d);
+  if (pattern_rule < LIFE_RULES)
+    npats = npatterns[pattern_rule];
+  else
+    npats = 0;
+  if (lp->pattern >= npats + 2) {
+     lp->pattern = 0;
+     if (rule3d == 0) {
+       pattern_rule++;
+       if (pattern_rule >= LIFE_RULES)
+         pattern_rule = 0;
+       lp->rule3d = Array2Rule(pattern_rule);
+       initrule(mi);
+     } else if (rule3d == 1) {
+       pattern_rule++;
+       if (pattern_rule >= LIFE_GLIDERS)
+         pattern_rule = 0;
+       lp->rule3d = Array2Rule(pattern_rule);
+       initrule(mi);
+    }
+	}
+	if (lp->pattern >= npats)
+		RandomSoup(mi, 30, 10);
+	else
+		GetPattern(mi, pattern_rule, lp->pattern);
+
+	DrawScreen(mi);
 }
