@@ -3,7 +3,7 @@
 #define __XLOCK_MODE_H__
 
 /*-
- * @(#)mode.h	3.9 96/05/25 xlockmore
+ * @(#)mode.h	3.11 96/09/20 xlockmore
  *
  * mode.h - mode management for xlock, the X Window System lockscreen.
  *
@@ -14,7 +14,7 @@
  * Revision History:
  *
  * Changes of David Bagley (bagleyd@hertz.njit.edu)
- * 18-Mar-96: Ron Hitchens <ron@utw.com>
+ * 18-Mar-96: Ron Hitchens <ronh@utw.com>
  *		Extensive revision to define new data types for
  *		the new mode calling scheme.
  * 02-Jun-95: Extracted out of resource.c.
@@ -94,9 +94,10 @@ typedef struct {
 #define WI_FLAG_INROOT		0x010
 #define WI_FLAG_NOLOCK	0x020
 #define WI_FLAG_INSTALL		0x040
-#define WI_FLAG_USE3D		0x080
-#define WI_FLAG_VERBOSE		0x100
-#define WI_FLAG_JUST_INITTED	0x200	/* private state flag */
+#define WI_FLAG_DEBUG		0x080
+#define WI_FLAG_USE3D		0x100
+#define WI_FLAG_VERBOSE		0x200
+#define WI_FLAG_JUST_INITTED	0x400	/* private state flag */
 
 typedef struct {
 	float       delta3d;
@@ -156,6 +157,7 @@ typedef struct ModeInfo_s {
 #define MI_WIN_IS_INROOT(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_INROOT))
 #define MI_WIN_IS_NOLOCK(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_NOLOCK))
 #define MI_WIN_IS_INSTALL(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_INSTALL))
+#define MI_WIN_IS_DEBUG(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_DEBUG))
 #define MI_WIN_IS_USE3D(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_USE3D))
 #define MI_WIN_IS_VERBOSE(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_VERBOSE))
 
@@ -169,6 +171,7 @@ typedef struct ModeInfo_s {
 /* are these of interest to modes? */
 #define MI_BG_COLOR(mi)		((mi)->screeninfo->bgcol)
 #define MI_FG_COLOR(mi)		((mi)->screeninfo->fgcol)
+#define MI_NONE_COLOR(mi)	((mi)->screeninfo->nonecol)	/* -install */
 #define MI_RIGHT_COLOR(mi)	((mi)->screeninfo->rightcol)
 #define MI_LEFT_COLOR(mi)	((mi)->screeninfo->leftcol)
 
@@ -211,39 +214,61 @@ extern void release_last_mode();
 extern ModeHook init_ant;
 extern ModeHook draw_ant;
 extern ModeHook release_ant;
+extern ModeHook refresh_ant;
 extern ModeSpecOpt ant_opts;
 
 extern ModeHook init_bat;
 extern ModeHook draw_bat;
+extern ModeHook release_bat;
+extern ModeHook refresh_bat;
 extern ModeSpecOpt bat_opts;
 
+extern ModeHook init_blot;
+extern ModeHook draw_blot;
+extern ModeHook release_blot;
+extern ModeHook refresh_blot;
+extern ModeSpecOpt blot_opts;
+
 extern ModeHook init_bouboule;
 extern ModeHook draw_bouboule;
+extern ModeHook release_bouboule;
+extern ModeHook refresh_bouboule;
 extern ModeSpecOpt bouboule_opts;
 
 extern ModeHook init_bounce;
 extern ModeHook draw_bounce;
+extern ModeHook release_bounce;
+extern ModeHook refresh_bounce;
 extern ModeSpecOpt bounce_opts;
 
 extern ModeHook init_braid;
 extern ModeHook draw_braid;
+extern ModeHook release_braid;
+extern ModeHook refresh_braid;
 extern ModeSpecOpt braid_opts;
 
-extern ModeHook init_blot;
-extern ModeHook draw_blot;
-extern ModeSpecOpt blot_opts;
-
 extern ModeHook init_bug;
 extern ModeHook draw_bug;
+extern ModeHook release_bug;
+extern ModeHook refresh_bug;
 extern ModeSpecOpt bug_opts;
 
 extern ModeHook init_clock;
 extern ModeHook draw_clock;
+extern ModeHook release_clock;
+extern ModeHook refresh_clock;
 extern ModeSpecOpt clock_opts;
 
+extern ModeHook init_daisy;
+extern ModeHook draw_daisy;
+extern ModeHook release_daisy;
+extern ModeHook refresh_daisy;
+extern ModeSpecOpt daisy_opts;
+
 extern ModeHook init_demon;
 extern ModeHook draw_demon;
 extern ModeHook release_demon;
+extern ModeHook refresh_demon;
 extern ModeSpecOpt demon_opts;
 
 extern ModeHook init_eyes;
@@ -255,155 +280,259 @@ extern ModeSpecOpt eyes_opts;
 extern ModeHook init_flag;
 extern ModeHook draw_flag;
 extern ModeHook release_flag;
+extern ModeHook refresh_flag;
 extern ModeSpecOpt flag_opts;
 
 extern ModeHook init_flame;
 extern ModeHook draw_flame;
+extern ModeHook release_flame;
+extern ModeHook refresh_flame;
 extern ModeSpecOpt flame_opts;
 
 extern ModeHook init_forest;
 extern ModeHook draw_forest;
+extern ModeHook release_forest;
+extern ModeHook refresh_forest;
+extern ModeHook refresh_forest;
 extern ModeSpecOpt forest_opts;
 
 extern ModeHook init_galaxy;
 extern ModeHook draw_galaxy;
+extern ModeHook release_galaxy;
+extern ModeHook refresh_galaxy;
 extern ModeSpecOpt galaxy_opts;
 
 extern ModeHook init_geometry;
 extern ModeHook draw_geometry;
+extern ModeHook release_geometry;
+extern ModeHook refresh_geometry;
 extern ModeSpecOpt geometry_opts;
 
 extern ModeHook init_grav;
 extern ModeHook draw_grav;
+extern ModeHook release_grav;
+extern ModeHook refresh_grav;
 extern ModeSpecOpt grav_opts;
 
 extern ModeHook init_helix;
 extern ModeHook draw_helix;
+extern ModeHook release_helix;
+extern ModeHook refresh_helix;
 extern ModeSpecOpt helix_opts;
 
 extern ModeHook init_hop;
 extern ModeHook draw_hop;
+extern ModeHook release_hop;
+extern ModeHook refresh_hop;
 extern ModeSpecOpt hop_opts;
 
 extern ModeHook init_hyper;
 extern ModeHook draw_hyper;
+extern ModeHook release_hyper;
+extern ModeHook refresh_hyper;
 extern ModeSpecOpt hyper_opts;
 
 extern ModeHook init_image;
 extern ModeHook draw_image;
+extern ModeHook release_image;
+extern ModeHook refresh_image;
 extern ModeSpecOpt image_opts;
 
 extern ModeHook init_kaleid;
 extern ModeHook draw_kaleid;
+extern ModeHook release_kaleid;
+extern ModeHook refresh_kaleid;
 extern ModeSpecOpt kaleid_opts;
 
 extern ModeHook init_laser;
 extern ModeHook draw_laser;
+extern ModeHook release_laser;
+extern ModeHook refresh_laser;
 extern ModeSpecOpt laser_opts;
 
 extern ModeHook init_life;
 extern ModeHook draw_life;
+extern ModeHook release_life;
+extern ModeHook refresh_life;
+extern ModeHook change_life;
 extern ModeSpecOpt life_opts;
 
 extern ModeHook init_life1d;
 extern ModeHook draw_life1d;
+extern ModeHook release_life1d;
+extern ModeHook refresh_life1d;
 extern ModeSpecOpt life1d_opts;
 
 extern ModeHook init_life3d;
 extern ModeHook draw_life3d;
+extern ModeHook release_life3d;
+extern ModeHook refresh_life3d;
+extern ModeHook change_life3d;
 extern ModeSpecOpt life3d_opts;
 
+extern ModeHook init_lightning;
+extern ModeHook draw_lightning;
+extern ModeHook release_lightning;
+extern ModeHook refresh_lightning;
+extern ModeSpecOpt lightning_opts;
+
 extern ModeHook init_lissie;
 extern ModeHook draw_lissie;
+extern ModeHook release_lissie;
+extern ModeHook refresh_lissie;
 extern ModeSpecOpt lissie_opts;
 
 extern ModeHook init_marquee;
 extern ModeHook draw_marquee;
+extern ModeHook release_marquee;
+extern ModeHook refresh_marquee;
 extern ModeSpecOpt marquee_opts;
 
 extern ModeHook init_maze;
 extern ModeHook draw_maze;
+extern ModeHook release_maze;
+extern ModeHook refresh_maze;
 extern ModeSpecOpt maze_opts;
 
 extern ModeHook init_mountain;
 extern ModeHook draw_mountain;
+extern ModeHook release_mountain;
+extern ModeHook refresh_mountain;
 extern ModeSpecOpt mountain_opts;
 
 extern ModeHook init_nose;
 extern ModeHook draw_nose;
+extern ModeHook release_nose;
+extern ModeHook refresh_nose;
 extern ModeSpecOpt nose_opts;
 
+extern ModeHook init_penrose;
+extern ModeHook draw_penrose;
+extern ModeHook release_penrose;
+
+				     /* extern ModeHook refresh_penrose; *//* Needed */
+extern ModeSpecOpt penrose_opts;
+
 extern ModeHook init_petal;
 extern ModeHook draw_petal;
+extern ModeHook release_petal;
+extern ModeHook refresh_petal;
 extern ModeSpecOpt petal_opts;
 
 extern ModeHook init_puzzle;
 extern ModeHook draw_puzzle;
+extern ModeHook release_puzzle;
+
+				    /* extern ModeHook refresh_puzzle; *//* Needed, lots of special cases */
 extern ModeSpecOpt puzzle_opts;
 
 extern ModeHook init_pyro;
 extern ModeHook draw_pyro;
+extern ModeHook release_pyro;
+extern ModeHook refresh_pyro;
 extern ModeSpecOpt pyro_opts;
 
 extern ModeHook init_qix;
 extern ModeHook draw_qix;
+extern ModeHook release_qix;
+extern ModeHook refresh_qix;
 extern ModeSpecOpt qix_opts;
 
 extern ModeHook init_rock;
 extern ModeHook draw_rock;
+extern ModeHook release_rock;
+extern ModeHook refresh_rock;
 extern ModeSpecOpt rock_opts;
 
 extern ModeHook init_rotor;
 extern ModeHook draw_rotor;
+extern ModeHook release_rotor;
+extern ModeHook refresh_rotor;
 extern ModeSpecOpt rotor_opts;
 
 extern ModeHook init_shape;
 extern ModeHook draw_shape;
+extern ModeHook release_shape;
+extern ModeHook refresh_shape;
 extern ModeSpecOpt shape_opts;
 
 extern ModeHook init_slip;
 extern ModeHook draw_slip;
+extern ModeHook release_slip;
+
+				  /* extern ModeHook refresh_slip; *//* Probably not practical */
 extern ModeSpecOpt slip_opts;
 
 extern ModeHook init_sphere;
 extern ModeHook draw_sphere;
+extern ModeHook release_sphere;
+extern ModeHook refresh_sphere;
 extern ModeSpecOpt sphere_opts;
 
 extern ModeHook init_spiral;
 extern ModeHook draw_spiral;
+extern ModeHook release_spiral;
+extern ModeHook refresh_spiral;
 extern ModeSpecOpt spiral_opts;
 
 extern ModeHook init_spline;
 extern ModeHook draw_spline;
+extern ModeHook release_spline;
+extern ModeHook refresh_spline;
 extern ModeSpecOpt spline_opts;
 
 extern ModeHook init_swarm;
 extern ModeHook draw_swarm;
+extern ModeHook release_swarm;
+extern ModeHook refresh_swarm;
 extern ModeSpecOpt swarm_opts;
 
 extern ModeHook init_swirl;
 extern ModeHook draw_swirl;
+extern ModeHook release_swirl;
+extern ModeHook refresh_swirl;
 extern ModeSpecOpt swirl_opts;
 
+extern ModeHook init_tri;
+extern ModeHook draw_tri;
+extern ModeHook release_tri;
+extern ModeHook refresh_tri;
+extern ModeSpecOpt tri_opts;
+
 extern ModeHook init_triangle;
 extern ModeHook draw_triangle;
+extern ModeHook release_triangle;
+extern ModeHook refresh_triangle;
 extern ModeSpecOpt triangle_opts;
 
 extern ModeHook init_wator;
 extern ModeHook draw_wator;
+extern ModeHook release_wator;
+extern ModeHook refresh_wator;
 extern ModeSpecOpt wator_opts;
 
+extern ModeHook init_wire;
+extern ModeHook draw_wire;
+extern ModeHook release_wire;
+extern ModeHook refresh_wire;
+extern ModeSpecOpt wire_opts;
+
 extern ModeHook init_world;
 extern ModeHook draw_world;
+extern ModeHook release_world;
+extern ModeHook refresh_world;
 extern ModeSpecOpt world_opts;
 
 extern ModeHook init_worm;
 extern ModeHook draw_worm;
+extern ModeHook release_worm;
+extern ModeHook refresh_worm;
 extern ModeSpecOpt worm_opts;
 
 extern ModeHook init_blank;
 extern ModeHook draw_blank;
 extern ModeHook release_blank;
+extern ModeHook refresh_blank;
 extern ModeSpecOpt blank_opts;
 
 extern ModeHook init_random;
