@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)passwd.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)passwd.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -19,7 +19,7 @@ static char sccsid[] = "@(#)passwd.c	3.9 96/05/25 xlockmore";
  *            Added in ability for any of the root accounts to unlock
  *            screen.  Message now gets sent to syslog if root does the
  *            unlocking.
- * 23-Dec-95: Ron Hitchens <ron@utw.com> reorganized.
+ * 23-Dec-95: Ron Hitchens <ronh@utw.com> reorganized.
  * 10-Dec-95: More context handling stuff for DCE thanks to
  *            Terje Marthinussen <terjem@cc.uit.no>.
  * 01-Sep-95: DCE code added thanks to Heath A. Kehoe
@@ -30,6 +30,7 @@ static char sccsid[] = "@(#)passwd.c	3.9 96/05/25 xlockmore";
 
 #include "xlock.h"
 
+extern char *ProgramName;
 extern Bool allowroot;
 extern Bool inroot;
 extern Bool inwindow;
@@ -49,6 +50,20 @@ extern Bool nolock;
 #include <sys/types.h>
 #endif
 
+#ifdef __bsdi__
+#include <sys/param.h>
+#if _BSDI_VERSION >= 199608
+#define       BSD_AUTH
+#endif
+#endif
+
+#ifdef        BSD_AUTH
+#include <login_cap.h>
+static login_cap_t *lc = NULL;
+static login_cap_t *rlc = NULL;
+
+#endif
+
 #ifdef MULTIPLE_ROOT
 #include <fcntl.h>
 #include <errno.h>
@@ -66,7 +81,11 @@ void        set_multiple();
    screen. */
 struct pwln {
 	char       *pw_name;
+#ifdef        BSD_AUTH
+	login_cap_t *pw_lc;
+#else
 	char       *pw_passwd;
+#endif
 	struct pwln *next;
 };
 typedef struct pwln pwlnode;
@@ -86,7 +105,11 @@ new_pwlnode(void)
 		return ((pwlptr) ENOMEM);
 
 	pwl->pw_name = (char *) NULL;
+#ifdef BSD_AUTH
+	pwl->pw_lc = NULL;
+#else
 	pwl->pw_passwd = (char *) NULL;
+#endif
 	pwl->next = (pwlptr) NULL;
 
 	return (pwl);
@@ -103,7 +126,7 @@ new_pwlnode(void)
 #include <prot.h>
 #endif
 
-#if defined( __linux__ ) && defined( __ELF__ )
+#if defined( __linux__ ) && defined( __ELF__ ) && !defined( HAS_SHADOW )
 /* Linux may or may not have shadow passwords, so it is best to make the same
    binary work with both shadow and non-shadow passwords. It's easy with the
    ELF libc since it has getspnam() and there is no need for any additional
@@ -153,7 +176,7 @@ struct passwd {
 #endif /* LESS_THAN_AIX3_2 */
 
 #ifdef HPUX_SECURE_PASSWD
-#define seteuid(_eu) setresuid(-1, _eu, -1)
+/* #define seteuid(_eu) setresuid(-1, _eu, -1) */
 #define passwd s_passwd
 #define getpwnam(_s) getspwnam(_s)
 #define getpwuid(_u) getspwuid(_u)
@@ -214,18 +237,49 @@ static int  check_dce_net_passwd(char *, char *);
 
 #endif
 
-#ifdef KERBEROS
+#if defined( HAS_KRB4 ) || defined( HAS_KRB5 )
+#ifdef HAS_KRB4
 #include <krb.h>
+#else /* HAS_KRB5 */
+#include <krb5.h>
+#endif
 #include <sys/types.h>
 #include <sys/param.h>
-static int
-krb_check_password(struct passwd *, char *)
+static int  krb_check_password(struct passwd *, char *);
+
 #endif
 
-	extern char *crypt(const char *, const char *);
+#if 0
+extern char *crypt(char *, char *);
+extern char *crypt(const char *, const char *);
+#endif
 
-	static void
-	            getUserName(void)
+#if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) &&  !defined( HP_PASSWDETC ) && !defined( VMS )))
+static struct passwd *
+my_passwd_entry(void)
+{
+	int         uid;
+	char       *user;
+	struct passwd *pw;
+
+	uid = getuid();
+	pw = 0;
+	user = getenv("LOGNAME");
+	if (!user)
+		user = getenv("USER");
+	if (user) {
+		pw = getpwnam(user);
+		if (pw && (pw->pw_uid != uid))
+			pw = 0;
+	}
+	if (!pw)
+		pw = getpwuid(uid);
+	return (pw);
+}
+#endif
+
+static void
+getUserName(void)
 {
 
 #ifdef VMS
@@ -285,7 +339,8 @@ krb_check_password(struct passwd *, char *)
 
 	struct passwd *pw;
 
-	if ((pw = (struct passwd *) getpwuid(getuid())) == NULL)
+	if (!(pw = my_passwd_entry()))
+		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 		error("%s: could not get user name.\n");
 	(void) strcpy(user, pw->pw_name);
 
@@ -308,7 +363,7 @@ passwd_invalid(char *passwd)
 }
 #endif
 
-#if !defined( ultrix ) && !defined( DCE_PASSWD )
+#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )
 /* This routine is not needed if MULTIPE_ROOT */
 static void
 getCryptedUserPasswd(void)
@@ -392,7 +447,8 @@ getCryptedUserPasswd(void)
 	struct spwd *spw;
 
 #endif
-	if ((pw = (struct passwd *) getpwuid(getuid())) == NULL)
+	if (!(pw = my_passwd_entry()))
+		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 		error("%s: could not get encrypted user password.\n");
 #ifdef HAS_SHADOW
 	if ((spw = getspnam(pw->pw_name)) != NULL)
@@ -492,8 +548,10 @@ getCryptedRootPasswd(void)
 
 #endif
 
-	if ((pw = (struct passwd *) getpwuid(0)) == NULL)
-		error("%s: could not get encrypted root password.\n");
+	if (!(pw = getpwnam(ROOT)))
+		if (!(pw = getpwuid(0)))
+			/*if ((pw = (struct passwd *) getpwuid(0)) == NULL) */
+			error("%s: could not get encrypted root password.\n");
 #ifdef HAS_SHADOW
 	spw = getspnam(pw->pw_name);
 	if (spw)
@@ -511,7 +569,7 @@ getCryptedRootPasswd(void)
 #endif /* !VMS */
 }
 
-#endif /* !ultrix && !DCE_PASSWD */
+#endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH */
 
 
 /* 
@@ -574,6 +632,57 @@ checkPasswd(char *buffer)
 					    buffer, NULL) >= 0)));
 #else /* !ultrix */
 
+#ifdef BSD_AUTH
+	char       *pass;
+	char       *style;
+	char       *name;
+
+	done = 0;
+#ifdef        MULTIPLE_ROOT
+	/* Scan through the linked list until you match a password.  Print
+	 * message to log if password match doesn't equal the user.
+	 *
+	 * This should be changed to allow the user name to be typed in also
+	 * to make this more secure.
+	 */
+	for (pwll = pwllh; done == 0 && pwll->next; pwll = pwll->next) {
+		name = pwll->pw_name;
+		lc = pwll->pw_lc;
+#else
+	name = user;
+#endif
+	if ((pass = strchr(buffer, ':')) != NULL) {
+		*pass++ = '\0';
+		style = login_getstyle(lc, buffer, "auth-xlock");
+		if (auth_response(name, lc->lc_class, style,
+				  "response", NULL, "", pass) > 0)
+			done = 1;
+		else if (rlc != NULL) {
+			style = login_getstyle(rlc, buffer, "auth-xlock");
+			if (auth_response(ROOT, rlc->lc_class, style,
+					  "response", NULL, "", pass) > 0)
+				done = 1;
+		}
+		pass[-1] = ':';
+	}
+	if (done == 0) {
+		style = login_getstyle(lc, NULL, "auth-xlock");
+		if (auth_response(name, lc->lc_class, style,
+				  "response", NULL, "", buffer) > 0)
+			done = 1;
+		else if (rlc != NULL) {
+			style = login_getstyle(rlc, NULL, "auth-xlock");
+			if (auth_response(ROOT, rlc->lc_class, style,
+					  "response", NULL, "", buffer) > 0)
+				done = 1;
+		}
+	}
+#ifdef        MULTIPLE_ROOT
+}
+#endif
+
+#else /* !BSD_AUTH */
+
 #ifdef AFS
 	char       *reason;
 
@@ -581,12 +690,12 @@ checkPasswd(char *buffer)
 	done = !ka_UserAuthenticate(user, "", 0, buffer, 0, &reason);
 	if (!done)
 #endif /* !AFS */
-#ifdef KERBEROS
+#if defined(HAS_KRB4) || defined(HAS_KRB5)
 		if (!strcmp(userpass, "*"))
 			done = (krb_check_password((struct passwd *) getpwuid(getuid()), buffer) ||
 				(allowroot && !strcmp((char *) crypt(buffer, rootpass), rootpass)));
 		else
-#endif /* !KERBEROS */
+#endif /* !HAS_KRB4 && !HAS_KRB5 */
 #ifdef MULTIPLE_ROOT
 			/* Scan through the linked list until you match a password.  Print
 			 * message to log if password match doesn't equal the user.
@@ -616,6 +725,7 @@ checkPasswd(char *buffer)
 		 */
 		done = False;
 
+#endif /* !BSD_AUTH */
 #endif /* !ultrix */
 #endif /* !DCE_PASSWD */
 #endif /* !VMS */
@@ -873,7 +983,7 @@ check_dce_net_passwd(char *usr, char *pass)
 
 #endif /* DCE_PASSWD */
 
-#ifdef KERBEROS
+#ifdef HAS_KRB4
 int
 krb_check_password(struct passwd *pwd, char *pass)
 {
@@ -882,7 +992,8 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 	/* find local realm */
 	if (krb_get_lrealm(realm, 1) != KSUCCESS)
-		(void) strncpy(realm, KRB_REALM, sizeof (realm));
+		/*(void) strncpy(realm, KRB_REALM, sizeof (realm)); */
+		(void) strncpy(realm, krb_get_default_realm(), sizeof (realm));
 
 	/* Construct a ticket file */
 	(void) sprintf(tkfile, "/tmp/xlock_tkt_%d", getpid());
@@ -900,14 +1011,131 @@ krb_check_password(struct passwd *pwd, char *pass)
 	}
 	return 0;
 }
-#endif /* KERBEROS */
+#endif /* HAS_KERB4 */
 
+#ifdef HAS_KRB5
+/*-
+ * Pretty much all of this was snatched out of the kinit from the Kerberos5b6
+ * distribution.  The reason why I felt it was necessary to use kinit was
+ * that if someone is locked for a long time, their credentials could expire,
+ * so xlock must be able to get a new ticket.  -- dah <rodmur@ecst.csuchico.edu>
+ */
+#define KRB5_DEFAULT_OPTIONS 0
+#define KRB5_DEFAULT_LIFE 60*60*10	/* 10 hours */
+static int
+krb_check_password(struct passwd *pwd, char *pass)
+{
+	krb5_context kcontext;
+	krb5_timestamp now;
+	krb5_ccache ccache = NULL;
+	krb5_principal me;
+	krb5_principal server;
+	int         options = KRB5_DEFAULT_OPTIONS;
+	krb5_deltat lifetime = KRB5_DEFAULT_LIFE;
+	krb5_error_code code;
+	krb5_creds  my_creds;
+	char       *client_name;
+	krb5_address **addrs = (krb5_address **) 0;
+	krb5_preauthtype *preauth = NULL;
+
+	krb5_data   tgtname =
+	{
+		0,
+		KRB5_TGS_NAME_SIZE,
+		KRB5_TGS_NAME
+	};
+
+	krb5_init_context(&kcontext);
+	krb5_init_ets(kcontext);
+
+	if ((code = krb5_timeofday(kcontext, &now))) {
+		com_err(ProgramName, code, "while getting time of day");
+		return 0;	/* seems better to deny access, than just exit, which
+				   was what happened in kinit  */
+	}
+	if ((code = krb5_cc_default(kcontext, &ccache))) {
+		com_err(ProgramName, code, "while getting default ccache");
+		return 0;
+	}
+	code = krb5_cc_get_principal(kcontext, ccache, &me);
+	if (code) {
+		if ((code = krb5_parse_name(kcontext, pwd->pw_name, &me))) {
+			com_err(ProgramName, code, "when parsing name %s", pwd->pw_name);
+			return 0;
+		}
+	}
+	if ((code = krb5_unparse_name(kcontext, me, &client_name))) {
+		com_err(ProgramName, code, "when unparsing name");
+		return 0;
+	}
+	memset((char *) &my_creds, 0, sizeof (my_creds));
+
+	my_creds.client = me;
+
+	if ((code = krb5_build_principal_ext(kcontext, &server,
+				      krb5_princ_realm(kcontext, me)->length,
+					krb5_princ_realm(kcontext, me)->data,
+					     tgtname.length, tgtname.data,
+				      krb5_princ_realm(kcontext, me)->length,
+					krb5_princ_realm(kcontext, me)->data,
+					     0))) {
+		com_err(ProgramName, code, "while building server name");
+		return 0;
+	}
+	my_creds.server = server;
+
+	my_creds.times.starttime = 0;
+	my_creds.times.endtime = now + lifetime;
+	my_creds.times.renew_till = 0;
+
+	if (strlen(pass) == 0)
+		strcpy(pass, "*");
+	/* if pass is NULL, krb5_get_in_tkt_with_password will prompt with
+	   krb5_default_pwd_prompt1 for password, you don't want that in
+	   this application, most likely the user won't have '*' for a
+	   password -- dah <rodmur@ecst.csuchico.edu> */
+
+	code = krb5_get_in_tkt_with_password(kcontext, options, addrs,
+					     NULL, preauth, pass, 0,
+					     &my_creds, 0);
+
+	memset(pass, 0, sizeof (pass));
+
+	if (code) {
+		if (code == KRB5KRB_AP_ERR_BAD_INTEGRITY)
+			return 0;	/* bad password entered */
+		else {
+			com_err(ProgramName, code, "while getting initial credentials");
+			return 0;
+		}
+	}
+	code = krb5_cc_initialize(kcontext, ccache, me);
+	if (code != 0) {
+		com_err(ProgramName, code, "when initializing cache");
+		return 0;
+	}
+	code = krb5_cc_store_cred(kcontext, ccache, &my_creds);
+	if (code) {
+		com_err(ProgramName, code, "while storing credentials");
+		return 0;
+	}
+	krb5_free_principal(kcontext, server);
+
+	krb5_free_context(kcontext);
+
+	return 1;		/* success */
+}
+#endif /* HAS_KRB5 */
+
+#ifndef VMS
 #undef passwd
 #undef pw_name
 #undef pw_passwd
 #ifndef SUNOS_ADJUNCT_PASSWD
 #include <pwd.h>
 #endif
+#endif
+
 #ifdef MULTIPLE_ROOT
 
 void
@@ -925,18 +1153,18 @@ get_multiple(struct passwd *pw)
 		}
 		pwllh = pwll;
 	}
-	if ((pwll->pw_name = (char *) malloc(strlen(pw->pw_name) + 1)) ==
-	    (char *) -1) {
+	if ((pwll->pw_name = strdup(pw->pw_name)) == NULL) {
 		perror("new");
 		exit(1);
 	}
-	(void) strcpy(pwll->pw_name, pw->pw_name);
-	if ((pwll->pw_passwd = (char *) malloc(strlen(pw->pw_passwd) + 1)) ==
-	    (char *) -1) {
+#ifdef        BSD_AUTH
+	pwll->pw_lc = login_getclass(pw->pw_class);
+#else
+	if ((pwll->pw_passwd = strdup(pw->pw_passwd)) == NULL) {
 		perror("new");
 		exit(1);
 	}
-	(void) strcpy(pwll->pw_passwd, pw->pw_passwd);
+#endif
 	if ((pwll->next = new_pwlnode()) == (pwlptr) ENOMEM) {
 		perror("new");
 		exit(1);
@@ -946,6 +1174,37 @@ get_multiple(struct passwd *pw)
 void
 set_multiple(void)
 {
+#ifdef BSD_AUTH
+	struct passwd *pw;
+	pwlptr      pwll;
+
+	if (pwllh == (pwlptr) NULL) {
+		if ((pwll = new_pwlnode()) == (pwlptr) ENOMEM) {
+			perror("new");
+			exit(1);
+		}
+		pwllh = pwll;
+	}
+	for (pwll = pwllh; pwll->next; pwll = pwll->next);
+
+	while ((pw = getpwent()) != (struct passwd *) NULL) {
+		if (pw->pw_uid)
+			continue;
+		if ((pwll->pw_name = strdup(pw->pw_name)) == NULL) {
+			perror("new");
+			exit(1);
+		}
+		pwll->pw_lc = login_getclass(pw->pw_class);
+
+		if ((pwll->next = new_pwlnode()) == (pwlptr) ENOMEM) {
+			perror("new");
+			exit(1);
+		}
+	}
+
+	if (pwll->next = new_pwlnode())
+		pwll = pwll->next;
+#else /* !BSD_AUTH */
 	/* If you thought the above was sick, then you'll think this is
 	 * downright horrific.  This is set up so that a child process
 	 * is created to read in the password entries using getpwent(3C).
@@ -1064,7 +1323,7 @@ set_multiple(void)
 					perror("read");
 				sbuf = 0;
 			}
-			if ((pwll->pw_name = (char *) malloc(xlen + 1)) == (char *) -1)
+			if ((pwll->pw_name = (char *) malloc(xlen + 1)) == NULL)
 				break;
 			pbuf = pwll->pw_name;
 			cbuf = &buf[sbuf];
@@ -1088,7 +1347,7 @@ set_multiple(void)
 					perror("read");
 				sbuf = 0;
 			}
-			if ((pwll->pw_passwd = (char *) malloc(xlen + 1)) == (char *) -1)
+			if ((pwll->pw_passwd = (char *) malloc(xlen + 1)) == NULL)
 				break;
 			pbuf = pwll->pw_passwd;
 			cbuf = &buf[sbuf];
@@ -1110,6 +1369,7 @@ set_multiple(void)
 		}
 		close(pipefd[0]);
 	}
+#endif /* !BSD_AUTH */
 }
 #endif
 
@@ -1134,7 +1394,8 @@ gpass(void)
 		 * $HOME/.xlockrc instead.
 		 */
 		if ((home = getenv("HOME")) == 0) {
-			struct passwd *p = getpwuid(getuid());
+			/* struct passwd *p = getpwuid(getuid()); */
+			struct passwd *p = my_passwd_entry();
 
 			if (p == 0) {
 				error("%s: Who are you?\n");
@@ -1197,6 +1458,13 @@ initPasswd(void)
 	getUserName();
 #if !defined( ultrix ) && !defined( DCE_PASSWD )
 	if (!nolock && !inroot && !inwindow && grabmouse) {
+#ifdef        BSD_AUTH
+		struct passwd *pwd = getpwnam(user);
+
+		lc = login_getclass(pwd->pw_class);
+		if (allowroot && (pwd = getpwnam(ROOT)) != NULL)
+			rlc = login_getclass(pwd->pw_class);
+#else /* !BSD_AUTH */
 #ifdef USE_XLOCKRC
 		gpass();
 #else
@@ -1204,6 +1472,7 @@ initPasswd(void)
 #endif
 		if (allowroot)
 			getCryptedRootPasswd();
+#endif /* !BSD_AUTH */
 	}
 #endif /* !ultrix && !DCE_PASSWD */
 #ifdef DCE_PASSWD
