@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char sccsid[] = "@(#)resource.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)resource.c	3.11 96/07/23 xlockmore";
 
 #endif
 
@@ -16,11 +16,11 @@ static char sccsid[] = "@(#)resource.c	3.9 96/05/25 xlockmore";
  * Changes of David Bagley (bagleyd@hertz.njit.edu)
  *  3-Apr-96: Jouk Jansen <joukj@alpha.chem.uva.nl>.
  *            Supply for wildcards for filenames for random selection
- * 18-Mar-96: Ron Hitchens <ron@utw.com>
+ * 18-Mar-96: Ron Hitchens <ronh@utw.com>
  *            Setup chosen mode with set_default_mode() for new hook scheme.
  *  6-Mar-96: Jouk Jansen <joukj@alpha.chem.uva.nl>
  *            Remote node checking for VMS fixed
- * 20-Dec-95: Ron Hitchens <ron@utw.com>
+ * 20-Dec-95: Ron Hitchens <ronh@utw.com>
  *            Resource parsing fixed for "nolock".
  * 02-Aug-95: Patch to use default delay, etc., from mode.h thanks to
  *            Roland Bock <exp120@physik.uni-kiel.d400.de>
@@ -48,7 +48,7 @@ static char sccsid[] = "@(#)resource.c	3.9 96/05/25 xlockmore";
  * 06-Jun-91: Added flame mode.
  * 24-May-91: Added -name and -usefirst and -resources.
  * 16-May-91: Added random mode and pyro mode.
- * 26-Mar-91: CheckResources: delay must be >= 0.
+ * 26-Mar-91: checkResources: delay must be >= 0.
  * 29-Oct-90: Added #include <ctype.h> for missing isupper() on some OS revs.
  *	      moved -mode option, reordered Xrm database evaluation.
  * 28-Oct-90: Added text strings.
@@ -114,7 +114,7 @@ static int  descr();
 #endif
 #endif
 #ifndef DEF_MODE
-#define DEF_MODE	"blank"
+#define DEF_MODE	"random"
 #endif
 #ifndef DEF_FONT
 #ifdef AIXV3
@@ -137,11 +137,10 @@ static int  descr();
 #define DEF_CYCLES	"1000"	/* timeout in cycles for a batch */
 #define DEF_SATURATION	"1.0"	/* color ramp saturation 0->1 */
 #define DEF_DELTA3D	"1.5"	/* space between things in 3d mode relative to their size */
-#define DEF_RIGHT3D	"rgb:ffff/0/0"
-#define DEF_LEFT3D	"rgb:0/0/ffff"
 #define DEF_NICE	"10"	/* xlock process nicelevel */
 #define DEF_CLASSNAME	"XLock"
 #define DEF_GEOMETRY	""
+#define DEF_ICONGEOMETRY	""
 #ifndef DEF_MESSAGESFILE
 #define DEF_MESSAGESFILE ""
 #endif
@@ -165,23 +164,28 @@ static int  descr();
 */
 #define DEF_NEIGHBORS  "4"
 
-#if defined( HAS_RPLAY ) || defined( DEF_PLAY )
+#ifdef HAS_RPLAY
 #define DEF_LOCKSOUND	"thank-you"
 #define DEF_INFOSOUND	"identify-please"
 #define DEF_VALIDSOUND	"complete"
 #define DEF_INVALIDSOUND	"not-programmed"
-#ifndef HAS_RPLAY
+#else /* !HAS_RPLAY */
+#ifdef DEF_PLAY
+#define DEF_LOCKSOUND	"thank-you.au"
+#define DEF_INFOSOUND	"identify-please.au"
+#define DEF_VALIDSOUND	"complete.au"
+#define DEF_INVALIDSOUND	"not-programmed.au"
 #define HAS_RPLAY
-#endif
-#else
+#else /* !DEF_PLAY */
 #ifdef VMS_PLAY
 #define DEF_LOCKSOUND	"[]thank-you.au"
 #define DEF_INFOSOUND	"[]identify-please.au"
 #define DEF_VALIDSOUND	"[]complete.au"
 #define DEF_INVALIDSOUND	"[]not-programmed.au"
 #define HAS_RPLAY
-#endif
-#endif
+#endif /* !VMS_PLAY */
+#endif /* !DEF_PLAY */
+#endif /* !HAS_RPLAY */
 
 #ifdef AUTO_LOGOUT
 #define DEF_LOGOUT	"30"	/* minutes until auto-logout */
@@ -221,7 +225,7 @@ static char *mode;
 static char *displayname = NULL;
 static char *classname;
 static char modename[1024];
-static char modeclass[1024];
+static char modeclassname[1024];
 static Bool remote;
 
 #if !defined( VMS ) || defined( XVMSUTILS ) || ( __VMS_VER >= 70000000)
@@ -264,6 +268,8 @@ static XrmOptionDescRec genTable[] =
 	{"+timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "off"},
 	{"-install", ".install", XrmoptionNoArg, (caddr_t) "on"},
 	{"+install", ".install", XrmoptionNoArg, (caddr_t) "off"},
+	{"-debug", ".debug", XrmoptionNoArg, (caddr_t) "on"},
+	{"+debug", ".debug", XrmoptionNoArg, (caddr_t) "off"},
 	{"-nice", ".nice", XrmoptionSepArg, (caddr_t) NULL},
 	{"-timeout", ".timeout", XrmoptionSepArg, (caddr_t) NULL},
 	{"-lockdelay", ".lockdelay", XrmoptionSepArg, (caddr_t) NULL},
@@ -281,8 +287,10 @@ static XrmOptionDescRec genTable[] =
 	{"-use3d", ".use3d", XrmoptionNoArg, (caddr_t) "on"},
 	{"+use3d", ".use3d", XrmoptionNoArg, (caddr_t) "off"},
 	{"-delta3d", ".delta3d", XrmoptionSepArg, (caddr_t) NULL},
+	{"-none3d", ".none3d", XrmoptionSepArg, (caddr_t) NULL},
 	{"-right3d", ".right3d", XrmoptionSepArg, (caddr_t) NULL},
 	{"-left3d", ".left3d", XrmoptionSepArg, (caddr_t) NULL},
+	{"-both3d", ".both3d", XrmoptionSepArg, (caddr_t) NULL},
 
     /* For modes with text, marquee & nose */
 	{"-program", ".program", XrmoptionSepArg, (caddr_t) NULL},
@@ -310,6 +318,7 @@ static XrmOptionDescRec genTable[] =
 	{"-cpasswd", ".cpasswd", XrmoptionSepArg, (caddr_t) NULL},
 #endif
 	{"-geometry", ".geometry", XrmoptionSepArg, (caddr_t) NULL},
+	{"-icongeometry", ".icongeometry", XrmoptionSepArg, (caddr_t) NULL},
 #ifdef AUTO_LOGOUT
 	{"-forceLogout", ".forceLogout", XrmoptionSepArg, (caddr_t) NULL},
 #endif
@@ -382,6 +391,7 @@ static OptionStruct opDesc[] =
 	{"-/+inroot", "turn on/off making xlock run in the root window"},
 	{"-/+timeelapsed", "turn on/off clock"},
 	{"-/+install", "whether to use private colormap if needed (yes/no)"},
+	{"-/+debug", "whether to use debug mode (yes/no)"},
 	{"-delay usecs", "microsecond delay between screen updates"},
 	{"-batchcount num", "number of things per batch"},
 	{"-cycles num", "number of cycles per batch"},
@@ -398,11 +408,14 @@ static OptionStruct opDesc[] =
   {"-validate string", "text string to use for validating password message"},
       {"-invalid string", "text string to use for invalid password message"},
 	{"-geometry geom", "geometry for non-full screen lock"},
+   {"-icongeometry geom", "geometry for password window (location ignored)"},
 
 	{"-/+use3d", "turn on/off 3d view"},
  {"-delta3d value", "space between rocks in 3d mode relative to their size"},
+	{"-none3d color", "colour to be used for null in 3d mode"},
 	{"-right3d color", "colour to be used for the right eye in 3d mode"},
 	{"-left3d color", "colour to be used for the left eye in 3d mode"},
+	{"-both3d color", "colour to be used overlap in 3d mode"},
 
     /* For modes with text, marquee & nose */
    {"-program programname", "program to get messages from, usually fortune"},
@@ -443,6 +456,7 @@ char       *cpasswd;
 
 #endif
 char       *geometry;
+char       *icongeometry;
 int         nicelevel;
 int         delay;
 int         batchcount;
@@ -453,8 +467,10 @@ int         lockdelay;
 
 Bool        use3d;
 float       delta3d;
+char       *none3d;
 char       *right3d;
 char       *left3d;
+char       *both3d;
 
 #ifdef AUTO_LOGOUT
 int         forceLogout;
@@ -487,6 +503,7 @@ Bool        inwindow;
 Bool        inroot;
 Bool        timeelapsed;
 Bool        install;
+Bool        debug;
 
 #ifdef DT_SAVER
 Bool        dtsaver;
@@ -514,6 +531,7 @@ static argtype genvars[] =
 	{(caddr_t *) & cpasswd, "cpasswd", "cpasswd", "", t_String},
 #endif
     {(caddr_t *) & geometry, "geometry", "Geometry", DEF_GEOMETRY, t_String},
+	{(caddr_t *) & icongeometry, "icongeometry", "IconGeometry", DEF_ICONGEOMETRY, t_String},
 	{(caddr_t *) & nicelevel, "nice", "Nice", DEF_NICE, t_Int},
 	{(caddr_t *) & timeout, "timeout", "Timeout", DEF_TIMEOUT, t_Int},
    {(caddr_t *) & lockdelay, "lockdelay", "LockDelay", DEF_LOCKDELAY, t_Int},
@@ -529,11 +547,14 @@ static argtype genvars[] =
 	{(caddr_t *) & verbose, "verbose", "Verbose", "off", t_Bool},
     {(caddr_t *) & timeelapsed, "timeelapsed", "TimeElapsed", "off", t_Bool},
 	{(caddr_t *) & install, "install", "Install", "off", t_Bool},
+	{(caddr_t *) & debug, "debug", "Debug", "off", t_Bool},
 
 	{(caddr_t *) & use3d, "use3d", "Use3D", "off", t_Bool},
 	{(caddr_t *) & delta3d, "delta3d", "Delta3D", DEF_DELTA3D, t_Float},
+	{(caddr_t *) & none3d, "none3d", "None3D", DEF_NONE3D, t_String},
 	{(caddr_t *) & right3d, "right3d", "Right3D", DEF_RIGHT3D, t_String},
 	{(caddr_t *) & left3d, "left3d", "Left3D", DEF_LEFT3D, t_String},
+	{(caddr_t *) & both3d, "both3d", "Both3D", DEF_BOTH3D, t_String},
 
 	{(caddr_t *) & program, "program", "Program", DEF_PROGRAM, t_String},
 	{(caddr_t *) & messagesfile, "messagesfile", "Messagesfile", DEF_MESSAGESFILE, t_String},
@@ -622,7 +643,7 @@ Syntax(char *badOption)
 
 	(void) fprintf(stderr, "usage:  %s", ProgramName);
 	col = 8 + strlen(ProgramName);
-	for (i = 0; i < opDescEntries; i++) {
+	for (i = 0; i < (int) opDescEntries; i++) {
 		len = 3 + strlen(opDesc[i].opt);	/* space [ string ] */
 		if (col + len > 79) {
 			(void) fprintf(stderr, "\n   ");	/* 3 spaces */
@@ -663,7 +684,7 @@ Help(void)
 	(void) fprintf(stderr, "usage:\n        %s [-options ...]\n\n",
 		       ProgramName);
 	(void) fprintf(stderr, "where options include:\n");
-	for (i = 0; i < opDescEntries; i++) {
+	for (i = 0; i < (int) opDescEntries; i++) {
 		(void) fprintf(stderr, "    %-28s %s\n",
 			       opDesc[i].opt, opDesc[i].desc);
 	}
@@ -671,8 +692,13 @@ Help(void)
 	(void) fprintf(stderr, "    %-28s %s\n", "-mode mode", "animation mode");
 	(void) fprintf(stderr, "    where mode is one of:\n");
 	for (i = 0; i < numprocs; i++) {
+		int         j;
+
 		(void) fprintf(stderr, "          %-23s %s\n",
 			       LockProcs[i].cmdline_arg, LockProcs[i].desc);
+		for (j = 0; j < LockProcs[i].msopt->numvarsdesc; j++)
+			(void) fprintf(stderr, "              %-23s %s\n",
+				       LockProcs[i].msopt->desc[j].opt, LockProcs[i].msopt->desc[j].desc);
 	}
 	(void) putc('\n', stderr);
 
@@ -686,7 +712,7 @@ DumpResources(void)
 
 	(void) printf("%s.mode: %s\n", classname, DEF_MODE);
 
-	for (i = 0; i < NGENARGS; i++)
+	for (i = 0; i < (int) NGENARGS; i++)
 		(void) printf("%s.%s: %s\n",
 			      classname, genvars[i].name, genvars[i].def);
 
@@ -716,19 +742,20 @@ LowerString(char *s)
 }
 
 static void
-GetResource(XrmDatabase database, char *parentname, char *parentclass, char *name, char *class, int valueType, char *def, caddr_t * valuep)
+GetResource(XrmDatabase database, char *parentname, char *parentclassname,
+     char *name, char *classname, int valueType, char *def, caddr_t * valuep)
 {
 	char       *type;
 	XrmValue    value;
 	char       *string;
 	char        buffer[1024];
 	char        fullname[1024];
-	char        fullclass[1024];
+	char        fullclassname[1024];
 	int         len;
 
 	(void) sprintf(fullname, "%s.%s", parentname, name);
-	(void) sprintf(fullclass, "%s.%s", parentclass, class);
-	if (XrmGetResource(database, fullname, fullclass, &type, &value)) {
+	(void) sprintf(fullclassname, "%s.%s", parentclassname, classname);
+	if (XrmGetResource(database, fullname, fullclassname, &type, &value)) {
 		string = value.addr;
 		len = value.size;
 	} else {
@@ -890,8 +917,12 @@ open_display(void)
 				struct hostent *host;
 				char      **hp;
 				int         badhost = 1;
+
+#if defined(__cplusplus) || defined(c_plusplus)		/* !__bsdi__ */
 				extern int  gethostname(char *, size_t);
 
+#endif
+
 				if (gethostname(hostname, MAXHOSTNAMELEN))
 					error("%s: Can not get local hostname.\n");
 
@@ -918,32 +949,32 @@ open_display(void)
 }
 
 static void
-printvar(char *class, argtype var)
+printvar(char *classname, argtype var)
 {
 	switch (var.type) {
 		case t_String:
 			(void) fprintf(stderr, "%s.%s: %s\n",
-				       class, var.name, *((char **) var.var));
+				  classname, var.name, *((char **) var.var));
 			break;
 		case t_Bool:
 			(void) fprintf(stderr, "%s.%s: %s\n",
-				       class, var.name, *((int *) var.var)
+				       classname, var.name, *((int *) var.var)
 				       ? "True" : "False");
 			break;
 		case t_Int:
 			(void) fprintf(stderr, "%s.%s: %d\n",
-				       class, var.name, *((int *) var.var));
+				    classname, var.name, *((int *) var.var));
 			break;
 		case t_Float:
 			(void) fprintf(stderr, "%s.%s: %g\n",
-				       class, var.name, *((float *) var.var));
+				  classname, var.name, *((float *) var.var));
 			break;
 	}
 }
 
 
 void
-GetResources(int argc, char **argv)
+getResources(int argc, char **argv)
 {
 	XrmDatabase RDB = NULL;
 	XrmDatabase nameDB = NULL;
@@ -1072,9 +1103,9 @@ GetResources(int argc, char **argv)
 
 	/* the RDB is set, now query load the variables from the database */
 
-	for (i = 0; i < NGENARGS; i++)
+	for (i = 0; i < (int) NGENARGS; i++)
 		GetResource(RDB, ProgramName, classname,
-			    genvars[i].name, genvars[i].class,
+			    genvars[i].name, genvars[i].classname,
 			    genvars[i].type, genvars[i].def, genvars[i].var);
 
 	for (i = 0; i < numprocs; i++) {
@@ -1082,8 +1113,8 @@ GetResources(int argc, char **argv)
 		ModeSpecOpt *ms = LockProcs[i].msopt;
 
 		(void) sprintf(modename, "%s.%s", ProgramName, LockProcs[i].cmdline_arg);
-		(void) sprintf(modeclass, "%s.%s", classname, LockProcs[i].cmdline_arg);
-		for (j = 0; j < NMODEARGS; j++) {
+		(void) sprintf(modeclassname, "%s.%s", classname, LockProcs[i].cmdline_arg);
+		for (j = 0; j < (int) NMODEARGS; j++) {
 			char        buf[16];
 			void       *p = (void *) ((char *) (&LockProcs[i]) + modevaroffs[j]);
 
@@ -1091,20 +1122,20 @@ GetResources(int argc, char **argv)
 				(void) sprintf(buf, "%d", *(int *) p);
 			else
 				(void) sprintf(buf, "%g", *(float *) p);
-			GetResource(RDB, modename, modeclass,
-				    modevars[j].name, modevars[j].class,
+			GetResource(RDB, modename, modeclassname,
+				    modevars[j].name, modevars[j].classname,
 				    modevars[j].type, buf, (caddr_t *) p);
 			if (!strcmp(mode, LockProcs[i].cmdline_arg)) {
-				GetResource(RDB, modename, modeclass,
-					 modevars[j].name, modevars[j].class,
+				GetResource(RDB, modename, modeclassname,
+				     modevars[j].name, modevars[j].classname,
 				     modevars[j].type, buf, modevars[j].var);
 			}
 		}
-		if (!ms->numopts)
+		if (!ms->numvarsdesc)
 			continue;
 		v = ms->vars;
-		for (j = 0; j < ms->numopts; j++)
-			GetResource(RDB, modename, modeclass, v[j].name, v[j].class,
+		for (j = 0; j < ms->numvarsdesc; j++)
+			GetResource(RDB, modename, modeclassname, v[j].name, v[j].classname,
 				    v[j].type, v[j].def, v[j].var);
 	}
 
@@ -1149,9 +1180,9 @@ GetResources(int argc, char **argv)
 #endif
 
 	if (verbose) {
-		for (i = 0; i < NGENARGS; i++)
+		for (i = 0; i < (int) NGENARGS; i++)
 			printvar(classname, genvars[i]);
-		for (i = 0; i < NMODEARGS; i++)
+		for (i = 0; i < (int) NMODEARGS; i++)
 			printvar(modename, modevars[i]);
 	}
 #if !defined( VMS ) || defined( XVMSUTILS ) ||  ( __VMS_VER >= 70000000)
@@ -1168,10 +1199,10 @@ GetResources(int argc, char **argv)
 
 		num_list = scan_dir(directory_, images_list, sel_image, NULL);
 		image_list = *images_list;
-#ifdef DEBUG
-		for (i = 0; i < num_list; i++)
-			(void) printf("File no %d : %s\n", i, image_list[i]->d_name);
-#endif
+    if (debug)
+		  for (i = 0; i < num_list; i++)
+			  (void) printf("File no %d : %s\n", i, image_list[i]->d_name);
+    
 	} else
 		num_list = 0;
 #endif
@@ -1179,14 +1210,14 @@ GetResources(int argc, char **argv)
 
 
 void
-CheckResources(void)
+checkResources(void)
 {
 	int         i;
 
 	if (delay < 0)
 		Syntax("-delay argument must be positive.");
-	if (batchcount < 0)
-		Syntax("-batchcount argument must be positive.");
+	/*if (batchcount < 0)
+	   Syntax("-batchcount argument must be positive."); */
 	if (cycles < 0)
 		Syntax("-cycles argument must be positive.");
 	if (saturation < 0.0 || saturation > 1.0)
