@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)rock.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)rock.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -9,6 +10,7 @@ static char sccsid[] = "@(#)rock.c	3.9 96/05/25 xlockmore";
  *
  * Copyright (c) 1992 by Jamie Zawinski
  * 
+ *  7-Sep-96:  Fixed problems with 3d mode <theiling@coli.uni-sb.de>
  * 8-May-96: Blue on left instead of green for 3d.  It seems more common
  *           than green.  Use "-left3d Green" if you have the other kind.
  * 17-Jan-96: 3D mode for rock thanks to <theiling@coli.uni-sb.de>.
@@ -38,25 +40,34 @@ static char sccsid[] = "@(#)rock.c	3.9 96/05/25 xlockmore";
  * Too bad monitors do not emit polarized light.
  */
 
+#define MIN_ROCKS 1
 #define MIN_DEPTH 2		/* rocks disappear when they get this close */
 #define MAX_DEPTH 60		/* this is where rocks appear */
-#define MAX_WIDTH 100		/* how big (in pixels) rocks are at depth 1 */
+#define MIN_WIDTH 3		/* how small where pixmaps are not used */
+#define MAX_WIDTH 200		/* how big (in pixels) rocks are at depth 1 */
 #define DEPTH_SCALE 100		/* how many ticks there are between depths */
 #define RESOLUTION 1000
-#define MAX_DEP 0.3		/* how far the displacement can be (percents) */
-#define DIRECTION_CHANGE_RATE 50	/* 0 is always */
+/* Change rotation to 0 if it gets you nautious */
+#define ROTATE 1 /* 0 or 1 */
+/* Make MAX_DEP 0.0 and DIRECTION_CHANGE_RATE 100 if you get nautious */
+#define MAX_DEP 0.3	/* how far the displacement can be (percents) */
+#define DIRECTION_CHANGE_RATE 98	/* 0 is always */
 #define MAX_DEP_SPEED 5		/* Maximum speed for movement */
 #define MOVE_STYLE 0		/* Only 0 and 1. Distinguishes the fact that
 				   these are the rocks that are moving (1) 
 				   or the rocks source (0). */
 
+#define GETZDIFF(z) \
+        (MI_DELTA3D(mi)*40.0*(1.0-((MAX_DEPTH*DEPTH_SCALE/2)/(z+20.0*DEPTH_SCALE))))
+ /* the compiler needs to optimize the calculations here */
+
 /* there's not much point in the above being user-customizable, but those
    numbers might want to be tweaked for displays with an order of magnitude
    higher resolution or compute power. */
 
 
 ModeSpecOpt rock_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         real_size;
@@ -74,31 +85,37 @@ typedef struct {
 	int         rotate_p, speed, nrocks;
 	int         move_p;
 	int         dep_x, dep_y;
+	int         max_width;
 	arock      *arocks;
-	Pixmap      pixmaps[MAX_WIDTH];
+	Pixmap     *pixmaps;
 } rockstruct;
 
 static double cos_array[RESOLUTION], sin_array[RESOLUTION];
 static double depths[(MAX_DEPTH + 1) * DEPTH_SCALE];
 
-static rockstruct rocks[MAXSCREENS];
+static rockstruct *rocks = NULL;
 
-static void rock_reset(ModeInfo * mi, arock * arocks), rock_tick(ModeInfo * mi, arock * arocks, int d),
-            rock_compute(ModeInfo * mi, arock * arocks), rock_draw(ModeInfo * mi, arock * arocks, int draw_p);
-static void init_pixmaps(Window win), tick_rocks(ModeInfo * mi, int d);
-static int  compute_move(int axe);
+static void rock_reset(ModeInfo * mi, arock * arocks);
+static void rock_tick(ModeInfo * mi, arock * arocks, int d);
+static void rock_compute(ModeInfo * mi, arock * arocks);
+static void rock_draw(ModeInfo * mi, arock * arocks, int draw_p);
+static void init_pixmaps(ModeInfo * mi);
+static void tick_rocks(ModeInfo * mi, int d);
+static int  compute_move(rockstruct * rp, int axe);
 
 static void
 rock_reset(ModeInfo * mi, arock * arocks)
 {
-	arocks->real_size = MAX_WIDTH;
-	arocks->r = (RESOLUTION * 0.7) + (LRAND() % (30 * RESOLUTION));
-	arocks->theta = LRAND() % RESOLUTION;
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
+
+	arocks->real_size = rp->max_width;
+	arocks->r = (int) (RESOLUTION * 0.7 + NRAND(30 * RESOLUTION));
+	arocks->theta = NRAND(RESOLUTION);
 	arocks->depth = MAX_DEPTH * DEPTH_SCALE;
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		arocks->color = Scr[screen].pixels[LRAND() % Scr[screen].npixels];
+	if (MI_NPIXELS(mi) > 2)
+		arocks->color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	else
-		arocks->color = WhitePixel(dsp, screen);
+		arocks->color = MI_WIN_WHITE_PIXEL(mi);
 	rock_compute(mi, arocks);
 	rock_draw(mi, arocks, True);
 }
@@ -106,7 +123,7 @@ rock_reset(ModeInfo * mi, arock * arocks)
 static void
 rock_tick(ModeInfo * mi, arock * arocks, int d)
 {
-	rockstruct *rp = &rocks[screen];
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
 
 	if (arocks->depth > 0) {
 		rock_draw(mi, arocks, False);
@@ -121,44 +138,49 @@ rock_tick(ModeInfo * mi, arock * arocks, int d)
 			rock_compute(mi, arocks);
 			rock_draw(mi, arocks, True);
 		}
-	} else if ((LRAND() % 40) == 0)
+	} else if (!NRAND(40))
 		rock_reset(mi, arocks);
 }
 
 static void
 rock_compute(ModeInfo * mi, arock * arocks)
 {
-	rockstruct *rp = &rocks[screen];
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
 	double      factor = depths[arocks->depth];
 	double      rsize = arocks->real_size * factor;
 
 	arocks->size = (int) (rsize + 0.5);
-	arocks->diff = (int) -(rsize * MI_DELTA3D(mi) + 0.5);
-	arocks->x = rp->midx + (cos_array[arocks->theta] * arocks->r * factor);
-	arocks->y = rp->midy + (sin_array[arocks->theta] * arocks->r * factor);
+	arocks->diff = (int) (int) GETZDIFF(arocks->depth);
+	arocks->x = rp->midx + (int) (cos_array[arocks->theta] * arocks->r * factor);
+	arocks->y = rp->midy + (int) (sin_array[arocks->theta] * arocks->r * factor);
 	if (rp->move_p) {
 		double      move_factor = (double) (MOVE_STYLE - (double) arocks->depth /
 			  (double) ((MAX_DEPTH + 1) * (double) DEPTH_SCALE));
 
 		/* move_factor is 0 when the rock is close, 1 when far */
-		arocks->x += (double) rp->dep_x * move_factor;
-		arocks->y += (double) rp->dep_y * move_factor;
+		arocks->x += (int) ((double) rp->dep_x * move_factor);
+		arocks->y += (int) ((double) rp->dep_y * move_factor);
 	}
 }
 
 static void
 rock_draw(ModeInfo * mi, arock * arocks, int draw_p)
 {
-	Window      win = MI_WINDOW(mi);
-	rockstruct *rp = &rocks[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
 
 	if (draw_p) {
-		if (MI_WIN_IS_USE3D(mi))
-			XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
-		else
-			XSetForeground(dsp, Scr[screen].gc, arocks->color);
+		if (MI_WIN_IS_USE3D(mi)) {
+			if (MI_WIN_IS_INSTALL(mi))
+				XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
+			else
+				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		} else
+			XSetForeground(display, gc, arocks->color);
 	} else
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
 	if (arocks->x <= 0 || arocks->y <= 0 ||
 	    arocks->x >= rp->width || arocks->y >= rp->height) {
 		/* this means that if a rock were to go off the screen at 12:00, but
@@ -172,44 +194,58 @@ rock_draw(ModeInfo * mi, arock * arocks, int draw_p)
 	}
 	if (arocks->size <= 1) {
 		if (MI_WIN_IS_USE3D(mi)) {
+			if (draw_p) {
+				if (MI_WIN_IS_INSTALL(mi))
+					XSetFunction(display, gc, GXor);
+				XSetForeground(display, gc, MI_LEFT_COLOR(mi));
+			}
+			XDrawPoint(display, window, gc, arocks->x - arocks->diff, arocks->y);
 			if (draw_p)
-				XSetForeground(dsp, Scr[screen].gc, MI_LEFT_COLOR(mi));
-			XDrawPoint(dsp, win, Scr[screen].gc, arocks->x - arocks->diff, arocks->y);
-			if (draw_p)
-				XSetForeground(dsp, Scr[screen].gc, MI_RIGHT_COLOR(mi));
-			XDrawPoint(dsp, win, Scr[screen].gc, arocks->x + arocks->diff, arocks->y);
+				XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
+			XDrawPoint(display, window, gc, arocks->x + arocks->diff, arocks->y);
+			if (draw_p && MI_WIN_IS_INSTALL(mi))
+				XSetFunction(display, gc, GXcopy);
 		} else
-			XDrawPoint(dsp, win, Scr[screen].gc, arocks->x, arocks->y);
-	} else if (arocks->size <= 3 || !draw_p) {
+			XDrawPoint(display, window, gc, arocks->x, arocks->y);
+	} else if (arocks->size <= MIN_WIDTH || !draw_p) {
 		if (MI_WIN_IS_USE3D(mi)) {
-			if (draw_p)
-				XSetForeground(dsp, Scr[screen].gc, MI_LEFT_COLOR(mi));
-			XFillRectangle(dsp, win, Scr[screen].gc,
+			if (draw_p) {
+				if (MI_WIN_IS_INSTALL(mi))
+					XSetFunction(display, gc, GXor);
+				XSetForeground(display, gc, MI_LEFT_COLOR(mi));
+			}
+			XFillRectangle(display, window, gc,
 				       arocks->x - arocks->size / 2 - arocks->diff, arocks->y - arocks->size / 2,
 				       arocks->size, arocks->size);
 			if (draw_p)
-				XSetForeground(dsp, Scr[screen].gc, MI_RIGHT_COLOR(mi));
-			XFillRectangle(dsp, win, Scr[screen].gc,
+				XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
+			XFillRectangle(display, window, gc,
 				       arocks->x - arocks->size / 2 + arocks->diff, arocks->y - arocks->size / 2,
 				       arocks->size, arocks->size);
+			if (draw_p && MI_WIN_IS_INSTALL(mi))
+				XSetFunction(display, gc, GXcopy);
 		} else
-			XFillRectangle(dsp, win, Scr[screen].gc,
+			XFillRectangle(display, window, gc,
 				       arocks->x - arocks->size / 2, arocks->y - arocks->size / 2,
 				       arocks->size, arocks->size);
-	} else if (arocks->size < MAX_WIDTH) {
+	} else if (arocks->size < rp->max_width) {
 		if (MI_WIN_IS_USE3D(mi)) {
-			XSetForeground(dsp, Scr[screen].gc, MI_LEFT_COLOR(mi));
-			XCopyPlane(dsp, rp->pixmaps[arocks->size], win, Scr[screen].gc,
+			if (MI_WIN_IS_INSTALL(mi))
+				XSetFunction(display, gc, GXor);
+			XSetForeground(display, gc, MI_LEFT_COLOR(mi));
+			XCopyPlane(display, rp->pixmaps[arocks->size - MIN_WIDTH], window, gc,
 				   0, 0, arocks->size, arocks->size,
 				   arocks->x - arocks->size / 2 - arocks->diff, arocks->y - arocks->size / 2,
 				   1L);
-			XSetForeground(dsp, Scr[screen].gc, MI_RIGHT_COLOR(mi));
-			XCopyPlane(dsp, rp->pixmaps[arocks->size], win, Scr[screen].gc,
+			XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
+			XCopyPlane(display, rp->pixmaps[arocks->size - MIN_WIDTH], window, gc,
 				   0, 0, arocks->size, arocks->size,
 				   arocks->x - arocks->size / 2 + arocks->diff, arocks->y - arocks->size / 2,
 				   1L);
+			if (MI_WIN_IS_INSTALL(mi))
+				XSetFunction(display, gc, GXcopy);
 		} else
-			XCopyPlane(dsp, rp->pixmaps[arocks->size], win, Scr[screen].gc,
+			XCopyPlane(display, rp->pixmaps[arocks->size - MIN_WIDTH], window, gc,
 				   0, 0, arocks->size, arocks->size,
 				   arocks->x - arocks->size / 2, arocks->y - arocks->size / 2,
 				   1L);
@@ -217,19 +253,26 @@ rock_draw(ModeInfo * mi, arock * arocks, int draw_p)
 }
 
 static void
-init_pixmaps(Window win)
+init_pixmaps(ModeInfo * mi)
 {
-	rockstruct *rp = &rocks[screen];
+	Display    *display = MI_DISPLAY(mi);
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
 	int         i;
 	XGCValues   gcv;
 	GC          fg_gc = 0, bg_gc = 0;
 
-	for (i = 0; i < MIN_DEPTH; i++)
-		rp->pixmaps[i] = 0;
-	for (i = MIN_DEPTH; i < MAX_WIDTH; i++) {
-		int         w = (1 + (i / 32)) << 5;	/* server might be faster if word-aligned */
-		int         h = i;
-		Pixmap      p = XCreatePixmap(dsp, win, w, h, 1);
+	rp->max_width = MI_CYCLES(mi);
+	if (rp->max_width > rp->width - MIN_WIDTH)
+		rp->max_width = rp->width - MIN_WIDTH;
+	if (rp->max_width < 0)
+		rp->max_width = 0;
+	else if (rp->max_width > MAX_WIDTH)
+		rp->max_width = MAX_WIDTH;
+	rp->pixmaps = (Pixmap *) calloc(rp->max_width, sizeof (Pixmap));
+	for (i = 0; i < rp->max_width; i++) {
+		int         w = (1 + ((i + MIN_WIDTH) / 32)) << 5;	/* server might be faster if word-aligned */
+		int         h = i + MIN_WIDTH;
+		Pixmap      p = XCreatePixmap(display, MI_WINDOW(mi), w, h, 1);
 		XPoint      points[7];
 
 		rp->pixmaps[i] = p;
@@ -237,91 +280,47 @@ init_pixmaps(Window win)
 			error("%s: Could not allocate pixmaps, in rock, exitting\n");
 		if (!fg_gc) {	/* must use drawable of pixmap, not window (fmh) */
 			gcv.foreground = 1;
-			fg_gc = XCreateGC(dsp, p, GCForeground, &gcv);
+			fg_gc = XCreateGC(display, p, GCForeground, &gcv);
 			gcv.foreground = 0;
-			bg_gc = XCreateGC(dsp, p, GCForeground, &gcv);
+			bg_gc = XCreateGC(display, p, GCForeground, &gcv);
 		}
-		XFillRectangle(dsp, p, bg_gc, 0, 0, w, h);
-		points[0].x = i * 0.15;
-		points[0].y = i * 0.85;
-		points[1].x = i * 0.00;
-		points[1].y = i * 0.20;
-		points[2].x = i * 0.30;
-		points[2].y = i * 0.00;
-		points[3].x = i * 0.40;
-		points[3].y = i * 0.10;
-		points[4].x = i * 0.90;
-		points[4].y = i * 0.10;
-		points[5].x = i * 1.00;
-		points[5].y = i * 0.55;
-		points[6].x = i * 0.45;
-		points[6].y = i * 1.00;
-		XFillPolygon(dsp, p, fg_gc, points, 7, Nonconvex, CoordModeOrigin);
+		XFillRectangle(display, p, bg_gc, 0, 0, w, h);
+		points[0].x = (int) ((double) h * 0.15);
+		points[0].y = (int) ((double) h * 0.85);
+		points[1].x = (int) ((double) h * 0.00);
+		points[1].y = (int) ((double) h * 0.20);
+		points[2].x = (int) ((double) h * 0.30);
+		points[2].y = (int) ((double) h * 0.00);
+		points[3].x = (int) ((double) h * 0.40);
+		points[3].y = (int) ((double) h * 0.10);
+		points[4].x = (int) ((double) h * 0.90);
+		points[4].y = (int) ((double) h * 0.10);
+		points[5].x = (int) ((double) h * 1.00);
+		points[5].y = (int) ((double) h * 0.55);
+		points[6].x = (int) ((double) h * 0.45);
+		points[6].y = (int) ((double) h * 1.00);
+		XFillPolygon(display, p, fg_gc, points, 7, Nonconvex, CoordModeOrigin);
 	}
-	XFreeGC(dsp, fg_gc);
-	XFreeGC(dsp, bg_gc);
-}
-
-void
-init_rock(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	rockstruct *rp = &rocks[screen];
-	int         i;
-	static int  first = 1;
-
-	rp->width = MI_WIN_WIDTH(mi);
-	rp->height = MI_WIN_HEIGHT(mi);
-	rp->midx = rp->width / 2;
-	rp->midy = rp->height / 2;
-	rp->speed = 100;
-	rp->rotate_p = True;
-	rp->move_p = True;
-	rp->dep_x = 0;
-	rp->dep_y = 0;
-	rp->nrocks = MI_BATCHCOUNT(mi);
-	if (rp->nrocks < 1)
-		rp->nrocks = 1;
-	if (rp->speed < 1)
-		rp->speed = 1;
-	if (rp->speed > 100)
-		rp->speed = 100;
-
-	if (first) {
-		first = 0;
-		for (i = 0; i < RESOLUTION; i++) {
-			sin_array[i] = sin((((double) i) / (RESOLUTION / 2)) * M_PI);
-			cos_array[i] = cos((((double) i) / (RESOLUTION / 2)) * M_PI);
-		}
-		/* we actually only need i/speed of these, but wtf */
-		for (i = 1; i < (sizeof (depths) / sizeof (depths[0])); i++)
-			depths[i] = atan(((double) 0.5) / (((double) i) / DEPTH_SCALE));
-		depths[0] = M_PI_2;	/* avoid division by 0 */
-	}
-	if (!rp->arocks) {
-		rp->arocks = (arock *) calloc(rp->nrocks, sizeof (arock));
-		init_pixmaps(win);
-	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, rp->width, rp->height);
+	XFreeGC(display, fg_gc);
+	XFreeGC(display, bg_gc);
 }
 
 static void
 tick_rocks(ModeInfo * mi, int d)
 {
-	rockstruct *rp = &rocks[screen];
+	rockstruct *rp = &rocks[MI_SCREEN(mi)];
 	int         i;
 
 	if (rp->move_p) {
-		rp->dep_x = compute_move(0);
-		rp->dep_y = compute_move(1);
+		rp->dep_x = compute_move(rp, 0);
+		rp->dep_y = compute_move(rp, 1);
 	}
 	for (i = 0; i < rp->nrocks; i++)
 		rock_tick(mi, &rp->arocks[i], d);
 }
 
 static int
-compute_move(int axe)
+compute_move(rockstruct * rp, int axe)
 				/* 0 for x, 1 for y */
 {
 	static int  current_dep[2] =
@@ -332,7 +331,6 @@ compute_move(int axe)
 	{0, 0};
 	static int  limit[2] =
 	{0, 0};
-	rockstruct *rp = &rocks[screen];
 	int         change = 0;
 
 	limit[0] = rp->midx;
@@ -360,18 +358,82 @@ compute_move(int axe)
 	else if (speed[axe] < -MAX_DEP_SPEED)
 		speed[axe] = -MAX_DEP_SPEED;
 
-	if ((LRAND() % DIRECTION_CHANGE_RATE) == 0) {
+	if (NRAND(100) >= DIRECTION_CHANGE_RATE) {
 		/* We change direction */
 		change = LRAND() & 1;
-		if (change != 1)
+		if (change != 1) {
 			if (direction[axe] == 0)
 				direction[axe] = change - 1;	/* 0 becomes either 1 or -1 */
 			else
 				direction[axe] = 0;	/* -1 or 1 become 0 */
+	  }
 	}
 	return (current_dep[axe]);
 }
 
+void
+init_rock(ModeInfo * mi)
+{
+	rockstruct *rp;
+	int         i;
+	static int  first = 1;
+
+	if (rocks == NULL) {
+		if ((rocks = (rockstruct *) calloc(MI_NUM_SCREENS(mi),
+					       sizeof (rockstruct))) == NULL)
+			return;
+	}
+	rp = &rocks[MI_SCREEN(mi)];
+
+	rp->width = MI_WIN_WIDTH(mi);
+	rp->height = MI_WIN_HEIGHT(mi);
+	rp->midx = rp->width / 2;
+	rp->midy = rp->height / 2;
+	rp->speed = 100;
+	rp->rotate_p = ROTATE;
+	rp->move_p = True;
+	rp->dep_x = 0;
+	rp->dep_y = 0;
+	rp->nrocks = MI_BATCHCOUNT(mi);
+	if (rp->nrocks < -MIN_ROCKS) {
+		if (rp->arocks) {
+			(void) free((void *) rp->arocks);
+			rp->arocks = NULL;
+		}
+		rp->nrocks = NRAND(-rp->nrocks - MIN_ROCKS + 1) + MIN_ROCKS;
+	} else if (rp->nrocks < MIN_ROCKS)
+		rp->nrocks = MIN_ROCKS;
+	if (rp->speed < 1)
+		rp->speed = 1;
+	if (rp->speed > 100)
+		rp->speed = 100;
+
+	if (first) {
+		first = 0;
+		for (i = 0; i < RESOLUTION; i++) {
+			sin_array[i] = sin((((double) i) / (RESOLUTION / 2)) * M_PI);
+			cos_array[i] = cos((((double) i) / (RESOLUTION / 2)) * M_PI);
+		}
+		/* we actually only need i/speed of these, but wtf */
+		for (i = 1; i < (int) (sizeof (depths) / sizeof (depths[0])); i++)
+			depths[i] = atan(((double) 0.5) / (((double) i) / DEPTH_SCALE));
+		depths[0] = M_PI_2;	/* avoid division by 0 */
+	}
+	if (!rp->arocks)
+		rp->arocks = (arock *) calloc(rp->nrocks, sizeof (arock));
+	if (!rp->pixmaps)
+		init_pixmaps(mi);
+
+	/* don't want any exposure events from XCopyPlane */
+	XSetGraphicsExposures(MI_DISPLAY(mi), MI_GC(mi), False);
+	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi)) {
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			       0, 0, rp->width, rp->height);
+	} else
+		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
+
 void
 draw_rock(ModeInfo * mi)
 {
@@ -391,11 +453,40 @@ draw_rock(ModeInfo * mi)
 				current_delta--;
 		}
 	} else {
-		if ((LRAND() % 50) == 0) {
-			new_delta = ((LRAND() % 11) - 5);
-			if ((LRAND() % 10) == 0)
+		if (!NRAND(50)) {
+			new_delta = (NRAND(11) - 5);
+			if (!NRAND(10))
 				new_delta *= 5;
 		}
 	}
 	tick_rocks(mi, current_delta);
 }
+
+void
+release_rock(ModeInfo * mi)
+{
+	if (rocks != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			rockstruct *rp = &rocks[screen];
+			int         i;
+
+			if (rp->arocks != NULL)
+				(void) free((void *) rp->arocks);
+			if (rp->pixmaps != NULL) {
+				for (i = 0; i < rp->max_width; i++)
+					XFreePixmap(MI_DISPLAY(mi), rp->pixmaps[i]);
+				(void) free((void *) rp->pixmaps);
+			}
+		}
+		(void) free((void *) rocks);
+		rocks = NULL;
+	}
+}
+
+void
+refresh_rock(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
+}
