@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)spiral.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)spiral.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -27,12 +28,15 @@ static char sccsid[] = "@(#)spiral.c	3.9 96/05/25 xlockmore";
 #define MAXTRAIL 512		/* The length of the trail */
 #define MAXDOTS 40
 #define MINDOTS 1
-#define TWOPI (2*M_PI)		/* for convienence */
+#define TWOPI (2.0*M_PI)	/* for convienence */
 #define JAGGINESS 4		/* This sets the "Craziness" of the spiral (I like 4) */
 #define SPEED 2.0
 
+/* How many segments to draw per cycle when redrawing */
+#define REDRAWSTEP 3
+
 ModeSpecOpt spiral_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	float       hx, hy, ha, hr;
@@ -40,61 +44,35 @@ typedef struct {
 
 typedef struct {
 	Traildots  *traildots;
-	float       cx;
-	float       cy;
+	float       cx, cy;
 	float       angle;
 	float       radius;
-	float       dr;
-	float       da;
-	float       dx;
-	float       dy;
+	float       dr, da;
+	float       dx, dy;
 	int         erase;
 	int         inc;
 	float       colors;
-	int         width;
-	int         height;
+	int         width, height;
 	float       top, bottom, left, right;
 	int         dots, nlength;
+	int         redrawing, redrawpos;
 } spiralstruct;
 
-static spiralstruct spirals[MAXSCREENS];
+static spiralstruct *spirals = NULL;
 
-static void draw_dots(Window win, int in);
+static void draw_dots(ModeInfo * mi, int in);
 
-
-static int
-tfx(float x)
-{
-
-	int         a;
-	spiralstruct *sp = &spirals[screen];
-
-	a = (int) ((x / sp->right) * (float) sp->width);
-
-	return (a);
-}
-
-static int
-tfy(float y)
-{
-
-	int         a;
-	spiralstruct *sp = &spirals[screen];
-
-	a = sp->height - (int) ((y / sp->top) * (float) sp->height);
-
-	return (a);
-
-}
+#define TFX(sp,x) ((int)((x/sp->right)*(float)sp->width))
+#define TFY(sp,y) ((int)((y/sp->top)*(float)sp->height))
 
 static void
-draw_dots(Window win, int in)
+draw_dots(ModeInfo * mi, int in)
 {
 
 	float       i, inc;
 	float       x, y;
 
-	spiralstruct *sp = &spirals[screen];
+	spiralstruct *sp = &spirals[MI_SCREEN(mi)];
 
 	inc = TWOPI / (float) sp->dots;
 	for (i = 0.0; i < TWOPI; i += inc) {
@@ -102,22 +80,87 @@ draw_dots(Window win, int in)
 			sp->traildots[in].hr;
 		y = sp->traildots[in].hy + SINF(i + sp->traildots[in].ha) *
 			sp->traildots[in].hr;
-		XDrawPoint(dsp, win, Scr[screen].gc, tfx(x), tfy(y));
+		XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			   TFX(sp, x), TFY(sp, y));
 	}
-
 }
 
-/* DrawSpiral */
+void
+init_spiral(ModeInfo * mi)
+{
+	spiralstruct *sp;
+	int         i;
+
+	if (spirals == NULL) {
+		if ((spirals = (spiralstruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (spiralstruct))) == NULL)
+			return;
+	}
+	sp = &spirals[MI_SCREEN(mi)];
+
+	sp->width = MI_WIN_WIDTH(mi);
+	sp->height = MI_WIN_HEIGHT(mi);
+
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	/* Init */
+	sp->nlength = MI_CYCLES(mi);
+
+	if (!sp->traildots)
+		sp->traildots = (Traildots *) malloc(sp->nlength * sizeof (Traildots));
+
+	/* initialize the allocated array */
+	for (i = 0; i < sp->nlength; i++) {
+		sp->traildots[i].hx = 0.0;
+		sp->traildots[i].hy = 0.0;
+		sp->traildots[i].ha = 0.0;
+		sp->traildots[i].hr = 0.0;
+	}
+	sp->redrawing = 0;
+
+	/* keep the window parameters proportional */
+	sp->top = 10000.0;
+	sp->bottom = 0;
+	sp->right = (float) (sp->width) / (float) (sp->height) * (10000.0);
+	sp->left = 0;
+
+	/* assign the initial values */
+	sp->cx = (float) (5000.0 - NRAND(2000)) / 10000.0 * sp->right;
+	sp->cy = (float) (5000.0 - NRAND(2000));
+	sp->radius = (float) (NRAND(200) + 200);
+	sp->angle = 0.0;
+	sp->dx = (float) (10 - NRAND(20)) * SPEED;
+	sp->dy = (float) (10 - NRAND(20)) * SPEED;
+	sp->dr = (float) ((NRAND(10) + 4) * (1 - (LRAND() & 1) * 2));
+	sp->da = (float) NRAND(360) / 7200.0 + 0.01;
+	sp->colors = 0.0;
+	sp->erase = 0;
+	sp->inc = 0;
+	sp->traildots[sp->inc].hx = sp->cx;
+	sp->traildots[sp->inc].hy = sp->cy;
+	sp->traildots[sp->inc].ha = sp->angle;
+	sp->traildots[sp->inc].hr = sp->radius;
+	sp->inc++;
+
+	sp->dots = MI_BATCHCOUNT(mi);
+	if (sp->dots < -MINDOTS)
+		sp->dots = NRAND(sp->dots - MINDOTS + 1) + MINDOTS;
+	/* Absolute minimum */
+	if (sp->dots < MINDOTS)
+		sp->dots = MINDOTS;
+}
+
 void
 draw_spiral(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	spiralstruct *sp = &spirals[screen];
-
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	spiralstruct *sp = &spirals[MI_SCREEN(mi)];
+	int         i, j;
 
 	if (sp->erase == 1) {
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-		draw_dots(win, sp->inc);
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		draw_dots(mi, sp->inc);
 	}
 	sp->cx += sp->dx;
 	sp->traildots[sp->inc].hx = sp->cx;
@@ -142,18 +185,18 @@ draw_spiral(ModeInfo * mi)
 	/* Randomly give some variations to:  */
 
 	/* spiral direction (if it is within the boundaries) */
-	if ((LRAND() % 3000 < 1 * JAGGINESS) &&
+	if ((NRAND(3000) < 1 * JAGGINESS) &&
 	    (((sp->cx > 2000.0) && (sp->cx < 8000.0)) &&
 	     ((sp->cy > 2000.0) && (sp->cy < 8000.0)))) {
-		sp->dx = (float) (10 - LRAND() % 20) * SPEED;
-		sp->dy = (float) (10 - LRAND() % 20) * SPEED;
+		sp->dx = (float) (10 - NRAND(20)) * SPEED;
+		sp->dy = (float) (10 - NRAND(20)) * SPEED;
 	}
 	/* The speed of the change in size of the spiral */
-	if (LRAND() % 3000 < 1 * JAGGINESS) {
-		if (LRAND() % 100 < 50)
-			sp->dr += (float) (LRAND() % 3 + 1);
+	if (NRAND(3000) < 1 * JAGGINESS) {
+		if (LRAND() & 1)
+			sp->dr += (float) (NRAND(3) + 1);
 		else
-			sp->dr -= (float) (LRAND() % 3 + 1);
+			sp->dr -= (float) (NRAND(3) + 1);
 
 		/* don't let it get too wild */
 		if (sp->dr > 18.0)
@@ -162,11 +205,11 @@ draw_spiral(ModeInfo * mi)
 			sp->dr = 4.0;
 	}
 	/* The speed of rotation */
-	if (LRAND() % 3000 < 1 * JAGGINESS)
-		sp->da = (float) (LRAND() % 360) / 7200.0 + 0.01;
+	if (NRAND(3000) < 1 * JAGGINESS)
+		sp->da = (float) NRAND(360) / 7200.0 + 0.01;
 
 	/* Reverse rotation */
-	if (LRAND() % 3000 < 1 * JAGGINESS)
+	if (NRAND(3000) < 1 * JAGGINESS)
 		sp->da *= -1.0;
 
 	sp->angle += sp->da;
@@ -177,79 +220,56 @@ draw_spiral(ModeInfo * mi)
 	else if (sp->angle < 0.0)
 		sp->angle += TWOPI;
 
-	sp->colors += (float) (Scr[screen].npixels) / ((float) (2 * sp->nlength));
-	if (sp->colors >= (float) (Scr[screen].npixels))
+	sp->colors += (float) MI_NPIXELS(mi) / ((float) (2 * sp->nlength));
+	if (sp->colors >= (float) MI_NPIXELS(mi))
 		sp->colors = 0.0;
 
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[(int) sp->colors]);
+	if (MI_NPIXELS(mi) > 2)
+		XSetForeground(display, gc, MI_PIXEL(mi, (int) sp->colors));
 	else
-		XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
-	draw_dots(win, sp->inc);
+		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	draw_dots(mi, sp->inc);
 	sp->inc++;
 
 	if (sp->inc > sp->nlength - 1) {
 		sp->inc -= sp->nlength;
 		sp->erase = 1;
 	}
+	if (sp->redrawing) {
+		for (i = 0; i < REDRAWSTEP; i++) {
+			j = (sp->inc - sp->redrawpos + sp->nlength) % sp->nlength;
+			draw_dots(mi, j);
+
+			if (++(sp->redrawpos) >= sp->nlength) {
+				sp->redrawing = 0;
+				break;
+			}
+		}
+	}
 }
 
 void
-init_spiral(ModeInfo * mi)
+release_spiral(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	spiralstruct *sp = &spirals[screen];
-	int         i;
+	if (spirals != NULL) {
+		int         screen;
 
-	sp->width = MI_WIN_WIDTH(mi);
-	sp->height = MI_WIN_HEIGHT(mi);
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			spiralstruct *sp = &spirals[screen];
 
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, sp->width, sp->height);
-
-	/* Init */
-	sp->nlength = MI_CYCLES(mi);
-
-	if (!sp->traildots)
-		sp->traildots = (Traildots *) malloc(sp->nlength * sizeof (Traildots));
-
-	/* initialize the allocated array */
-	for (i = 0; i < sp->nlength; i++) {
-		sp->traildots[i].hx = 0.0;
-		sp->traildots[i].hy = 0.0;
-		sp->traildots[i].ha = 0.0;
-		sp->traildots[i].hr = 0.0;
+			if (sp->traildots)
+				(void) free((void *) sp->traildots);
+		}
+		(void) free((void *) spirals);
+		spirals = NULL;
 	}
+}
 
-	/* keep the window parameters proportional */
-	sp->top = 10000.0;
-	sp->bottom = 0;
-	sp->right = (float) (sp->width) / (float) (sp->height) * (10000.0);
-	sp->left = 0;
+void
+refresh_spiral(ModeInfo * mi)
+{
+	spiralstruct *sp = &spirals[MI_SCREEN(mi)];
 
-	/* assign the initial values */
-	sp->cx = (float) (5000.0 - (LRAND() % 2000)) / 10000.0 * sp->right;
-	sp->cy = (float) (5000.0 - (LRAND() % 2000));
-	sp->radius = (float) (LRAND() % 200 + 200);
-	sp->angle = 0.0;
-	sp->dx = (float) (10 - LRAND() % 20) * SPEED;
-	sp->dy = (float) (10 - LRAND() % 20) * SPEED;
-	sp->dr = (float) ((LRAND() % 10 + 4) * (1 - LRAND() % 2 * 2));
-	sp->da = (float) (LRAND() % 360) / 7200.0 + 0.01;
-	sp->colors = 0.0;
-	sp->erase = 0;
-	sp->inc = 0;
-	sp->traildots[sp->inc].hx = sp->cx;
-	sp->traildots[sp->inc].hy = sp->cy;
-	sp->traildots[sp->inc].ha = sp->angle;
-	sp->traildots[sp->inc].hr = sp->radius;
-	sp->inc++;
-
-	sp->dots = MI_BATCHCOUNT(mi);
-	if (sp->dots > MAXDOTS)
-		sp->dots = LRAND() % (MAXDOTS - MINDOTS + 1) + MINDOTS;
-	else
-		/* Absolute minimum */
-	if (sp->dots < 1)
-		sp->dots = 1;
+	sp->redrawing = 1;
+	sp->redrawpos = 0;
 }
