@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)swarm.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)swarm.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -23,10 +24,10 @@ static char sccsid[] = "@(#)swarm.c	3.9 96/05/25 xlockmore";
 /* Macros */
 #define X(t,b)	(sp->x[(t)*sp->beecount+(b)])
 #define Y(t,b)	(sp->y[(t)*sp->beecount+(b)])
-#define balance_rand(v)	((LRAND()%(v))-((v)/2))		/* random number around 0 */
+#define balance_rand(v)	((NRAND(v))-((v)/2))	/* random number around 0 */
 
 ModeSpecOpt swarm_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 typedef struct {
 	int         pix;
@@ -46,24 +47,38 @@ typedef struct {
 	short       wyv;
 } swarmstruct;
 
-static swarmstruct swarms[MAXSCREENS];
+static swarmstruct *swarms = NULL;
 
 void
 init_swarm(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	swarmstruct *sp = &swarms[screen];
+	swarmstruct *sp;
 	int         b;
 
-	sp->beecount = MI_BATCHCOUNT(mi);
+	if (swarms == NULL) {
+		if ((swarms = (swarmstruct *) calloc(MI_NUM_SCREENS(mi),
+					      sizeof (swarmstruct))) == NULL)
+			return;
+	}
+	sp = &swarms[MI_SCREEN(mi)];
 
+	sp->beecount = MI_BATCHCOUNT(mi);
+	if (sp->beecount < 0) {
+		/* if sp->beecount is random ... the size can change */
+		if (sp->segs != NULL) {
+			(void) free((void *) sp->segs);
+			(void) free((void *) sp->old_segs);
+			sp->segs = NULL;
+			sp->old_segs = NULL;
+		}
+		sp->beecount = NRAND(-sp->beecount) + 1;	/* Add 1 so its not too boring */
+	}
 	sp->width = MI_WIN_WIDTH(mi);
 	sp->height = MI_WIN_HEIGHT(mi);
 	sp->border = (sp->width + sp->height) / 50;
 
 	/* Clear the background. */
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, sp->width, sp->height);
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 
 	/* Allocate memory. */
 
@@ -78,8 +93,8 @@ init_swarm(ModeInfo * mi)
 	/* Initialize point positions, velocities, etc. */
 
 	/* wasp */
-	sp->wx[0] = sp->border + LRAND() % (sp->width - 2 * sp->border);
-	sp->wy[0] = sp->border + LRAND() % (sp->height - 2 * sp->border);
+	sp->wx[0] = sp->border + NRAND(sp->width - 2 * sp->border);
+	sp->wy[0] = sp->border + NRAND(sp->height - 2 * sp->border);
 	sp->wx[1] = sp->wx[0];
 	sp->wy[1] = sp->wy[0];
 	sp->wxv = 0;
@@ -87,22 +102,22 @@ init_swarm(ModeInfo * mi)
 
 	/* bees */
 	for (b = 0; b < sp->beecount; b++) {
-		X(0, b) = LRAND() % sp->width;
+		X(0, b) = NRAND(sp->width);
 		X(1, b) = X(0, b);
-		Y(0, b) = LRAND() % sp->height;
+		Y(0, b) = NRAND(sp->height);
 		Y(1, b) = Y(0, b);
 		sp->xv[b] = balance_rand(7);
 		sp->yv[b] = balance_rand(7);
 	}
 }
 
-
-
 void
 draw_swarm(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
-	swarmstruct *sp = &swarms[screen];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
+	swarmstruct *sp = &swarms[MI_SCREEN(mi)];
 	int         b;
 
 	/* <=- Wasp -=> */
@@ -139,8 +154,8 @@ draw_swarm(ModeInfo * mi)
 		sp->wy[0] += sp->wyv;
 	}
 	/* Don't let things settle down. */
-	sp->xv[LRAND() % sp->beecount] += balance_rand(3);
-	sp->yv[LRAND() % sp->beecount] += balance_rand(3);
+	sp->xv[NRAND(sp->beecount)] += balance_rand(3);
+	sp->yv[NRAND(sp->beecount)] += balance_rand(3);
 
 	/* <=- Bees -=> */
 	for (b = 0; b < sp->beecount; b++) {
@@ -186,18 +201,43 @@ draw_swarm(ModeInfo * mi)
 		sp->old_segs[b].y2 = Y(2, b);
 	}
 
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XDrawLine(dsp, win, Scr[screen].gc,
+	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XDrawLine(display, window, gc,
 		  sp->wx[1], sp->wy[1], sp->wx[2], sp->wy[2]);
-	XDrawSegments(dsp, win, Scr[screen].gc, sp->old_segs, sp->beecount);
+	XDrawSegments(display, window, gc, sp->old_segs, sp->beecount);
 
-	XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
-	XDrawLine(dsp, win, Scr[screen].gc,
+	XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	XDrawLine(display, window, gc,
 		  sp->wx[0], sp->wy[0], sp->wx[1], sp->wy[1]);
-	if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2) {
-		XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[sp->pix]);
-		if (++sp->pix >= Scr[screen].npixels)
+	if (MI_NPIXELS(mi) > 2) {
+		XSetForeground(display, gc, MI_PIXEL(mi, sp->pix));
+		if (++sp->pix >= MI_NPIXELS(mi))
 			sp->pix = 0;
 	}
-	XDrawSegments(dsp, win, Scr[screen].gc, sp->segs, sp->beecount);
+	XDrawSegments(display, window, gc, sp->segs, sp->beecount);
+}
+
+void
+release_swarm(ModeInfo * mi)
+{
+	if (swarms != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			swarmstruct *sp = &swarms[screen];
+
+			if (sp->segs != NULL)
+				(void) free((void *) sp->segs);
+			if (sp->old_segs != NULL)
+				(void) free((void *) sp->old_segs);
+		}
+		(void) free((void *) swarms);
+		swarms = NULL;
+	}
+}
+
+void
+refresh_swarm(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
 }
