@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)utils.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)utils.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -104,7 +105,7 @@ seconds(void)
 #endif
 }
 
-int
+static int
 readable(char *filename)
 {
 	FILE       *file;
@@ -176,13 +177,13 @@ strcat_firstword(char *fword, char *words)
 }
 
 int
-is_ribbon(void)
+isRibbon(void)
 {
 	return (getwordsfrom == FROM_RESRC);
 }
 
 char       *
-get_words(void)
+getWords(void)
 {
 	FILE       *pp;
 	static char buf[BUFSIZ], progerr[BUFSIZ], progrun[BUFSIZ];
@@ -224,13 +225,15 @@ get_words(void)
 					(void) sprintf(buf, "\"%s\" produced no output!",
 					 (!program) ? DEF_PROGRAM : program);
 				else {
+					(void) memset((char *) progerr, 0, sizeof (progerr));
 					(void) strcpy(progerr, "sh: ");
 					strcat_firstword(progerr, (!program) ? DEF_PROGRAM : program);
 					(void) strcat(progerr, ": not found\n");
 					if (!strcmp(buf, progerr))
-						switch (LRAND() % 7) {
+						switch (NRAND(12)) {
 							case 0:
-								(void) strcat(buf, "( Get with the program, bub. )\n");
+								(void) strcat(buf,
+									      "( Get with the program, bub. )\n");
 								break;
 							case 1:
 								(void) strcat(buf,
@@ -238,21 +241,36 @@ get_words(void)
 								break;
 							case 2:
 								(void) strcat(buf,
-									      "\nThe resource you want to\nset is `program'\n");
+									      "\nThe resource you want to\nset is `program'.\n");
 								break;
 							case 3:
-								(void) strcat(buf, "\nHelp!!  Help!!\nAAAAAAGGGGHHH!!  \n\n");
+								(void) strcat(buf,
+									      "\nHelp!!  Help!!\nAAAAAAGGGGHHH!!  \n\n");
 								break;
 							case 4:
-								(void) strcat(buf, " Hey, who called me `Big Nose'? \n");
+								(void) strcat(buf,
+									      "( Hey, who called me `Big Nose'? )\n");
 								break;
 							case 5:
-								(void) strcat(buf, "( Hello?  Are you paying attention? )\n");
+								(void) strcat(buf,
+									      "( Hello?  Are you paying attention? )\n");
 								break;
 							case 6:
 								(void) strcat(buf,
 									      "sh: what kind of fool do you take me for? \n");
 								break;
+							case 7:
+								(void) strcat(buf,
+									      "\nRun me with -program \"fortune -o\".\n");
+								break;
+							case 8:
+								(void) strcat(buf,
+									      "( Where is your fortune? )\n");
+								break;
+							case 9:
+								(void) strcat(buf,
+									      "( Your fortune has not been written yet!! )");
+								break;
 						}
 				}
 				p = buf;
@@ -275,9 +293,9 @@ get_words(void)
 						(void) fgets(p, sizeof (buf) - strlen(buf), pp);
 						/* get first '%%' (the one after the number of quotes) */
 						(void) fgets(p, sizeof (buf) - strlen(buf), pp);
-						no_quote = LRAND() % len_mess_file;
+						no_quote = NRAND(len_mess_file);
 						for (i = 0; i <= no_quote; ++i) {
-							int         len_cur = 0;
+							unsigned int len_cur = 0;
 
 							buf[0] = '\0';
 							p = buf + strlen(buf);
@@ -342,7 +360,7 @@ get_words(void)
 }
 
 XFontStruct *
-get_font(Display * display)
+getFont(Display * display)
 {
 	XFontStruct *mode_font;
 
@@ -362,20 +380,20 @@ get_font(Display * display)
 	return mode_font;
 }
 
-void
+static void
 randomImage(char *im_file)
 {
 #if !defined( VMS ) || defined( XVMSUTILS ) || ( __VMS_VER >= 70000000 )
-  extern char directory_[512];
-  extern struct dirent **image_list;
-  extern int  num_list;
-  int         num;
- 
-  if (num_list > 0) {
-    num = LRAND() % num_list;
-    (void) strcpy(im_file, directory_);
-    (void) strcat(im_file, image_list[num]->d_name);
-  }
+	extern char directory_[512];
+	extern struct dirent **image_list;
+	extern int  num_list;
+	int         num;
+
+	if (num_list > 0) {
+		num = NRAND(num_list);
+		(void) strcpy(im_file, directory_);
+		(void) strcat(im_file, image_list[num]->d_name);
+	}
 #endif
 }
 
@@ -493,7 +511,8 @@ sel_image(struct dirent *name)
 #define _MEMBL_ 64
 int
 scan_dir(const char *directoryname, struct dirent ***namelist,
- int         (*select) (void *), int (*compare) (const void *, const void *))
+	 int         (*select) (struct dirent *),
+	 int         (*compare) (const void *, const void *))
 {
 	DIR        *dir_;
 	struct dirent *new_entry, **namelist_;
@@ -538,3 +557,153 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 }
 
 #endif
+
+extern int  XbmReadFileToImage(char *filename,
+			       int *width, int *height, unsigned char **bits);
+
+#ifdef HAS_XPM
+#if 1
+#include <X11/xpm.h>
+#else
+#include <xpm.h>
+#endif
+#endif
+#include "ras.h"
+
+static XImage blogo =
+{
+	0, 0,			/* width, height */
+	0, XYBitmap, 0,		/* xoffset, format, data */
+	LSBFirst, 8,		/* byte-order, bitmap-unit */
+	LSBFirst, 8, 1		/* bitmap-bit-order, bitmap-pad, depth */
+};
+
+void
+getImage(ModeInfo * mi, XImage ** logo,
+	 int width, int height, unsigned char *bits, char **name,
+	 int *graphics_format, Bool * newcolormap)
+{
+	extern char *imagefile;
+	Display    *display = MI_DISPLAY(mi);
+
+#ifdef HAS_XPM
+	XpmAttributes attrib;
+	int         screen = MI_SCREEN(mi);
+
+	attrib.visual = DefaultVisual(display, screen);
+	attrib.colormap = DefaultColormap(display, screen);
+	attrib.depth = DefaultDepth(display, screen);
+	attrib.valuemask = XpmVisual | XpmColormap | XpmDepth;
+#endif
+
+	*newcolormap = False;
+	*graphics_format = 0;
+
+	randomImage(imagefile);
+
+	if (strlen(imagefile))
+		if (readable(imagefile)) {
+			if (MI_NPIXELS(mi) > 2) {
+				if (RasterSuccess == RasterFileToImage(display, imagefile, logo
+				/* , MI_FG_COLOR(mi), MI_BG_COLOR(mi) */ ))
+					*graphics_format = IS_RASTERFILE;
+				if (*graphics_format == IS_RASTERFILE &&
+				    MI_WIN_IS_INSTALL(mi) && !MI_WIN_IS_INROOT(mi))
+					*newcolormap = True;
+#ifdef HAS_XPM
+				if (*graphics_format <= 0)
+					if (XpmSuccess == XpmReadFileToImage(display, imagefile, logo,
+						  (XImage **) NULL, &attrib))
+						*graphics_format = IS_XPMFILE;
+#endif
+			}
+			if (*graphics_format <= 0)
+				if (!blogo.data) {
+					if (BitmapSuccess == XbmReadFileToImage(imagefile,
+										&blogo.width, &blogo.height, (unsigned char **) &blogo.data)) {
+						blogo.bytes_per_line = (blogo.width + 7) / 8;
+						*graphics_format = IS_XBMFILE;
+						*logo = &blogo;
+					}
+				} else {
+					*graphics_format = IS_XBMDONE;
+					*logo = &blogo;
+				}
+			if (*graphics_format <= 0)
+				(void) fprintf(stderr,
+					       "\"%s\" is in an unrecognized format\n", imagefile);
+		} else
+			(void) fprintf(stderr,
+				  "could not read file \"%s\"\n", imagefile);
+#ifdef HAS_XPM
+	if (MI_NPIXELS(mi) > 2 && *graphics_format <= 0)
+		if (XpmSuccess == XpmCreateImageFromData(display, name,
+					    logo, (XImage **) NULL, &attrib))
+			*graphics_format = IS_XPM;
+#endif
+	if (*graphics_format <= 0) {
+		if (!blogo.data) {
+			blogo.data = (char *) bits;
+			blogo.width = width;
+			blogo.height = height;
+			blogo.bytes_per_line = (blogo.width + 7) / 8;
+			*graphics_format = IS_XBM;
+		} else
+			*graphics_format = IS_XBMDONE;
+		*logo = &blogo;
+	}
+}
+
+void
+destroyImage(XImage * logo, int graphics_format)
+{
+	switch (graphics_format) {
+#if 0
+		case IS_XBM:
+			blogo.data = NULL;
+			break;
+#endif
+		case IS_XBMFILE:
+			if (blogo.data) {
+				(void) free((void *) blogo.data);
+				blogo.data = NULL;
+			}
+			break;
+		case IS_XPM:
+		case IS_XPMFILE:
+		case IS_RASTERFILE:
+			if (logo)
+				XDestroyImage(logo);
+			break;
+	}
+	logo = NULL;
+}
+
+/* This stops some flashing, could be more efficient */
+void
+XEraseImage(Display * display, Window window, GC gc,
+	    int x, int y, int xlast, int ylast, int xsize, int ysize)
+{
+	if (ylast < y) {
+		if (y < ylast + ysize)
+			XFillRectangle(display, window, gc, xlast, ylast, xsize, y - ylast);
+		else
+			XFillRectangle(display, window, gc, xlast, ylast, xsize, ysize);
+	} else if (ylast > y) {
+		if (y > ylast - ysize)
+			XFillRectangle(display, window, gc, xlast, y + ysize, xsize, ylast - y);
+		else
+			XFillRectangle(display, window, gc, xlast, ylast, xsize, ysize);
+	}
+	if (xlast < x) {
+		if (x < xlast + xsize)
+			XFillRectangle(display, window, gc, xlast, ylast, x - xlast, ysize);
+		else
+			XFillRectangle(display, window, gc, xlast, ylast, xsize, ysize);
+	} else if (xlast > x) {
+		if (x > xlast - xsize)
+			XFillRectangle(display, window, gc, x + xsize, ylast, xlast - x, ysize);
+		else
+			XFillRectangle(display, window, gc, xlast, ylast, xsize, ysize);
+	}
+}
