@@ -68,8 +68,7 @@ static char vroot_rcsid[] = "$Id: vroot.h,v 1.1 1994/08/25 22:04:24 jwz Exp $";
 #include <X11/Xlib.h>
 
 static      Window
-VirtualRootWindowOfScreen(screen)
-	Screen     *screen;
+VirtualRootWindowOfScreen(Screen * screen)
 {
 	static Screen *save_screen = (Screen *) 0;
 	static Window root = (Window) 0;
@@ -87,7 +86,7 @@ VirtualRootWindowOfScreen(screen)
 		__SWM_VROOT = XInternAtom(dpy, "__SWM_VROOT", False);
 		if (XQueryTree(dpy, root, &rootReturn, &parentReturn,
 			       &children, &numChildren)) {
-			for (i = 0; i < numChildren; i++) {
+			for (i = 0; i < (int) numChildren; i++) {
 				Atom        actual_type;
 				int         actual_format;
 				unsigned long nitems, bytesafter;
