@@ -1,5 +1,6 @@
+
 #ifndef lint
-static char sccsid[] = "@(#)world.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)world.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -52,12 +53,13 @@ static char sccsid[] = "@(#)world.c	3.9 96/05/25 xlockmore";
 #include "bitmaps/terra-29.xbm"
 
 #define NUM_EARTHS 30
-#define SIZE_X 64
-#define SIZE_Y 64
+#define SIZE_X terra00_width	/* 64 */
+#define SIZE_Y terra00_height	/* 64 */
 #define NUM_REV 4
+#define MINWORLDS 1
 
 ModeSpecOpt world_opts =
-{0, NULL, NULL, NULL};
+{0, NULL, 0, NULL, NULL};
 
 static XImage Earths[NUM_EARTHS] =
 {
@@ -92,15 +94,13 @@ static XImage Earths[NUM_EARTHS] =
 	{SIZE_X, SIZE_Y, 0, XYBitmap, (char *) terra28_bits, LSBFirst, 8, LSBFirst, 8, 1},
 	{SIZE_X, SIZE_Y, 0, XYBitmap, (char *) terra29_bits, LSBFirst, 8, LSBFirst, 8, 1}};
 
-#define MAXICONS 64
-
 typedef struct {
 	int         x;
 	int         y;
 	unsigned long color;
 	int         frame;
 	int         direction;
-} point;
+} planetstruct;
 
 typedef struct {
 	int         width;
@@ -110,107 +110,135 @@ typedef struct {
 	int         xb;
 	int         yb;
 	int         frame_num;
-	int         iconmode;
-	int         iconcount;
-	point       icons[MAXICONS];
+	int         nplanets;
+	planetstruct *planets;
 } worldstruct;
 
-static worldstruct worlds[MAXSCREENS];
-
-void
-draw_world(ModeInfo * mi)
-{
-	Window      win = MI_WINDOW(mi);
-	worldstruct *wp = &worlds[screen];
-	int         i;
-
-#ifndef NOFLASH
-	int         col[MAXICONS], j;
-
-	for (i = 0; i < wp->ncols; i++)
-		col[i] = 0;
-#endif
-	if (wp->frame_num == NUM_EARTHS * NUM_REV) {
-		wp->frame_num = 0;
-		XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-		for (i = 0; i < wp->iconcount; i++) {
-			if (!wp->iconmode)
-				XFillRectangle(dsp, win, Scr[screen].gc,
-					    wp->xb + SIZE_X * wp->icons[i].x,
-					    wp->yb + SIZE_Y * wp->icons[i].y,
-					       SIZE_X, SIZE_Y);
-#ifdef NOFLASH
-			wp->icons[i].x = LRAND() % wp->ncols;
-#else
-			do {
-				j = LRAND() % wp->ncols;
-				if (!col[j])
-					wp->icons[i].x = j;
-				col[j]++;
-			} while (col[j] > 1);
-#endif
-			wp->icons[i].y = LRAND() % wp->nrows;
-			wp->icons[i].direction = LRAND() & 1;
-			wp->icons[i].frame = LRAND() % NUM_EARTHS;
-			if (!MI_WIN_IS_MONO(mi) && MI_NPIXELS(mi) > 2)
-				wp->icons[i].color =
-					Scr[screen].pixels[LRAND() % Scr[screen].npixels];
-			else
-				wp->icons[i].color = WhitePixel(dsp, screen);
-		}
-	}
-	for (i = 0; i < wp->iconcount; i++) {
-		XSetForeground(dsp, Scr[screen].gc, wp->icons[i].color);
-		if (wp->icons[i].frame == NUM_EARTHS)
-			wp->icons[i].frame = 0;
-		else {
-			if ((wp->icons[i].frame < 0) && wp->icons[i].direction == 0)
-				wp->icons[i].frame = NUM_EARTHS - 1;
-		}
-		XPutImage(dsp, win, Scr[screen].gc, (Earths + wp->icons[i].frame),
-			  0, 0,
-			  wp->xb + SIZE_X * wp->icons[i].x,
-			  wp->yb + SIZE_Y * wp->icons[i].y,
-			  SIZE_X, SIZE_Y);
-
-		(wp->icons[i].direction) ? wp->icons[i].frame++ : wp->icons[i].frame--;
-	}
-	wp->frame_num++;
-}
+static worldstruct *worlds = NULL;
 
 void
 init_world(ModeInfo * mi)
 {
-	Window      win = MI_WINDOW(mi);
+	worldstruct *wp;
 	int         x;
-	worldstruct *wp = &worlds[screen];
 
+	if (worlds == NULL) {
+		if ((worlds = (worldstruct *) calloc(MI_NUM_SCREENS(mi),
+					      sizeof (worldstruct))) == NULL)
+			return;
+	}
+	wp = &worlds[MI_SCREEN(mi)];
 	wp->frame_num = NUM_EARTHS * NUM_REV;
 	for (x = 0; x < NUM_EARTHS; x++)
 		Earths[x].bytes_per_line = 8;
 	wp->width = MI_WIN_WIDTH(mi);
 	wp->height = MI_WIN_HEIGHT(mi);
 	wp->ncols = wp->width / SIZE_X;
+	if (!wp->ncols)
+		wp->ncols = 1;
 	wp->nrows = wp->height / SIZE_Y;
-	wp->iconmode = (wp->ncols < 2 || wp->nrows < 2);
-	if (wp->iconmode) {
-		wp->xb = 0;
-		wp->yb = 0;
-		wp->iconcount = 1;	/* icon mode */
-	} else {
-		wp->xb = (wp->width - SIZE_X * wp->ncols) / 2;
-		wp->yb = (wp->height - SIZE_Y * wp->nrows) / 2;
-		wp->iconcount = MI_BATCHCOUNT(mi);
-#ifdef NOFLASH
-		if (wp->iconcount > MAXICONS)
-			wp->iconcount = 16;
-#else
-		if (wp->iconcount > MAXICONS)
-			wp->iconcount = 8;
-		if (wp->iconcount > wp->ncols)
-			wp->iconcount = wp->ncols;
+	if (!wp->nrows)
+		wp->nrows = 1;
+	wp->xb = (wp->width - SIZE_X * wp->ncols) / 2;
+	wp->yb = (wp->height - SIZE_Y * wp->nrows) / 2;
+	wp->nplanets = MI_BATCHCOUNT(mi);
+	if (wp->nplanets < -MINWORLDS)
+		wp->nplanets = NRAND(-wp->nplanets - MINWORLDS + 1) + MINWORLDS;
+	else if (wp->nplanets < MINWORLDS)
+		wp->nplanets = MINWORLDS;
+
+	if (wp->nplanets > wp->ncols * wp->nrows)
+		wp->nplanets = wp->ncols * wp->nrows;
+#ifndef NOFLASH
+	if (wp->nplanets > wp->ncols)
+		wp->nplanets = wp->ncols;
 #endif
+	if (wp->planets != NULL)
+		(void) free((void *) wp->planets);
+	wp->planets = (planetstruct *) malloc(wp->nplanets * sizeof (planetstruct));
+
+	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+}
+
+void
+draw_world(ModeInfo * mi)
+{
+	Display    *display = MI_DISPLAY(mi);
+	GC          gc = MI_GC(mi);
+	worldstruct *wp = &worlds[MI_SCREEN(mi)];
+	int         i;
+
+#ifndef NOFLASH
+	int        *col, j;
+
+	if ((col = (int *) calloc(wp->ncols, sizeof (int))) == NULL)
+		            return;
+
+#endif
+	if (wp->frame_num == NUM_EARTHS * NUM_REV) {
+		wp->frame_num = 0;
+		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		for (i = 0; i < wp->nplanets; i++) {
+			XFillRectangle(display, MI_WINDOW(mi), gc,
+				       wp->xb + SIZE_X * wp->planets[i].x, wp->yb + SIZE_Y * wp->planets[i].y,
+				       SIZE_X, SIZE_Y);
+#ifdef NOFLASH
+			wp->planets[i].x = NRAND(wp->ncols);
+#else
+			do {
+				j = NRAND(wp->ncols);
+				if (!col[j])
+					wp->planets[i].x = j;
+				col[j]++;
+			} while (col[j] > 1);
+#endif
+			wp->planets[i].y = NRAND(wp->nrows);
+			wp->planets[i].direction = LRAND() & 1;
+			wp->planets[i].frame = NRAND(NUM_EARTHS);
+			if (MI_NPIXELS(mi) > 2)
+				wp->planets[i].color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+			else
+				wp->planets[i].color = MI_WIN_WHITE_PIXEL(mi);
+		}
 	}
-	XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
-	XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, wp->width, wp->height);
+#ifndef NOFLASH
+	(void) free((void *) col);
+#endif
+	for (i = 0; i < wp->nplanets; i++) {
+		XSetForeground(display, gc, wp->planets[i].color);
+		if (wp->planets[i].frame == NUM_EARTHS)
+			wp->planets[i].frame = 0;
+		else {
+			if ((wp->planets[i].frame < 0) && wp->planets[i].direction == 0)
+				wp->planets[i].frame = NUM_EARTHS - 1;
+		}
+		XPutImage(display, MI_WINDOW(mi), gc, (Earths + wp->planets[i].frame),
+			  0, 0,
+			  wp->xb + SIZE_X * wp->planets[i].x,
+			  wp->yb + SIZE_Y * wp->planets[i].y,
+			  SIZE_X, SIZE_Y);
+
+		(wp->planets[i].direction) ? wp->planets[i].frame++ : wp->planets[i].frame--;
+	}
+	wp->frame_num++;
+}
+
+void
+release_world(ModeInfo * mi)
+{
+	if (worlds != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			if (worlds[screen].planets != NULL)
+				(void) free((void *) worlds[screen].planets);
+		(void) free((void *) worlds);
+		worlds = NULL;
+	}
+}
+
+void
+refresh_world(ModeInfo * mi)
+{
+	/* Do nothing, it will refresh by itself */
 }
