@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)xlock.c	3.9 96/05/25 xlockmore";
+static char sccsid[] = "@(#)xlock.c	3.11 96/09/20 xlockmore";
 
 #endif
 
@@ -23,13 +23,16 @@ static char sccsid[] = "@(#)xlock.c	3.9 96/05/25 xlockmore";
  * Revision History:
  *
  * Changes of David Bagley <bagleyd@hertz.njit.edu>
- * 18-Mar-96: Ron Hitchens <ron@utw.com>
+ * 01-Sep-96: Ron Hitchens <ronh@utw.com>
+ *            Updated xlock so it would refresh more reliably and
+ *            handle window resizing.
+ * 18-Mar-96: Ron Hitchens <ronh@utw.com>
  *            Implemented new ModeInfo hook calling scheme.
  *            Created mode_info() to gather and pass info to hooks.
  *            Watch for and use MI_PAUSE value if mode hook sets it.
  *            Catch SIGSEGV, SIGBUS and SIGFPE signals.  Other should
  *            be caught.  Eliminate some globals.
- * 23-Dec-95: Ron Hitchens <ron@utw.com>
+ * 23-Dec-95: Ron Hitchens <ronh@utw.com>
  *            Rewrote event loop so as not to use signals.
  * 01-Sep-95: initPasswd function, more stuff removed to passwd.c
  * 24-Jun-95: Cut out passwd stuff to passwd.c-> getPasswd & checkPasswd
@@ -155,7 +158,7 @@ static char sccsid[] = "@(#)xlock.c	3.9 96/05/25 xlockmore";
  */
 
 #include <signal.h>
-#if defined( __STDC__ ) || defined( VMS ) || defined( AIXV3 )
+#ifndef NONSTDC
 #include <stdarg.h>
 #endif
 #include <errno.h>
@@ -200,7 +203,7 @@ static char sccsid[] = "@(#)xlock.c	3.9 96/05/25 xlockmore";
 #endif
 
 extern char *getenv(const char *);
-extern void CheckResources(void);
+extern void checkResources(void);
 extern void initPasswd(void);
 extern int  checkPasswd(char *);
 
@@ -210,15 +213,19 @@ extern int  usleep(unsigned int);
 #endif
 
 #if defined( AUTO_LOGOUT ) || defined( LOGOUT_BUTTON )
-extern void logoutUser();
+extern void logoutUser(void);
 
 #endif
 
-#ifndef AIXV3
+#if 0				/* #if !defined( AIXV3 ) && !defined( __hpux ) && !defined( __bsdi__ ) */
 extern int  select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
+
 #endif
 extern int  nice(int);
 
+char       *ProgramName;	/* argv[0] */
+perscreen   Scr[MAXSCREENS];
+Display    *dsp = NULL;		/* server display connection */
 
 extern char user[PASSLENGTH];
 extern float saturation;
@@ -236,6 +243,7 @@ extern Bool enablesaver;
 extern Bool grabmouse;
 extern Bool inroot;
 extern Bool install;
+extern Bool debug;
 extern Bool inwindow;
 extern Bool mono;
 extern Bool nolock;
@@ -244,11 +252,6 @@ extern Bool usefirst;
 extern Bool use3d;
 extern Bool verbose;
 
-char       *ProgramName;	/* argv[0] */
-perscreen   Scr[MAXSCREENS];
-Display    *dsp = NULL;		/* server display connection */
-int         screen;		/* current screen */
-
 extern char *fontname;
 extern char *background;
 extern char *foreground;
@@ -258,8 +261,11 @@ extern char *text_info;
 extern char *text_valid;
 extern char *text_invalid;
 extern char *geometry;
+extern char *icongeometry;
+extern char *none3d;
 extern char *right3d;
 extern char *left3d;
+extern char *both3d;
 
 #if defined( HAS_RPLAY ) || defined( VMS_PLAY ) || defined( DEF_PLAY )
 extern char *locksound;
@@ -287,7 +293,7 @@ extern Bool dtsaver;
 #endif
 
 static int  onepause = 0;
-
+static int  screen = 0;		/* current screen */
 
 static int  screens;		/* number of screens */
 static Window win[MAXSCREENS];	/* window used to cover screen */
@@ -301,8 +307,10 @@ static Window root[MAXSCREENS];	/* convenience pointer to the root window */
 static GC   textgc[MAXSCREENS];	/* graphics context used for text rendering */
 static unsigned long fgcol[MAXSCREENS];		/* used for text rendering */
 static unsigned long bgcol[MAXSCREENS];		/* background of text screen */
-static long rightcol[MAXSCREENS];
-static long leftcol[MAXSCREENS];
+static unsigned long nonecol[MAXSCREENS];
+static unsigned long rightcol[MAXSCREENS];
+static unsigned long leftcol[MAXSCREENS];
+static unsigned long bothcol[MAXSCREENS];	/* Can change with -install */
 static int  iconx[MAXSCREENS];	/* location of left edge of icon */
 static int  icony[MAXSCREENS];	/* location of top edge of icon */
 static Cursor mycursor;		/* blank cursor */
@@ -310,8 +318,8 @@ static Pixmap lockc;
 static Pixmap lockm;		/* pixmaps for cursor and mask */
 static char no_bits[] =
 {0};				/* dummy array for the blank cursor */
-static int  passx;		/* position of the ?'s */
-static int  passy;
+static int  passx, passy;	/* position of the ?'s */
+static int  iconwidth, iconheight;
 static XFontStruct *font;
 static int  sstimeout;		/* screen saver parameters */
 static int  ssinterval;
@@ -349,14 +357,14 @@ system_play(char *string)
 {
 	char        progrun[BUFSIZ];
 
-	(void) sprintf(progrun, "( %s%s.au ) 2>&1", DEF_PLAY, string);
+	(void) sprintf(progrun, "( %s%s ) 2>&1", DEF_PLAY, string);
 	system(progrun);
 }
 
 #endif
 
 /* VARARGS1 */
-#if defined( __STDC__ ) || defined( VMS ) || defined( AIXV3 )
+#ifndef NONSTDC
 void
 error(char *s1,...)
 {
@@ -376,7 +384,7 @@ error(char *s1, char *s2)
 	exit(1);
 }
 
-#if defined( __STDC__ ) || defined( VMS ) || defined( AIXV3 )
+#ifndef NONSTDC
 void
 warning(char *s1,...)
 {
@@ -423,7 +431,6 @@ XUngrabHosts(Display * display)
 }
 
 
-#ifndef DEBUG
 /* Simple wrapper to get an asynchronous grab on the keyboard and mouse. If
    either grab fails, we sleep for one second and try again since some window
    manager might have had the mouse grabbed to drive the menu choice that
@@ -458,21 +465,50 @@ GrabKeyboardAndMouse(void)
 	}
 }
 
-#endif
-
-
 /* Assuming that we already have an asynch grab on the pointer, just grab it
    again with a new cursor shape and ignore the return code. */
 static void
 XChangeGrabbedCursor(Cursor cursor)
 {
-#ifndef DEBUG
-	if (grabmouse && !inwindow && !inroot)
+  if (!debug && grabmouse && !inwindow && !inroot)
 		(void) XGrabPointer(dsp, win[0], True, AllPointerEventMask,
 		    GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
-#endif
 }
 
+/* This is a private data structure, don't touch */
+static ModeInfo modeinfo[MAXSCREENS];
+
+/* 
+ *    Return True of False indicating if the given window has changed
+ *    size relative to the window geometry cached in the mode_info
+ *    struct.  If the window handle given does not match the one in
+ *    the cache, or if the width/height are not the same, then True
+ *    is returned and the window handle in the cache is cleared.
+ *    This causes mode_info() to reload the window info next time
+ *    it is called.
+ *    This function has non-obvious side-effects.  I feel so dirty.  Rh
+ */
+
+static int
+window_size_changed(int scrn, Window window)
+{
+	XWindowAttributes xgwa;
+	ModeInfo   *mi = &modeinfo[scrn];
+
+	if (MI_WINDOW(mi) != window) {
+		MI_WINDOW(mi) = None;	/* forces reload on next mode_info() */
+		return (True);
+	} else {
+		(void) XGetWindowAttributes(dsp, window, &xgwa);
+		if ((MI_WIN_WIDTH(mi) != xgwa.width) ||
+		    (MI_WIN_HEIGHT(mi) != xgwa.height)) {
+			MI_WINDOW(mi) = None;
+			return (True);
+		}
+	}
+
+	return (False);
+}
 
 /* 
  *    Return a pointer to an up-to-date ModeInfo struct for the given
@@ -483,13 +519,11 @@ XChangeGrabbedCursor(Cursor cursor)
  *      to acquire info that does change.
  */
 
-static
-ModeInfo   *
+static ModeInfo *
 mode_info(int scrn, Window window, int iconic)
 {
 	XWindowAttributes xgwa;
 	ModeInfo   *mi;
-	static ModeInfo modeinfo[MAXSCREENS];
 
 	mi = &modeinfo[scrn];
 
@@ -517,7 +551,9 @@ mode_info(int scrn, Window window, int iconic)
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_INROOT, inroot);
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_NOLOCK, nolock);
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_INSTALL, install);
-		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_USE3D, use3d);
+		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_DEBUG, debug);
+		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_USE3D, use3d &&
+			    !(mono || CellsOfScreen(MI_SCREENPTR(mi)) <= 2));
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_VERBOSE, verbose);
 
 		MI_WIN_SET_FLAG_STATE(mi, WI_FLAG_INFO_INITTED, True);
@@ -576,6 +612,7 @@ finish(void)
 	}
 	XFlush(dsp);
 	XCloseDisplay(dsp);
+	(void) nice(0);
 }
 
 static int
@@ -638,7 +675,7 @@ putText(Window w, GC gc, char *string, int bold, int left, int *px, int *py)
 #endif
 
 static void
-statusUpdate(int new, int scr)
+statusUpdate(int isnew, int scr)
 {
 	int         left, x, y, now, len;
 	char        buf[1024];
@@ -658,16 +695,16 @@ statusUpdate(int new, int scr)
 #ifdef LOGOUT_BUTTON
 	if (tried_logout && !last_tried_lo) {
 		last_tried_lo = 1;
-		new = 1;
+		isnew = 1;
 #ifdef AUTO_LOGOUT
 		forceLogout = 0;
 #endif
 	}
-	if (new)
+	if (isnew)
 		made_button = 0;
 #endif /* LOGOUT_BUTTON */
 
-	if (new || last_time != len)
+	if (isnew || last_time != len)
 		last_time = len;
 	else
 		return;
@@ -687,7 +724,7 @@ statusUpdate(int new, int scr)
 				       "%d minute%s until public logout button appears.    \n",
 			     loButton - len, loButton - len == 1 ? "" : "s");
 			putText(win[scr], textgc[scr], buf, False, left, &x, &y);
-		} else if (!made_button || (new && tried_logout)) {
+		} else if (!made_button || (isnew && tried_logout)) {
 			made_button = 1;
 			ysave = y;
 			y = xgwa.height * 0.6;
@@ -797,7 +834,7 @@ runMainLoop(int maxtime, int iconscreen)
 	int         fd = ConnectionNumber(dsp), r;
 	struct timeval sleep_time, first, repeat;
 	struct timeval elapsed, tmp;
-	struct fd_set reads;
+	fd_set      reads;
 	long        started;
 
 	first.tv_sec = first.tv_usec = 0;
@@ -915,6 +952,7 @@ ReadXString(char *s, int slen)
 	statusUpdate(True, thisscreen);
 	bp = 0;
 	*s = 0;
+
 	for (;;) {
 		unsigned long lasteventtime = seconds();
 
@@ -933,17 +971,23 @@ ReadXString(char *s, int slen)
 			onepause = 0;
 #else
 			statusUpdate(False, thisscreen);
-			if (runMainLoop(min(timeout, 5), thisscreen) == 0)
+			if (runMainLoop(MIN(timeout, 5), thisscreen) == 0)
 				break;
 #endif
 
-			if (seconds() - lasteventtime > timeout) {
+			if (seconds() - lasteventtime > (unsigned long) timeout) {
 				screen = thisscreen;
 				return 1;
 			}
 		}
+
 		screen = thisscreen;
 		XNextEvent(dsp, &event);
+
+		/*
+		 * This event handling code should be unified with the
+		 * similar code in justDisplay().
+		 */
 		switch (event.type) {
 			case KeyPress:
 				len = XLookupString((XKeyEvent *) & event, keystr, 20, NULL, NULL);
@@ -1018,19 +1062,28 @@ ReadXString(char *s, int slen)
 				}
 #endif
 				break;
-
+			case Expose:
+				if (event.xexpose.count != 0)
+					break;
+				/* fall through on last expose event */
 			case VisibilityNotify:
-				if (event.xvisibility.state != VisibilityUnobscured) {
-#ifndef DEBUG
-					if (!inwindow)
-						XRaiseWindow(dsp, win[screen]);
-#endif
-					call_refresh_hook((LockStruct *) NULL,
-							  mode_info(screen, win[screen], False));
-					s[0] = '\0';
-					return 1;
+				/* window was restacked or exposed */
+				if (!debug && !inwindow)
+					XRaiseWindow(dsp, event.xvisibility.window);
+				call_refresh_hook((LockStruct *) NULL,
+				      mode_info(screen, win[screen], False));
+				s[0] = '\0';
+				return 1;
+			case ConfigureNotify:
+				/* window config changed */
+				if (!debug && !inwindow)
+					XRaiseWindow(dsp, event.xconfigure.window);
+				if (window_size_changed(screen, win[screen])) {
+					call_init_hook((LockStruct *) NULL,
+						       mode_info(screen, win[screen], False));
 				}
-				break;
+				s[0] = '\0';
+				return 1;
 
 			case KeymapNotify:
 			case KeyRelease:
@@ -1041,6 +1094,12 @@ ReadXString(char *s, int slen)
 			case LeaveNotify:
 			case EnterNotify:
 			case NoExpose:
+			case CirculateNotify:
+			case DestroyNotify:
+			case GravityNotify:
+			case MapNotify:
+			case ReparentNotify:
+			case UnmapNotify:
 				break;
 
 			default:
@@ -1059,6 +1118,7 @@ getPassword(void)
 	int         x, y, left, done;
 	char        buffer[PASSLENGTH];
 
+	(void) nice(0);
 	if (!fullscreen)
 		XConfigureWindow(dsp, win[screen], sizeconfiguremask,
 				 &fullsizeconfigure);
@@ -1075,7 +1135,7 @@ getPassword(void)
 	XMapWindow(dsp, icon[screen]);
 	XRaiseWindow(dsp, icon[screen]);
 
-	x = left = iconx[screen] + ICONW + font->max_bounds.width;
+	x = left = iconx[screen] + iconwidth + font->max_bounds.width;
 	y = icony[screen] + font->ascent;
 
 	putText(win[screen], textgc[screen], text_name, True, left, &x, &y);
@@ -1088,7 +1148,9 @@ getPassword(void)
 	passx = x;
 	passy = y;
 
-	y = icony[screen] + ICONH + font->ascent + 2;
+	y += font->ascent + font->descent + 2;
+	if (y < icony[screen] + iconheight + font->ascent + 2)
+		y = icony[screen] + iconheight + font->ascent + 2;
 	x = left = iconx[screen];
 	putText(win[screen], textgc[screen], text_info, False, left, &x, &y);
 	putText(win[screen], textgc[screen], "\n", False, left, &x, &y);
@@ -1170,27 +1232,42 @@ getPassword(void)
 	if (!fullscreen)
 		XConfigureWindow(dsp, win[screen], sizeconfiguremask,
 				 &minisizeconfigure);
+	(void) nice(nicelevel);
 	return 1;
 }
 
+static int
+event_screen(Display * display, Window event_win)
+{
+	int         i;
 
-static void
+	for (i = 0; i < screens; i++) {
+		if (event_win == RootWindow(display, i)) {
+			return (i);
+		}
+	}
+	return (0);
+}
+
+static int
 justDisplay(void)
 {
+	int         timetodie = False;
+	int         not_done = True;
 	XEvent      event;
 
 	for (screen = 0; screen < screens; screen++) {
 		call_init_hook((LockStruct *) NULL, mode_info(screen, win[screen], False));
 	}
 
-	do {
+	while (not_done) {
 		while (!XPending(dsp)) {
 #ifdef OLD_EVENT_LOOP
 			(void) usleep(onepause ? onepause : delay);
 			onepause = 0;
 			for (screen = 0; screen < screens; screen++)
-				call_callback_hook((LockStruct *) NULL, mode_info(screen,
-							win[screen], False));
+				call_callback_hook((LockStruct *) NULL,
+				      mode_info(screen, win[screen], False));
 #ifdef AUTO_LOGOUT
 			checkLogout();
 #endif
@@ -1206,48 +1283,96 @@ justDisplay(void)
 #endif /* !OLD_EVENT_LOOP */
 		}
 		XNextEvent(dsp, &event);
-#ifndef DEBUG
-		if (event.type == VisibilityNotify) {
-			if (!inwindow) {
-				XRaiseWindow(dsp, event.xany.window);
-			}
-			for (screen = 0; screen < screens; screen++) {
-				call_refresh_hook((LockStruct *) NULL,
-				      mode_info(screen, win[screen], False));
-			}
-		}
-#endif
-		if ((event.type == ButtonPress) &&
-		    (((XButtonEvent *) & event)->button == Button2)) {
-			for (screen = 0; screen < screens; screen++) {
-				call_change_hook((LockStruct *) NULL,
-				      mode_info(screen, win[screen], False));
-			}
-			event.type = 0;		/* dummy out event type */
-		}
-	} while (event.type != ButtonPress &&
+
+		/*
+		 * This event handling code should be unified with the
+		 * similar code in ReadXString().
+		 */
+		switch (event.type) {
+			case Expose:
+				if (event.xexpose.count != 0) {
+					break;
+				}
+				/* fall through on last expose event of the series */
+
+			case VisibilityNotify:
+				if (!debug && !inwindow) {
+					XRaiseWindow(dsp, event.xany.window);
+				}
+				for (screen = 0; screen < screens; screen++) {
+					call_refresh_hook((LockStruct *) NULL,
+							  mode_info(screen, win[screen], False));
+				}
+				break;
+
+			case ConfigureNotify:
+				if (!debug && !inwindow)
+					XRaiseWindow(dsp, event.xconfigure.window);
+				for (screen = 0; screen < screens; screen++) {
+					if (window_size_changed(screen, win[screen])) {
+						call_init_hook((LockStruct *) NULL,
+							       mode_info(screen, win[screen], False));
+					}
+				}
+				break;
+
+			case ButtonPress:
+				if (event.xbutton.button == Button2) {
+					/* call change hook only for clicked window? */
+					for (screen = 0; screen < screens; screen++) {
+						call_change_hook((LockStruct *) NULL,
+						mode_info(screen, win[screen],
+							  False));
+					}
+				} else {
+					screen = event_screen(dsp, event.xbutton.root);
+					not_done = False;
+				}
+				break;
+
 #ifdef MOUSE_MOTION
-		 event.type != MotionNotify &&
+			case MotionNotify:
+				screen = event_screen(dsp, event.xmotion.root);
+				not_done = False;
+				break;
 #endif
-		 event.type != KeyPress);
-	for (screen = 0; screen < screens; screen++)
-		if (event.xbutton.root == RootWindow(dsp, screen))
-			break;
+
+			case KeyPress:
+				screen = event_screen(dsp, event.xkey.root);
+				not_done = False;
+				break;
+		}
+
+		if (!nolock && lockdelay &&
+        (lockdelay <= (seconds() - start_time))) {
+			timetodie = True;
+			not_done = False;
+		}
+	}
 
 	/* KLUDGE SO TVTWM AND VROOT WILL NOT MAKE XLOCK DIE */
 	if (screen >= screens)
 		screen = 0;
 
+	lockdelay = False;
 	if (usefirst)
 		XPutBackEvent(dsp, &event);
+	return timetodie;
 }
 
 
 static void
-sigcatch(void)
+sigcatch(int signum)
 {
+	ModeInfo   *mi = mode_info(0, win[0], False);
+	char       *name = (mi == NULL) ? "unknown" : MI_NAME(mi);
+	char        buf[512];
+
+	(void) sprintf(buf,
+    "Access control list restored.\n%%s: caught signal %d while running %s mode.\n",
+	  signum, name);
 	finish();
-	error("%s: caught terminate signal.\nAccess control list restored.\n");
+	error(buf);
 }
 
 
@@ -1283,14 +1408,14 @@ lockDisplay(void)
 #endif
 #endif
 
-		(void) signal(SIGHUP, (void (*)()) sigcatch);
-		(void) signal(SIGINT, (void (*)()) sigcatch);
-		(void) signal(SIGQUIT, (void (*)()) sigcatch);
-		(void) signal(SIGTERM, (void (*)()) sigcatch);
+		(void) signal(SIGHUP, (void (*)(int)) sigcatch);
+		(void) signal(SIGINT, (void (*)(int)) sigcatch);
+		(void) signal(SIGQUIT, (void (*)(int)) sigcatch);
+		(void) signal(SIGTERM, (void (*)(int)) sigcatch);
 		/* we should trap ALL signals, especially the deadly ones */
-		(void) signal(SIGSEGV, (void (*)()) sigcatch);
-		(void) signal(SIGBUS, (void (*)()) sigcatch);
-		(void) signal(SIGFPE, (void (*)()) sigcatch);
+		(void) signal(SIGSEGV, (void (*)(int)) sigcatch);
+		(void) signal(SIGBUS, (void (*)(int)) sigcatch);
+		(void) signal(SIGFPE, (void (*)(int)) sigcatch);
 
 		XGrabHosts(dsp);
 
@@ -1304,7 +1429,7 @@ lockDisplay(void)
 	XHPDisableReset(dsp);
 #endif
 	do {
-		justDisplay();
+		(void) justDisplay();
 	} while (getPassword());
 #if defined( __hpux ) || defined( __apollo )
 	XHPEnableReset(dsp);
@@ -1335,10 +1460,9 @@ main(int argc, char **argv)
 	start_time = time((long *) 0);
 	SRAND(start_time);	/* random mode needs the seed set. */
 
-	GetResources(argc, argv);
+	getResources(argc, argv);
 
-	CheckResources();
-	start_time += lockdelay;
+	checkResources();
 
 	initPasswd();
 
@@ -1358,14 +1482,34 @@ main(int argc, char **argv)
 		int         flags, x, y;
 		unsigned int w, h;
 
+		if (*icongeometry == '\0') {
+			iconwidth = DEF_ICONW;
+			iconheight = DEF_ICONH;
+		} else {
+			flags = XParseGeometry(icongeometry, &x, &y, &w, &h);
+			iconwidth = flags & WidthValue ? w : DEF_ICONW;
+			iconheight = flags & HeightValue ? h : DEF_ICONH;
+			if (iconwidth < MINICONW)
+				iconwidth = MINICONW;
+			else if (iconwidth > MAXICONW)
+				iconwidth = MAXICONW;
+			if (iconheight < MINICONH)
+				iconheight = MINICONH;
+			else if (iconheight > MAXICONH)
+				iconheight = MAXICONH;
+		}
 		if (*geometry == '\0') {
 			fullscreen = True;
 		} else {
 			flags = XParseGeometry(geometry, &x, &y, &w, &h);
+			if (w < MINICONW)
+				w = MINICONW;
+			if (h < MINICONH)
+				h = MINICONH;
 			minisizeconfigure.x = flags & XValue ? x : 0;
 			minisizeconfigure.y = flags & YValue ? y : 0;
-			minisizeconfigure.width = flags & WidthValue ? w : ICONW;
-			minisizeconfigure.height = flags & HeightValue ? h : ICONH;
+			minisizeconfigure.width = flags & WidthValue ? w : iconwidth;
+			minisizeconfigure.height = flags & HeightValue ? h : iconheight;
 		}
 	}
 	screens = ScreenCount(dsp);
@@ -1410,14 +1554,81 @@ main(int argc, char **argv)
 		Colormap    cmap = DefaultColormapOfScreen(scr);
 
 		root[screen] = RootWindowOfScreen(scr);
-		bgcol[screen] = allocpixel(cmap, background, "White");
-		fgcol[screen] = allocpixel(cmap, foreground, "Black");
-		rightcol[screen] = allocpixel(cmap, right3d, "Red");
-		leftcol[screen] = allocpixel(cmap, left3d, "Green");
+		fgcol[screen] = allocPixel(dsp, cmap, foreground, "Black");
+		bgcol[screen] = allocPixel(dsp, cmap, background, "White");
+		if (use3d) {
+			XColor      C;
+
+#ifdef CALCULATE_BOTH
+			XColor      C2;
+
+#endif
+			unsigned long planemasks[10];
+			unsigned long pixels[10];
+
+			if (!install || !XAllocColorCells(dsp, cmap, False, planemasks, 2, pixels,
+							  1)) {
+				/* did not get the needed colours.  Use normal 3d view without */
+				/* color overlapping */
+				nonecol[screen] = allocPixel(dsp, cmap, none3d, DEF_NONE3D);
+				rightcol[screen] = allocPixel(dsp, cmap, right3d, DEF_RIGHT3D);
+				leftcol[screen] = allocPixel(dsp, cmap, left3d, DEF_LEFT3D);
+				bothcol[screen] = allocPixel(dsp, cmap, both3d, DEF_BOTH3D);
+
+			} else {
+				/*
+				   * attention: the mixture of colours will only be guaranteed, if
+				   * the right black is used.  The problems with BlackPixel would
+				   * be that BlackPixel | leftcol need not be equal to leftcol.  The
+				   * same holds for rightcol (of course). That's why the right black
+				   * (black3dcol) must be used when GXor is used as put function.
+				   * I have allocated four colors above:
+				   * pixels[0],                                - 3d black
+				   * pixels[0] | planemasks[0],                - 3d right eye color
+				   * pixels[0] | planemasks[1],                - 3d left eye color
+				   * pixels[0] | planemasks[0] | planemasks[1] - 3d white
+				 */
+
+				if (!XParseColor(dsp, cmap, none3d, &C))
+					XParseColor(dsp, cmap, DEF_NONE3D, &C);
+				nonecol[screen] = C.pixel = pixels[0];
+				XStoreColor(dsp, cmap, &C);
+
+				if (!XParseColor(dsp, cmap, right3d, &C))
+					XParseColor(dsp, cmap, DEF_RIGHT3D, &C);
+				rightcol[screen] = C.pixel = pixels[0] | planemasks[0];
+				XStoreColor(dsp, cmap, &C);
+
+#ifdef CALCULATE_BOTH
+				C2.red = C.red;
+				C2.green = C.green;
+				C2.blue = C.blue;
+#else
+				if (!XParseColor(dsp, cmap, left3d, &C))
+					XParseColor(dsp, cmap, DEF_LEFT3D, &C);
+#endif
+				leftcol[screen] = C.pixel = pixels[0] | planemasks[1];
+				XStoreColor(dsp, cmap, &C);
+
+#ifdef CALCULATE_BOTH
+				C.red |= C2.red;	/* or them together... */
+				C.green |= C2.green;
+				C.blue |= C2.blue;
+#else
+				if (!XParseColor(dsp, cmap, both3d, &C))
+					XParseColor(dsp, cmap, DEF_BOTH3D, &C);
+#endif
+				bothcol[screen] = C.pixel = pixels[0] | planemasks[0] | planemasks[1];
+				XStoreColor(dsp, cmap, &C);
+			}
+
+		}
 		Scr[screen].bgcol = bgcol[screen];
 		Scr[screen].fgcol = fgcol[screen];
+		Scr[screen].nonecol = nonecol[screen];
 		Scr[screen].rightcol = rightcol[screen];
 		Scr[screen].leftcol = leftcol[screen];
+		Scr[screen].bothcol = bothcol[screen];
 		Scr[screen].cmap = None;
 
 		xswa.override_redirect = True;
@@ -1426,20 +1637,16 @@ main(int argc, char **argv)
 #ifdef MOUSE_MOTION
 			MotionNotify |
 #endif
-			VisibilityChangeMask;
-#ifdef DEBUG
-#define WIDTH WidthOfScreen(scr) - 100
-#define HEIGHT HeightOfScreen(scr) - 100
-#define CWMASK CWBackPixel | CWEventMask
-#else
-#define WIDTH (inwindow? WidthOfScreen(scr)/2 : WidthOfScreen(scr))
-#define HEIGHT (inwindow? HeightOfScreen(scr)/2 : HeightOfScreen(scr))
-#define CWMASK (((inwindow||inroot)? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
-#endif
+			VisibilityChangeMask | ExposureMask | StructureNotifyMask;
+#define WIDTH (inwindow? WidthOfScreen(scr)/2 : (debug? WidthOfScreen(scr) - 100 : WidthOfScreen(scr)))
+#define HEIGHT (inwindow? HeightOfScreen(scr)/2 : (debug? HeightOfScreen(scr) - 100 : HeightOfScreen(scr)))
+#define CWMASK (((debug||inwindow||inroot)? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
 #ifdef USE_VROOT
 		if (inroot) {
 			win[screen] = root[screen];
 			XChangeWindowAttributes(dsp, win[screen], CWBackPixel, &xswa);
+      /* this gives us these events from the root window */
+      XSelectInput(dsp, win[screen], VisibilityChangeMask | ExposureMask);
 		} else
 #endif
 #ifdef DT_SAVER
@@ -1466,8 +1673,7 @@ main(int argc, char **argv)
 							    CWMASK, &xswa);
 			}
 		}
-#ifdef DEBUG
-		{
+		if (debug || inwindow) {
 			XWMHints    xwmh;
 
 			xwmh.flags = InputHint;
@@ -1476,25 +1682,14 @@ main(int argc, char **argv)
 			       XA_WM_HINTS, XA_WM_HINTS, 32, PropModeReplace,
 					(unsigned char *) &xwmh, sizeof (xwmh) / sizeof (int));
 		}
-#else
-		if (inwindow) {
-			XWMHints    xwmh;
-
-			xwmh.flags = InputHint;
-			xwmh.input = True;
-			XChangeProperty(dsp, win[screen],
-			       XA_WM_HINTS, XA_WM_HINTS, 32, PropModeReplace,
-					(unsigned char *) &xwmh, sizeof (xwmh) / sizeof (int));
-		}
-#endif
 
 		if (mono || CellsOfScreen(scr) <= 2) {
 			Scr[screen].pixels[0] = fgcol[screen];
 			Scr[screen].pixels[1] = bgcol[screen];
 			Scr[screen].npixels = 2;
 		} else
-			fix_colormap(win[screen], screen, saturation,
-				     install, inroot, inwindow, verbose);
+			fixColormap(dsp, win[screen], screen, saturation,
+				    install, inroot, inwindow, verbose);
 
 		iconx[screen] = (DisplayWidth(dsp, screen) -
 			 XTextWidth(font, text_info, strlen(text_info))) / 2;
@@ -1510,7 +1705,7 @@ main(int argc, char **argv)
 		else
 			icon[screen] = XCreateWindow(dsp, win[screen],
 						iconx[screen], icony[screen],
-					     ICONW, ICONH, 1, CopyFromParent,
+				    iconwidth, iconheight, 1, CopyFromParent,
 						 InputOutput, CopyFromParent,
 						     CIMASK, &xswa);
 
@@ -1533,7 +1728,7 @@ main(int argc, char **argv)
 
 #if 0
 		if (install && cmap != DefaultColormapOfScreen(scr))
-			set_colormap(win[screen], cmap, inwindow);
+			setColormap(dsp, win[screen], cmap, inwindow);
 #endif
 
 		xgcv.font = font->fid;
@@ -1560,22 +1755,18 @@ main(int argc, char **argv)
 		XResetScreenSaver(dsp);		/* make sure not blank now */
 		XSetScreenSaver(dsp, 0, 0, 0, 0);	/* disable screen saver */
 	}
-#ifndef DEBUG
-	if (grabmouse && !inwindow && !inroot) {
-		GrabKeyboardAndMouse();
-	} else
+	if (!grabmouse || inwindow || inroot)
 		nolock = 1;
-#endif
-
+  else if (!debug)
+		GrabKeyboardAndMouse();
 	(void) nice(nicelevel);
 
 	(void) XSetIOErrorHandler(xio_error);
 
-	if (nolock)
-		justDisplay();
-	else if (lockdelay) {
-		justDisplay();
-		if (start_time < seconds())
+	if (nolock) {
+		(void) justDisplay();
+	} else if (lockdelay) {
+		if (justDisplay())
 			lockDisplay();
 		else
 			nolock = 1;
