@@ -2,7 +2,7 @@
 #define __XLOCK_XLOCK_H__
 
 /*-
- * @(#)xlock.h	3.9 96/05/25 xlockmore 
+ * @(#)xlock.h	3.11 96/09/20 xlockmore 
  *
  * xlock.h - external interfaces for new modes and SYSV OS defines.
  *
@@ -50,11 +50,30 @@
 #define DEF_PROGRAM "fortune -s"
 #endif
 
-#define ICONW             64
-#define ICONH             64
+#define DEF_ICONW             64	/* Age old default */
+#define DEF_ICONH             64
 
-#ifndef min
-#define min(a,b) ((a)<(b)?(a):(b))
+#define DEF_NONE3D "Black"
+#define DEF_RIGHT3D "Red"
+#define DEF_LEFT3D "Blue"
+#define DEF_BOTH3D "Magenta"
+
+#define MINICONW             1	/* Too many modes die when its just 1 */
+#define MINICONH             1
+
+#define MAXICONW             256	/* Want users to know the screen is locked */
+#define MAXICONH             256	/* by a particular user */
+
+#ifndef MIN
+#define MIN(a,b) (((a)<(b))?(a):(b))
+#endif
+
+#ifndef MAX
+#define MAX(a,b) (((a)>(b))?(a):(b))
+#endif
+
+#ifndef ABS
+#define ABS(a)  ((a<0)?(-(a)):(a))
 #endif
 
 #if defined( VMS ) || defined( __QNX__ )
@@ -103,9 +122,9 @@ typedef struct {
 	unsigned long pixels[NUMCOLORS];	/* pixel values in the colormap */
 	unsigned long bgcol, fgcol;	/* background and foreground pixel values */
 	unsigned long rightcol, leftcol;	/* 3D color pixel values */
+	unsigned long nonecol, bothcol;
 } perscreen;
 
-/* This stuff moved here from resource.c for the mode-specific options. */
 #define t_String        0
 #define t_Float         1
 #define t_Int           2
@@ -114,7 +133,7 @@ typedef struct {
 typedef struct {
 	caddr_t    *var;
 	char       *name;
-	char       *class;
+	char       *classname;
 	char       *def;
 	int         type;
 } argtype;
@@ -127,36 +146,40 @@ typedef struct {
 typedef struct {
 	int         numopts;
 	XrmOptionDescRec *opts;
+	int         numvarsdesc;
 	argtype    *vars;
 	OptionStruct *desc;
 } ModeSpecOpt;
 
-/* End moved from resource.c */
-
 /* this must follow definition of ModeSpecOpt */
 #include "mode.h"
 
-extern perscreen Scr[MAXSCREENS];
-extern Display *dsp;
-extern int  screen;
+#define IS_XBMDONE 1		/* Only need one mono image */
+#define IS_XBM 2
+#define IS_XBMFILE 3
+#define IS_XPM 4
+#define IS_XPMFILE 5
+#define IS_RASTERFILE 6
 
-extern void GetResources(int argc, char **argv);
-extern unsigned long allocpixel(Colormap cmap, char *name, char *def);
-extern void set_colormap(Window window, Colormap map, Bool inwindow);
-extern void fix_colormap(Window window, int screen, float saturation,
+extern void getResources(int argc, char **argv);
+extern unsigned long allocPixel(Display * display, Colormap cmap,
+				char *name, char *def);
+extern void setColormap(Display * display, Window window, Colormap map,
+			Bool inwindow);
+extern void fixColormap(Display * display, Window window,
+			int screen, float saturation,
 		     Bool install, Bool inroot, Bool inwindow, Bool verbose);
 extern long seconds(void);
 
-#if defined (__STDC__) || defined(VMS) || defined( AIXV3 )
-extern void error(char *s1,...);
-extern void warning(char *s1,...);
-#else
+#ifdef NONSTDC			/* This will go away if not being used */
 extern void error(char *s1, char *s2);
 extern void warning(char *s1, char *s2, char *s3);
-#endif
 
-extern int  readable(char *);
-extern void randomImage(char *im_file);
+#else
+extern void error(char *s1,...);
+extern void warning(char *s1,...);
+
+#endif
 
 #if defined( ultrix ) || (defined( VMS ) && ( __VMS_VER < 70000000 ))
 extern char *strdup(char *);
